@@ -20,8 +20,9 @@ from io import (
     )
 import re
 
+import fastbencode as bencode
+
 from .... import (
-    bencode,
     errors,
     iterablefile,
     lru_cache,
@@ -45,7 +46,7 @@ class _MPDiffInventoryGenerator(_mod_versionedfile._MPDiffGenerator):
     """Generate Inventory diffs serialized inventories."""
 
     def __init__(self, repo, inventory_keys):
-        super(_MPDiffInventoryGenerator, self).__init__(repo.inventories,
+        super().__init__(repo.inventories,
                                                         inventory_keys)
         self.repo = repo
         self.sha1s = {}
@@ -78,7 +79,7 @@ class _MPDiffInventoryGenerator(_mod_versionedfile._MPDiffGenerator):
             yield revision_id, parent_ids, sha1, diff
 
 
-class BundleWriter(object):
+class BundleWriter:
     """Writer for bundle-format files.
 
     This serves roughly the same purpose as ContainerReader, but acts as a
@@ -183,7 +184,7 @@ class BundleWriter(object):
             self._container.add_bytes_record([bytes], len(bytes), [])
 
 
-class BundleReader(object):
+class BundleReader:
     """Reader for bundle-format files.
 
     This serves roughly the same purpose as ContainerReader, but acts as a
@@ -288,7 +289,7 @@ class BundleSerializerV4(bundle_serializer.BundleSerializer):
         return serializer.format_registry.get(info[b'serializer'].decode('ascii'))
 
 
-class BundleWriteOperation(object):
+class BundleWriteOperation:
     """Perform the operation of writing revisions to a bundle"""
 
     def __init__(self, base, target, repository, fileobj, revision_ids=None):
@@ -429,7 +430,7 @@ class BundleWriteOperation(object):
                                                item_key[-1], file_id)
 
 
-class BundleInfoV4(object):
+class BundleInfoV4:
 
     """Provide (most of) the BundleInfo interface"""
 
@@ -503,7 +504,7 @@ class BundleInfoV4(object):
     target = property(_get_target)
 
 
-class RevisionInstaller(object):
+class RevisionInstaller:
     """Installs revisions into a repository"""
 
     def __init__(self, container, serializer, repository):
