@@ -25,20 +25,18 @@ methods. To free any associated resources, simply stop referencing the
 objects returned.
 """
 
+import contextlib
 import sys
+from typing import Set, TYPE_CHECKING, cast
 
 from ..lazy_import import lazy_import
 lazy_import(globals(), """
-import contextlib
-
 from breezy import (
     branch as _mod_branch,
     lockable_files,
-    lockdir,
     osutils,
     repository,
     revision as _mod_revision,
-    transport as _mod_transport,
     ui,
     urlutils,
     win32utils,
@@ -46,17 +44,13 @@ from breezy import (
 from breezy.bzr import (
     branch as _mod_bzrbranch,
     fetch,
+    fullhistory as fullhistorybranch,
+    knitpack_repo,
     remote,
     vf_search,
     workingtree_3,
     workingtree_4,
     )
-from breezy.bzr import fullhistory as fullhistorybranch
-from breezy.bzr import knitpack_repo
-from breezy.transport import (
-    do_catching_redirections,
-    local,
-    )
 from breezy.i18n import gettext
 """)
 
@@ -70,13 +64,22 @@ from .. import (
     config,
     controldir,
     errors,
+    lockdir,
+    transport as _mod_transport,
     )
+from ..transport import (
+    do_catching_redirections,
+    local,
+    )
+
+if TYPE_CHECKING:
+    from .branch import BzrBranch
 
 
 class MissingFeature(errors.BzrError):
 
     _fmt = ("Missing feature %(feature)s not provided by this "
-            "version of Bazaar or any plugin.")
+            "version of Breezy or any plugin.")
 
     def __init__(self, feature):
         self.feature = feature
@@ -436,7 +439,7 @@ class BzrDir(controldir.ControlDir):
                 result_branch = source_branch.sprout(
                     result, revision_id=revision_id,
                     repository_policy=repository_policy, repository=result_repo)
-            mutter("created new branch %r" % (result_branch,))
+            mutter("created new branch {!r}".format(result_branch))
 
             # Create/update the result working tree
             if (create_tree_if_local and not result.has_workingtree()
@@ -529,7 +532,7 @@ class BzrDir(controldir.ControlDir):
                 note(gettext("renamed {0} to {1}").format(
                     self.root_transport.abspath('.bzr'), to_path))
                 return
-            except (errors.TransportError, IOError, errors.PathError):
+            except (errors.TransportError, OSError, errors.PathError):
                 i += 1
                 if i > limit:
                     raise
@@ -769,7 +772,7 @@ class BzrDir(controldir.ControlDir):
         raise NotImplementedError(self.get_workingtree_transport)
 
     @classmethod
-    def create(cls, base, format=None, possible_transports=None):
+    def create(cls, base, format=None, possible_transports=None) -> "BzrDir":
         """Create a new BzrDir at the url 'base'.
 
         :param format: If supplied, the format of branch to create.  If not
@@ -780,11 +783,13 @@ class BzrDir(controldir.ControlDir):
         if cls is not BzrDir:
             raise AssertionError("BzrDir.create always creates the "
                                  "default format, not one of %r" % cls)
-        return controldir.ControlDir.create(
-            base, format=format, possible_transports=possible_transports)
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        return cast("BzrDir", controldir.ControlDir.create(
+            base, format=format, possible_transports=possible_transports))
 
     def __repr__(self):
-        return "<%s at %r>" % (self.__class__.__name__, self.user_url)
+        return "<{} at {!r}>".format(self.__class__.__name__, self.user_url)
 
     def update_feature_flags(self, updated_flags):
         """Update the features required by this bzrdir.
@@ -830,7 +835,7 @@ class BzrDirMeta1(BzrDir):
         """
         try:
             f = self.control_transport.get('branch-list')
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return []
 
         ret = []
@@ -851,7 +856,7 @@ class BzrDirMeta1(BzrDir):
             b"".join([name.encode('utf-8') + b"\n" for name in branches]))
 
     def __init__(self, _transport, _format):
-        super(BzrDirMeta1, self).__init__(_transport, _format)
+        super().__init__(_transport, _format)
         self.control_files = lockable_files.LockableFiles(
             self.control_transport, self._format._lock_file_name,
             self._format._lock_class)
@@ -874,7 +879,7 @@ class BzrDirMeta1(BzrDir):
         if name is None:
             name = self._get_selected_branch()
         path = self._get_branch_path(name)
-        if name != u"":
+        if name != "":
             self.control_files.lock_write()
             try:
                 branches = self._read_branch_list()
@@ -887,7 +892,7 @@ class BzrDirMeta1(BzrDir):
                 self.control_files.unlock()
         try:
             self.transport.delete_tree(path)
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             raise errors.NotBranchError(
                 path=urlutils.join(self.transport.base, path), controldir=self)
 
@@ -899,7 +904,7 @@ class BzrDirMeta1(BzrDir):
         """See BzrDir.destroy_repository."""
         try:
             self.transport.delete_tree('repository')
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             raise errors.NoRepositoryPresent(self)
 
     def create_workingtree(self, revision_id=None, from_branch=None,
@@ -970,10 +975,10 @@ class BzrDirMeta1(BzrDir):
                 try:
                     branches = self._read_branch_list()
                     dirname = urlutils.dirname(name)
-                    if dirname != u"" and dirname in branches:
+                    if dirname != "" and dirname in branches:
                         raise errors.ParentBranchExists(name)
                     child_branches = [
-                        b.startswith(name + u"/") for b in branches]
+                        b.startswith(name + "/") for b in branches]
                     if any(child_branches):
                         raise errors.AlreadyBranchError(name)
                     branches.append(name)
@@ -985,7 +990,7 @@ class BzrDirMeta1(BzrDir):
         branch_transport.create_prefix(mode=mode)
         try:
             self.transport.mkdir(path, mode=mode)
-        except errors.FileExists:
+        except _mod_transport.FileExists:
             pass
         return self.transport.clone(path)
 
@@ -999,7 +1004,7 @@ class BzrDirMeta1(BzrDir):
             raise errors.IncompatibleFormat(repository_format, self._format)
         try:
             self.transport.mkdir('repository', mode=self._get_mkdir_mode())
-        except errors.FileExists:
+        except _mod_transport.FileExists:
             pass
         return self.transport.clone('repository')
 
@@ -1013,7 +1018,7 @@ class BzrDirMeta1(BzrDir):
             raise errors.IncompatibleFormat(workingtree_format, self._format)
         try:
             self.transport.mkdir('checkout', mode=self._get_mkdir_mode())
-        except errors.FileExists:
+        except _mod_transport.FileExists:
             pass
         return self.transport.clone('checkout')
 
@@ -1120,7 +1125,7 @@ class BzrDirMeta1(BzrDir):
         return config.TransportConfig(self.transport, 'control.conf')
 
 
-class BzrFormat(object):
+class BzrFormat:
     """Base class for all formats of things living in metadirs.
 
     This class manages the format string that is stored in the 'format'
@@ -1142,7 +1147,7 @@ class BzrFormat(object):
     :ivar features: Dictionary mapping feature names to their necessity
     """
 
-    _present_features = set()
+    _present_features: Set[str] = set()
 
     def __init__(self):
         self.features = {}
@@ -1189,7 +1194,7 @@ class BzrFormat(object):
         format_string = cls.get_format_string()
         if not text.startswith(format_string):
             raise AssertionError(
-                "Invalid format header %r for %r" % (text, cls))
+                "Invalid format header {!r} for {!r}".format(text, cls))
         lines = text[len(format_string):].splitlines()
         ret = cls()
         for lineno, line in enumerate(lines):
@@ -1348,10 +1353,10 @@ class BzrDirFormat(BzrFormat, controldir.ControlDirFormat):
         try:
             transport = do_catching_redirections(make_directory, transport,
                                                  redirected)
-        except errors.FileExists:
+        except _mod_transport.FileExists:
             if not use_existing_dir:
                 raise
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             if not create_prefix:
                 raise
             transport.create_prefix()
@@ -1400,7 +1405,7 @@ class BzrDirFormat(BzrFormat, controldir.ControlDirFormat):
                                           # FIXME: RBC 20060121 don't peek under
                                           # the covers
                                           mode=temp_control._dir_mode)
-        except errors.FileExists:
+        except _mod_transport.FileExists:
             raise errors.AlreadyControlDirError(transport.base)
         if sys.platform == 'win32' and isinstance(transport, local.LocalTransport):
             win32utils.set_file_attr_hidden(transport._abspath('.bzr'))
@@ -1699,7 +1704,7 @@ class BzrDirMetaFormat1(BzrDirFormat):
             compatible with whatever sub formats are supported by self.
         :return: None.
         """
-        super(BzrDirMetaFormat1, self)._supply_sub_formats_to(other_format)
+        super()._supply_sub_formats_to(other_format)
         if getattr(self, '_repository_format', None) is not None:
             other_format.repository_format = self.repository_format
         if self._branch_format is not None:
@@ -1719,7 +1724,7 @@ class BzrDirMetaFormat1(BzrDirFormat):
         self._workingtree_format = wt_format
 
     def __repr__(self):
-        return "<%r>" % (self.__class__.__name__,)
+        return "<{!r}>".format(self.__class__.__name__)
 
     workingtree_format = property(__get_workingtree_format,
                                   __set_workingtree_format)
@@ -1848,24 +1853,6 @@ class ConvertMetaToColo(controldir.Converter):
         return BzrDir.open_from_transport(to_convert.root_transport)
 
 
-class ConvertMetaToColo(controldir.Converter):
-    """Convert a 'development-colo' bzrdir to a '2a' bzrdir."""
-
-    def __init__(self, target_format):
-        """Create a converter that converts a 'development-colo' metadir
-        to a '2a' metadir.
-
-        :param target_format: The final metadir format that is desired.
-        """
-        self.target_format = target_format
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        to_convert.transport.put_bytes('branch-format',
-                                       self.target_format.as_string())
-        return BzrDir.open_from_transport(to_convert.root_transport)
-
-
 class CreateRepository(controldir.RepositoryAcquisitionPolicy):
     """A policy of creating a new repository"""
 
@@ -1878,7 +1865,7 @@ class CreateRepository(controldir.RepositoryAcquisitionPolicy):
         :param stack_on_pwd: If stack_on is relative, the location it is
             relative to.
         """
-        super(CreateRepository, self).__init__(
+        super().__init__(
             stack_on, stack_on_pwd, require_stacking)
         self._controldir = controldir
 
@@ -1922,7 +1909,7 @@ class UseExistingRepository(controldir.RepositoryAcquisitionPolicy):
         :param stack_on_pwd: If stack_on is relative, the location it is
             relative to.
         """
-        super(UseExistingRepository, self).__init__(
+        super().__init__(
             stack_on, stack_on_pwd, require_stacking)
         self._repository = repository
 
