@@ -14,17 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
-
 import re
 import sys
+from typing import Type
 
 from ..lazy_import import lazy_import
 lazy_import(globals(), """
+import contextlib
 import time
 
 from breezy import (
-    cleanup,
     config,
     debug,
     graph,
@@ -43,8 +42,10 @@ from .. import (
     errors,
     lockable_files,
     lockdir,
+    transport as _mod_transport,
     )
 from ..bzr import (
+    index,
     btree_index,
     )
 
@@ -56,15 +57,12 @@ from ..repository import (
     _LazyListJoin,
     RepositoryWriteLockResult,
     )
-from ..bzr.repository import (
+from .repository import (
     MetaDirRepository,
     RepositoryFormatMetaDir,
     )
-from ..sixish import (
-    reraise,
-    viewitems,
-    )
-from ..bzr.vf_repository import (
+from .serializer import Serializer
+from .vf_repository import (
     MetaDirVersionedFileRepository,
     MetaDirVersionedFileRepositoryFormat,
     VersionedFileCommitBuilder,
@@ -76,6 +74,53 @@ from ..trace import (
     )
 
 
+class RetryWithNewPacks(errors.BzrError):
+    """Raised when we realize that the packs on disk have changed.
+
+    This is meant as more of a signaling exception, to trap between where a
+    local error occurred and the code that can actually handle the error and
+    code that can retry appropriately.
+    """
+
+    internal_error = True
+
+    _fmt = ("Pack files have changed, reload and retry. context: %(context)s"
+            " %(orig_error)s")
+
+    def __init__(self, context, reload_occurred, exc_info):
+        """create a new RetryWithNewPacks error.
+
+        :param reload_occurred: Set to True if we know that the packs have
+            already been reloaded, and we are failing because of an in-memory
+            cache miss. If set to True then we will ignore if a reload says
+            nothing has changed, because we assume it has already reloaded. If
+            False, then a reload with nothing changed will force an error.
+        :param exc_info: The original exception traceback, so if there is a
+            problem we can raise the original error (value from sys.exc_info())
+        """
+        errors.BzrError.__init__(self)
+        self.context = context
+        self.reload_occurred = reload_occurred
+        self.exc_info = exc_info
+        self.orig_error = exc_info[1]
+        # TODO: The global error handler should probably treat this by
+        #       raising/printing the original exception with a bit about
+        #       RetryWithNewPacks also not being caught
+
+
+class RetryAutopack(RetryWithNewPacks):
+    """Raised when we are autopacking and we find a missing file.
+
+    Meant as a signaling exception, to tell the autopack code it should try
+    again.
+    """
+
+    internal_error = True
+
+    _fmt = ("Pack files have changed, reload and try autopack again."
+            " context: %(context)s %(orig_error)s")
+
+
 class PackCommitBuilder(VersionedFileCommitBuilder):
     """Subclass of VersionedFileCommitBuilder to add texts with pack semantics.
 
@@ -85,10 +130,11 @@ class PackCommitBuilder(VersionedFileCommitBuilder):
 
     def __init__(self, repository, parents, config, timestamp=None,
                  timezone=None, committer=None, revprops=None,
-                 revision_id=None, lossy=False):
+                 revision_id=None, lossy=False, owns_transaction=True):
         VersionedFileCommitBuilder.__init__(self, repository, parents, config,
                                             timestamp=timestamp, timezone=timezone, committer=committer,
-                                            revprops=revprops, revision_id=revision_id, lossy=lossy)
+                                            revprops=revprops, revision_id=revision_id, lossy=lossy,
+                                            owns_transaction=owns_transaction)
         self._file_graph = graph.Graph(
             repository._pack_collection.text_index.combined_index)
 
@@ -97,7 +143,7 @@ class PackCommitBuilder(VersionedFileCommitBuilder):
         return {key[1] for key in self._file_graph.heads(keys)}
 
 
-class Pack(object):
+class Pack:
     """An in memory proxy for a pack and its indices.
 
     This is a base class that is not directly used, instead the classes
@@ -249,7 +295,7 @@ class ExistingPack(Pack):
         return not self.__eq__(other)
 
     def __repr__(self):
-        return "<%s.%s object at 0x%x, %s, %s" % (
+        return "<{}.{} object at 0x{:x}, {}, {}".format(
             self.__class__.__module__, self.__class__.__name__, id(self),
             self.pack_transport, self.name)
 
@@ -557,7 +603,7 @@ class NewPack(Pack):
         self._replace_index_with_readonly(index_type)
 
 
-class AggregateIndex(object):
+class AggregateIndex:
     """An aggregated index for the RepositoryPackCollection.
 
     AggregateIndex is reponsible for managing the PackAccess object,
@@ -643,7 +689,7 @@ class AggregateIndex(object):
             self.data_access.set_writer(None, None, (None, None))
 
 
-class Packer(object):
+class Packer:
     """Create a pack from packs."""
 
     def __init__(self, pack_collection, packs, suffix, revision_ids=None,
@@ -762,16 +808,16 @@ class Packer(object):
         return new_pack.data_inserted()
 
 
-class RepositoryPackCollection(object):
+class RepositoryPackCollection:
     """Management of packs within a repository.
 
     :ivar _names: map of {pack_name: (index_size,)}
     """
 
-    pack_factory = None
-    resumed_pack_factory = None
-    normal_packer_class = None
-    optimising_packer_class = None
+    pack_factory: Type[NewPack]
+    resumed_pack_factory: Type[ResumedPack]
+    normal_packer_class: Type[Packer]
+    optimising_packer_class: Type[Packer]
 
     def __init__(self, repo, transport, index_transport, upload_transport,
                  pack_transport, index_builder_class, index_class,
@@ -831,7 +877,7 @@ class RepositoryPackCollection(object):
         self.config_stack = config.LocationStack(self.transport.base)
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.repo)
+        return '{}({!r})'.format(self.__class__.__name__, self.repo)
 
     def add_pack_to_memory(self, pack):
         """Make a Pack object available to the repository to satisfy queries.
@@ -840,7 +886,7 @@ class RepositoryPackCollection(object):
         """
         if pack.name in self._packs_by_name:
             raise AssertionError(
-                'pack %s already in _packs_by_name' % (pack.name,))
+                'pack {} already in _packs_by_name'.format(pack.name))
         self.packs.append(pack)
         self._packs_by_name[pack.name] = pack
         self.revision_index.add_index(pack.revision_index, pack)
@@ -881,7 +927,7 @@ class RepositoryPackCollection(object):
         while True:
             try:
                 return self._do_autopack()
-            except errors.RetryAutopack:
+            except RetryAutopack:
                 # If we get a RetryAutopack exception, we should abort the
                 # current action, and retry.
                 pass
@@ -941,7 +987,7 @@ class RepositoryPackCollection(object):
                                   reload_func=reload_func)
             try:
                 result = packer.pack()
-            except errors.RetryWithNewPacks:
+            except RetryWithNewPacks:
                 # An exception is propagating out of this context, make sure
                 # this packer has cleaned up. Packer() doesn't set its new_pack
                 # state into the RepositoryPackCollection object, so we only
@@ -1142,7 +1188,7 @@ class RepositoryPackCollection(object):
                                                txt_index, sig_index, self._upload_transport,
                                                self._pack_transport, self._index_transport, self,
                                                chk_index=chk_index)
-        except errors.NoSuchFile as e:
+        except _mod_transport.NoSuchFile as e:
             raise errors.UnresumableWriteGroup(self.repo, [name], str(e))
         self.add_pack_to_memory(result)
         self._resumed_packs.append(result)
@@ -1157,7 +1203,7 @@ class RepositoryPackCollection(object):
         self.ensure_loaded()
         if a_new_pack.name in self._names:
             raise errors.BzrError(
-                'Pack %r already exists in %s' % (a_new_pack.name, self))
+                'Pack {!r} already exists in {}'.format(a_new_pack.name, self))
         self._names[a_new_pack.name] = tuple(a_new_pack.index_sizes)
         self.add_pack_to_memory(a_new_pack)
 
@@ -1222,12 +1268,12 @@ class RepositoryPackCollection(object):
                 try:
                     pack.pack_transport.move(pack.file_name(),
                                              '../obsolete_packs/' + pack.file_name())
-                except errors.NoSuchFile:
+                except _mod_transport.NoSuchFile:
                     # perhaps obsolete_packs was removed? Let's create it and
                     # try again
                     try:
                         pack.pack_transport.mkdir('../obsolete_packs/')
-                    except errors.FileExists:
+                    except _mod_transport.FileExists:
                         pass
                     pack.pack_transport.move(pack.file_name(),
                                              '../obsolete_packs/' + pack.file_name())
@@ -1337,7 +1383,7 @@ class RepositoryPackCollection(object):
 
         # do a two-way diff against our original content
         current_nodes = set()
-        for name, sizes in viewitems(self._names):
+        for name, sizes in self._names.items():
             current_nodes.add(
                 (name, b' '.join(b'%d' % size for size in sizes)))
 
@@ -1481,7 +1527,7 @@ class RepositoryPackCollection(object):
             # Re-raise the original exception, because something went missing
             # and a restart didn't find it
             raise
-        raise errors.RetryAutopack(self.repo, False, sys.exc_info())
+        raise RetryAutopack(self.repo, False, sys.exc_info())
 
     def _restart_pack_operations(self):
         """Reload the pack names list, and restart the autopack code."""
@@ -1503,7 +1549,7 @@ class RepositoryPackCollection(object):
             preserve = set()
         try:
             obsolete_pack_files = obsolete_pack_transport.list_dir('.')
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return found
         for filename in obsolete_pack_files:
             name, ext = osutils.splitext(filename)
@@ -1550,7 +1596,7 @@ class RepositoryPackCollection(object):
         # FIXME: just drop the transient index.
         # forget what names there are
         if self._new_pack is not None:
-            with cleanup.ExitStack() as stack:
+            with contextlib.ExitStack() as stack:
                 stack.callback(setattr, self, '_new_pack', None)
                 # If we aborted while in the middle of finishing the write
                 # group, _remove_pack_indices could fail because the indexes are
@@ -1560,7 +1606,7 @@ class RepositoryPackCollection(object):
                                ignore_missing=True)
                 self._new_pack.abort()
         for resumed_pack in self._resumed_packs:
-            with cleanup.ExitStack() as stack:
+            with contextlib.ExitStack() as stack:
                 # See comment in previous finally block.
                 stack.callback(self._remove_pack_indices, resumed_pack,
                                ignore_missing=True)
@@ -1678,8 +1724,8 @@ class PackRepository(MetaDirVersionedFileRepository):
     # them to None ensures that if the constructor is changed to not initialize
     # them, or a subclass fails to call the constructor, that an error will
     # occur rather than the system working but generating incorrect data.
-    _commit_builder_class = None
-    _serializer = None
+    _commit_builder_class: Type[VersionedFileCommitBuilder]
+    _serializer: Serializer
 
     def __init__(self, _format, a_controldir, control_files, _commit_builder_class,
                  _serializer):
@@ -1869,27 +1915,27 @@ class RepositoryFormatPack(MetaDirVersionedFileRepositoryFormat):
 
     # Set this attribute in derived classes to control the repository class
     # created by open and initialize.
-    repository_class = None
+    repository_class: Type[PackRepository]
     # Set this attribute in derived classes to control the
     # _commit_builder_class that the repository objects will have passed to
     # their constructor.
-    _commit_builder_class = None
+    _commit_builder_class: Type[VersionedFileCommitBuilder]
     # Set this attribute in derived clases to control the _serializer that the
     # repository objects will have passed to their constructor.
-    _serializer = None
+    _serializer: Serializer
     # Packs are not confused by ghosts.
-    supports_ghosts = True
+    supports_ghosts: bool = True
     # External references are not supported in pack repositories yet.
-    supports_external_lookups = False
+    supports_external_lookups: bool = False
     # Most pack formats do not use chk lookups.
-    supports_chks = False
+    supports_chks: bool = False
     # What index classes to use
-    index_builder_class = None
-    index_class = None
-    _fetch_uses_deltas = True
-    fast_deltas = False
-    supports_funky_characters = True
-    revision_graph_can_have_wrong_parents = True
+    index_builder_class: Type[index.GraphIndexBuilder]
+    index_class: Type[object]
+    _fetch_uses_deltas: bool = True
+    fast_deltas: bool = False
+    supports_funky_characters: bool = True
+    revision_graph_can_have_wrong_parents: bool = True
 
     def initialize(self, a_controldir, shared=False):
         """Create a pack based repository.
@@ -1933,7 +1979,7 @@ class RepositoryFormatPack(MetaDirVersionedFileRepositoryFormat):
                                      _serializer=self._serializer)
 
 
-class RetryPackOperations(errors.RetryWithNewPacks):
+class RetryPackOperations(RetryWithNewPacks):
     """Raised when we are packing and we find a missing file.
 
     Meant as a signaling exception, to tell the RepositoryPackCollection.pack
@@ -1946,7 +1992,7 @@ class RetryPackOperations(errors.RetryWithNewPacks):
             " context: %(context)s %(orig_error)s")
 
 
-class _DirectPackAccess(object):
+class _DirectPackAccess:
     """Access to data in one or more packs with less translation."""
 
     def __init__(self, index_to_packs, reload_func=None, flush_func=None):
@@ -2049,21 +2095,21 @@ class _DirectPackAccess(object):
                     # If we don't have a _reload_func there is nothing that can
                     # be done
                     raise
-                raise errors.RetryWithNewPacks(index,
-                                               reload_occurred=True,
-                                               exc_info=sys.exc_info())
+                raise RetryWithNewPacks(index,
+                                        reload_occurred=True,
+                                        exc_info=sys.exc_info())
             try:
                 reader = pack.make_readv_reader(transport, path, offsets)
                 for names, read_func in reader.iter_records():
                     yield read_func(None)
-            except errors.NoSuchFile:
+            except _mod_transport.NoSuchFile:
                 # A NoSuchFile error indicates that a pack file has gone
                 # missing on disk, we need to trigger a reload, and start over.
                 if self._reload_func is None:
                     raise
-                raise errors.RetryWithNewPacks(transport.abspath(path),
-                                               reload_occurred=False,
-                                               exc_info=sys.exc_info())
+                raise RetryWithNewPacks(transport.abspath(path),
+                                        reload_occurred=False,
+                                        exc_info=sys.exc_info())
 
     def set_writer(self, writer, index, transport_packname):
         """Set a writer to use for adding data."""
@@ -2099,4 +2145,4 @@ class _DirectPackAccess(object):
                 is_error = True
         if is_error:
             # GZ 2017-03-27: No real reason this needs the original traceback.
-            reraise(*retry_exc.exc_info)
+            raise retry_exc.exc_info[1]
