@@ -16,8 +16,6 @@
 
 """Reconcilers are able to fix some potential data errors in a branch."""
 
-from __future__ import absolute_import
-
 __all__ = [
     'BranchReconciler',
     'KnitReconciler',
@@ -26,7 +24,6 @@ __all__ = [
     ]
 
 from .. import (
-    cleanup,
     errors,
     revision as _mod_revision,
     ui,
@@ -35,10 +32,10 @@ from ..reconcile import ReconcileResult
 from ..i18n import gettext
 from ..trace import mutter
 from ..tsort import topo_sort
-from .versionedfile import AdapterFactory, FulltextContentFactory
+from .versionedfile import AdapterFactory, ChunkedContentFactory
 
 
-class VersionedFileRepoReconciler(object):
+class VersionedFileRepoReconciler:
     """Reconciler that reconciles a repository.
 
     The goal of repository reconciliation is to make any derived data
@@ -152,8 +149,8 @@ class VersionedFileRepoReconciler(object):
                 # The check for the left most parent only handles knit
                 # compressors, but this code only applies to knit and weave
                 # repositories anyway.
-                bytes = record.get_bytes_as('fulltext')
-                yield FulltextContentFactory(record.key, wanted_parents, record.sha1, bytes)
+                chunks = record.get_bytes_as('chunked')
+                yield ChunkedContentFactory(record.key, wanted_parents, record.sha1, chunks)
             else:
                 adapted_record = AdapterFactory(
                     record.key, wanted_parents, record)
@@ -264,7 +261,7 @@ class KnitReconciler(VersionedFileRepoReconciler):
         new_inventories.insert_record_stream(stream)
         # if this worked, the set of new_inventory_vf.names should equal
         # the revisionds list
-        if not(set(new_inventories.keys()) == set(revision_keys)):
+        if set(new_inventories.keys()) != set(revision_keys):
             raise AssertionError()
         self.pb.update(gettext('Writing weave'))
         self.repo._activate_new_inventory()
@@ -310,8 +307,8 @@ class KnitReconciler(VersionedFileRepoReconciler):
             self.pb.update(gettext('Fixing text parents'), num,
                            len(per_id_bad_parents))
             versions_with_bad_parents = per_id_bad_parents[file_id]
-            id_unused_versions = set(key[-1] for key in unused_versions
-                                     if key[0] == file_id)
+            id_unused_versions = {key[-1] for key in unused_versions
+                                     if key[0] == file_id}
             if file_id in file_id_versions:
                 file_versions = file_id_versions[file_id]
             else:
@@ -344,10 +341,10 @@ class KnitReconciler(VersionedFileRepoReconciler):
 
         def fix_parents(stream):
             for record in stream:
-                bytes = record.get_bytes_as('fulltext')
+                chunks = record.get_bytes_as('chunked')
                 new_key = (new_file_id, record.key[-1])
                 parents = new_parents[new_key]
-                yield FulltextContentFactory(new_key, parents, record.sha1, bytes)
+                yield ChunkedContentFactory(new_key, parents, record.sha1, chunks)
         stream = self.repo.texts.get_record_stream(
             needed_keys, 'topological', True)
         self.repo._remove_file_id(new_file_id)
@@ -377,7 +374,7 @@ class PackReconciler(VersionedFileRepoReconciler):
 
     def __init__(self, repo, other=None, thorough=False,
                  canonicalize_chks=False):
-        super(PackReconciler, self).__init__(repo, other=other,
+        super().__init__(repo, other=other,
                                              thorough=thorough)
         self.canonicalize_chks = canonicalize_chks
 
@@ -426,7 +423,7 @@ class PackReconciler(VersionedFileRepoReconciler):
         self.repo._pack_collection._obsolete_packs(packs)
 
 
-class BranchReconciler(object):
+class BranchReconciler:
     """Reconciler that works on a branch."""
 
     def __init__(self, a_branch, thorough=False):
