@@ -16,8 +16,6 @@
 
 """Server for smart-server protocol."""
 
-from __future__ import absolute_import
-
 import errno
 import os.path
 import socket
@@ -49,7 +47,7 @@ from breezy import (
 """)
 
 
-class SmartTCPServer(object):
+class SmartTCPServer:
     """Listens on a TCP socket and accepts connections from smart clients.
 
     Each connection will be served by a SmartServerSocketStreamMedium running in
@@ -247,7 +245,7 @@ class SmartTCPServer(object):
 
     def get_url(self):
         """Return the url of the server"""
-        return "bzr://%s:%s/" % (self._sockname[0], self._sockname[1])
+        return "bzr://{}:{}/".format(self._sockname[0], self._sockname[1])
 
     def _make_handler(self, conn):
         return medium.SmartServerSocketStreamMedium(
@@ -267,7 +265,7 @@ class SmartTCPServer(object):
         still_active = []
         for handler, thread in self._active_connections:
             thread.join(timeout)
-            if thread.isAlive():
+            if thread.is_alive():
                 still_active.append((handler, thread))
         self._active_connections = still_active
 
@@ -279,19 +277,17 @@ class SmartTCPServer(object):
         thread_name = 'smart-server-child' + thread_name_suffix
         handler = self._make_handler(conn)
         connection_thread = threading.Thread(
-            None, handler.serve, name=thread_name)
+            None, handler.serve, name=thread_name, daemon=True)
         self._active_connections.append((handler, connection_thread))
-        connection_thread.setDaemon(True)
         connection_thread.start()
         return connection_thread
 
     def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
-        self._server_thread = threading.Thread(None,
-                                               self.serve, args=(
-                                                   thread_name_suffix,),
-                                               name='server-' + self.get_url())
-        self._server_thread.setDaemon(True)
+        self._server_thread = threading.Thread(
+            None, self.serve, args=(thread_name_suffix,),
+            name='server-' + self.get_url(),
+            daemon=True)
         self._server_thread.start()
         self._started.wait()
 
@@ -309,7 +305,7 @@ class SmartTCPServer(object):
         except self._socket_error:
             # ignore errors on close
             pass
-        if not self._stopped.isSet():
+        if not self._stopped.is_set():
             # server has not stopped (though it may be stopping)
             # its likely in accept(), so give it a connection
             temp_socket = socket.socket()
@@ -352,7 +348,7 @@ class SmartServerHooks(Hooks):
                       "in which case the server will exit normally.", (2, 4))
 
 
-SmartTCPServer.hooks = SmartServerHooks()
+SmartTCPServer.hooks = SmartServerHooks()  # type: ignore
 
 
 def _local_path_for_transport(transport):
@@ -378,7 +374,7 @@ def _local_path_for_transport(transport):
             return None
 
 
-class BzrServerFactory(object):
+class BzrServerFactory:
     """Helper class for serve_bzr."""
 
     def __init__(self, userdir_expander=None, get_base_path=None):
@@ -435,10 +431,7 @@ class BzrServerFactory(object):
         self.transport = transport
 
     def _get_stdin_stdout(self):
-        if sys.version_info[0] < 3:
-            return sys.stdin, sys.stdout
-        else:
-            return sys.stdin.buffer, sys.stdout.buffer
+        return sys.stdin.buffer, sys.stdout.buffer
 
     def _make_smart_server(self, host, port, inet, timeout):
         if timeout is None:
