@@ -16,8 +16,6 @@
 
 """Command that signs unsigned commits by the current user. """
 
-from __future__ import absolute_import
-
 from . import (
     controldir,
     errors,
@@ -28,7 +26,6 @@ from . import (
 from .commands import Command
 from .option import Option
 from .i18n import gettext, ngettext
-from .sixish import text_type
 
 
 class cmd_sign_my_commits(Command):
@@ -102,14 +99,14 @@ class cmd_verify_signatures(Command):
                help='Comma separated list of GPG key patterns which are'
                ' acceptable for verification.',
                short_name='k',
-               type=text_type,),
+               type=str,),
         'revision',
         'verbose',
         ]
     takes_args = ['location?']
 
     def run(self, acceptable_keys=None, revision=None, verbose=None,
-            location=u'.'):
+            location='.'):
         bzrdir = controldir.ControlDir.open_containing(location)[0]
         branch = bzrdir.open_branch()
         repo = branch.repository
@@ -137,7 +134,7 @@ class cmd_verify_signatures(Command):
                 if to_revid is None:
                     to_revno = branch.revno()
                 if from_revno is None or to_revno is None:
-                    raise errors.BzrCommandError(
+                    raise errors.CommandError(
                         gettext('Cannot verify a range of non-revision-history'
                                 ' revisions'))
                 for revno in range(from_revno, to_revno + 1):
