@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+__docformat__ = "google"
+
 """Working tree content filtering support.
 
 A filter consists of a read converter, write converter pair.
@@ -38,50 +40,50 @@ where:
 Note that context is currently only supported for write converters.
 """
 
-from __future__ import absolute_import
 
 from io import (
     BytesIO,
     )
 
-from ..lazy_import import lazy_import
-lazy_import(globals(), """
-from breezy import (
-    config,
-    osutils,
-    registry,
-    )
-""")
+from typing import List, Tuple, Dict, Callable
 
 from .. import (
     errors,
+    osutils,
+    registry,
     )
 
 
-class ContentFilter(object):
+class ContentFilter:
 
     def __init__(self, reader, writer):
         """Create a filter that converts content while reading and writing.
 
-        :param reader: function for converting convenience to canonical content
-        :param writer: function for converting canonical to convenience content
+        Args:
+          reader: function for converting convenience to canonical content
+          writer: function for converting canonical to convenience content
         """
         self.reader = reader
         self.writer = writer
 
     def __repr__(self):
-        return "reader: %s, writer: %s" % (self.reader, self.writer)
+        return "reader: {}, writer: {}".format(self.reader, self.writer)
 
 
-class ContentFilterContext(object):
+Preferences = List[Tuple[str, str]]
+Stack = List[ContentFilter]
+
+
+class ContentFilterContext:
     """Object providing information that filters can use."""
 
     def __init__(self, relpath=None, tree=None):
         """Create a context.
 
-        :param relpath: the relative path or None if this context doesn't
+        Args:
+          relpath: the relative path or None if this context doesn't
            support that information.
-        :param tree: the Tree providing this file or None if this context
+          tree: the Tree providing this file or None if this context
            doesn't support that information.
         """
         self._relpath = relpath
@@ -121,9 +123,11 @@ class ContentFilterContext(object):
 def filtered_input_file(f, filters):
     """Get an input file that converts external to internal content.
 
-    :param f: the original input file
-    :param filters: the stack of filters to apply
-    :return: a file-like object, size
+    Args:
+      f: the original input file
+      filters: the stack of filters to apply
+
+    Returns: a file-like object, size
     """
     chunks = [f.read()]
     for filter in filters:
@@ -136,11 +140,13 @@ def filtered_input_file(f, filters):
 def filtered_output_bytes(chunks, filters, context=None):
     """Convert byte chunks from internal to external format.
 
-    :param chunks: an iterator containing the original content
-    :param filters: the stack of filters to apply
-    :param context: a ContentFilterContext object passed to
+    Args:
+      chunks: an iterator containing the original content
+      filters: the stack of filters to apply
+      context: a ContentFilterContext object passed to
         each filter
-    :return: an iterator containing the content to output
+
+    Returns: an iterator containing the content to output
     """
     if filters:
         for filter in reversed(filters):
@@ -152,8 +158,9 @@ def filtered_output_bytes(chunks, filters, context=None):
 def internal_size_sha_file_byname(name, filters):
     """Get size and sha of internal content given external content.
 
-    :param name: path to file
-    :param filters: the stack of filters to apply
+    Args:
+      name: path to file
+      filters: the stack of filters to apply
     """
     with open(name, 'rb', 65000) as f:
         if filters:
@@ -161,7 +168,7 @@ def internal_size_sha_file_byname(name, filters):
         return osutils.size_sha_file(f)
 
 
-class FilteredStat(object):
+class FilteredStat:
 
     def __init__(self, base, st_size=None):
         self.st_mode = base.st_mode
@@ -171,12 +178,12 @@ class FilteredStat(object):
 
 
 # The registry of filter stacks indexed by name.
-filter_stacks_registry = registry.Registry()
+filter_stacks_registry = registry.Registry[str, Callable[[str], List[ContentFilter]]]()
 
 
 # Cache of preferences -> stack
 # TODO: make this per branch (say) rather than global
-_stack_cache = {}
+_stack_cache: Dict[Preferences, Stack] = {}
 
 
 def _get_registered_names():
@@ -186,13 +193,14 @@ def _get_registered_names():
     return filter_stacks_registry.keys()
 
 
-def _get_filter_stack_for(preferences):
+def _get_filter_stack_for(preferences: Preferences) -> Stack:
     """Get the filter stack given a sequence of preferences.
 
-    :param preferences: a sequence of (name,value) tuples where
-      name is the preference name and
-      value is the key into the filter stack map registered
-      for that preference.
+    Args:
+      preferences: a sequence of (name,value) tuples where
+          name is the preference name and
+          value is the key into the filter stack map registered
+          for that preference.
     """
     if preferences is None:
         return []
@@ -224,8 +232,11 @@ def _reset_registry(value=None):
       # run tests
       _reset_registry(old)
 
-    :param value: the value to set the registry to or None for an empty one.
-    :return: the existing value before it reset.
+    Args:
+      value: the value to set the registry to or None for an empty one.
+
+    Returns:
+      the existing value before it reset.
     """
     global filter_stacks_registry
     original = filter_stacks_registry
