@@ -16,12 +16,11 @@
 
 """Remote dirs, repositories and branches."""
 
-from __future__ import absolute_import
-
 import gzip
-from io import BytesIO
 import re
 
+from dulwich.refs import SymrefLoop
+
 from .. import (
     config,
     debug,
@@ -31,30 +30,31 @@ from .. import (
     ui,
     urlutils,
     )
+from ..controldir import BranchReferenceLoop
 from ..push import (
     PushResult,
     )
 from ..errors import (
     AlreadyBranchError,
     BzrError,
+    ConnectionReset,
     DivergedBranches,
     InProcessTransport,
     InvalidRevisionId,
-    NoSuchFile,
+    LockContention,
     NoSuchRevision,
     NoSuchTag,
     NotBranchError,
     NotLocalUrl,
     PermissionDenied,
+    TransportError,
+    UnexpectedHttpStatus,
     UninitializableFormat,
     )
 from ..revision import NULL_REVISION
 from ..revisiontree import RevisionTree
-from ..sixish import (
-    text_type,
-    viewitems,
-    )
 from ..transport import (
+    NoSuchFile,
     Transport,
     register_urlparse_netloc_protocol,
     )
@@ -79,9 +79,9 @@ from .dir import (
     )
 from .errors import (
     GitSmartRemoteNotSupported,
-    NoSuchRef,
     )
 from .mapping import (
+    encode_git_path,
     mapping_registry,
     )
 from .object_store import (
@@ -110,6 +110,8 @@ from dulwich.errors import (
 from dulwich.pack import (
     Pack,
     pack_objects_to_data,
+    load_pack_index,
+    PACK_SPOOL_FILE_MAX_SIZE,
     )
 from dulwich.protocol import ZERO_SHA
 from dulwich.refs import (
@@ -121,21 +123,13 @@ from dulwich.repo import (
     )
 import os
 import select
-import tempfile
 
-try:
-    import urllib.parse as urlparse
-    from urllib.parse import splituser
-except ImportError:
-    import urlparse
-    from urllib import splituser
+import urllib.parse as urlparse
 
 # urlparse only supports a limited number of schemes by default
 register_urlparse_netloc_protocol('git')
 register_urlparse_netloc_protocol('git+ssh')
 
-from dulwich.pack import load_pack_index
-
 
 class GitPushResult(PushResult):
 
@@ -179,13 +173,18 @@ class RemoteGitError(BzrError):
     _fmt = "Remote server error: %(msg)s"
 
 
+class ProtectedBranchHookDeclined(BzrError):
+
+    _fmt = "Protected branch hook declined"
+
+
 class HeadUpdateFailed(BzrError):
 
     _fmt = ("Unable to update remote HEAD branch. To update the master "
             "branch, specify the URL %(base_url)s,branch=master.")
 
     def __init__(self, base_url):
-        super(HeadUpdateFailed, self).__init__()
+        super().__init__()
         self.base_url = base_url
 
 
@@ -211,12 +210,32 @@ def parse_git_error(url, message):
         return PermissionDenied(url, message)
     if message.endswith(' does not appear to be a git repository'):
         return NotBranchError(url, message)
+    if message == 'A repository for this project does not exist yet.':
+        return NotBranchError(url, message)
+    if message == 'pre-receive hook declined':
+        return PermissionDenied(url, message)
     if re.match('(.+) is not a valid repository name',
                 message.splitlines()[0]):
         return NotBranchError(url, message)
+    if message == (
+            'GitLab: You are not allowed to push code to protected branches '
+            'on this project.'):
+        return PermissionDenied(url, message)
     m = re.match(r'Permission to ([^ ]+) denied to ([^ ]+)\.', message)
     if m:
         return PermissionDenied(m.group(1), 'denied to %s' % m.group(2))
+    if message == 'Host key verification failed.':
+        return TransportError('Host key verification failed')
+    if message == '[Errno 104] Connection reset by peer':
+        return ConnectionReset(message)
+    if message == 'The remote server unexpectedly closed the connection.':
+        return TransportError(message)
+    m = re.match(r'unexpected http resp ([0-9]+) for (.*)', message)
+    if m:
+        # TODO(jelmer): Have dulwich raise an exception and look at that instead?
+        return UnexpectedHttpStatus(path=m.group(2), code=int(m.group(1)), extra=message)
+    if message == 'protected branch hook declined':
+        return ProtectedBranchHookDeclined(msg=message)
     # Don't know, just return it to the user as-is
     return RemoteGitError(message)
 
@@ -229,14 +248,19 @@ def parse_git_hangup(url, e):
     """
     stderr_lines = getattr(e, 'stderr_lines', None)
     if not stderr_lines:
-        return e
+        return ConnectionReset('Connection closed early', e)
+    if all(line.startswith(b'remote: ') for line in stderr_lines):
+        stderr_lines = [
+            line[len(b'remote: '):] for line in stderr_lines]
     interesting_lines = [
         line for line in stderr_lines
-        if line and line.replace('=', '')]
+        if line and line.replace(b'=', b'')]
     if len(interesting_lines) == 1:
         interesting_line = interesting_lines[0]
-        return parse_git_error(url, interesting_line)
-    return RemoteGitError('\n'.join(stderr_lines))
+        return parse_git_error(
+            url, interesting_line.decode('utf-8', 'surrogateescape'))
+    return RemoteGitError(
+        b'\n'.join(stderr_lines).decode('utf-8', 'surrogateescape'))
 
 
 class GitSmartTransport(Transport):
@@ -295,7 +319,7 @@ class TCPGitSmartTransport(GitSmartTransport):
             self._host, self._port, report_activity=self._report_activity)
 
 
-class SSHSocketWrapper(object):
+class SSHSocketWrapper:
 
     def __init__(self, sock):
         self.sock = sock
@@ -373,19 +397,22 @@ class RemoteGitBranchFormat(GitBranchFormat):
         raise UninitializableFormat(self)
 
 
-class DefaultProgressReporter(object):
+class DefaultProgressReporter:
 
     _GIT_PROGRESS_PARTIAL_RE = re.compile(r"(.*?): +(\d+)% \((\d+)/(\d+)\)")
     _GIT_PROGRESS_TOTAL_RE = re.compile(r"(.*?): (\d+)")
 
     def __init__(self, pb):
         self.pb = pb
+        self.errors = []
 
     def progress(self, text):
         text = text.rstrip(b"\r\n")
-        text = text.decode('utf-8')
+        text = text.decode('utf-8', 'surrogateescape')
         if text.lower().startswith('error: '):
-            trace.show_error('git: %s', text[len(b'error: '):])
+            error = text[len('error: '):]
+            self.errors.append(error)
+            trace.show_error('git: %s', error)
         else:
             trace.mutter("git: %s", text)
             g = self._GIT_PROGRESS_PARTIAL_RE.match(text)
@@ -401,6 +428,9 @@ class DefaultProgressReporter(object):
                     trace.note("%s", text)
 
 
+_LOCK_REF_ERROR_MATCHER = re.compile('cannot lock ref \'(.*)\': (.*)')
+
+
 class RemoteGitDir(GitDir):
 
     def __init__(self, transport, format, client, client_path):
@@ -418,7 +448,9 @@ class RemoteGitDir(GitDir):
         return RemoteGitRepository
 
     def archive(self, format, committish, write_data, progress=None,
-                write_error=None, subdirs=None, prefix=None):
+                write_error=None, subdirs=None, prefix=None, recurse_nested=False):
+        if recurse_nested:
+            raise NotImplementedError('recurse_nested is not yet supported')
         if progress is None:
             pb = ui.ui_factory.nested_progress_bar()
             progress = DefaultProgressReporter(pb).progress
@@ -435,11 +467,11 @@ class RemoteGitDir(GitDir):
                 write_error,
                 format=(format.encode('ascii') if format else None),
                 subdirs=subdirs,
-                prefix=(prefix.encode('utf-8') if prefix else None))
-        except GitProtocolError as e:
-            raise parse_git_error(self.transport.external_url(), e)
+                prefix=(encode_git_path(prefix) if prefix else None))
         except HangupException as e:
             raise parse_git_hangup(self.transport.external_url(), e)
+        except GitProtocolError as e:
+            raise parse_git_error(self.transport.external_url(), e)
         finally:
             if pb is not None:
                 pb.finished()
@@ -460,10 +492,10 @@ class RemoteGitDir(GitDir):
             self._refs = remote_refs_dict_to_container(
                 result.refs, result.symrefs)
             return result
-        except GitProtocolError as e:
-            raise parse_git_error(self.transport.external_url(), e)
         except HangupException as e:
             raise parse_git_hangup(self.transport.external_url(), e)
+        except GitProtocolError as e:
+            raise parse_git_error(self.transport.external_url(), e)
         finally:
             if pb is not None:
                 pb.finished()
@@ -471,8 +503,10 @@ class RemoteGitDir(GitDir):
     def send_pack(self, get_changed_refs, generate_pack_data, progress=None):
         if progress is None:
             pb = ui.ui_factory.nested_progress_bar()
-            progress = DefaultProgressReporter(pb).progress
+            progress_reporter = DefaultProgressReporter(pb)
+            progress = progress_reporter.progress
         else:
+            progress_reporter = None
             pb = None
 
         def get_changed_refs_wrapper(remote_refs):
@@ -480,13 +514,22 @@ class RemoteGitDir(GitDir):
                 update_refs_container(self._refs, remote_refs)
             return get_changed_refs(remote_refs)
         try:
-            return self._client.send_pack(
+            result = self._client.send_pack(
                 self._client_path, get_changed_refs_wrapper,
                 generate_pack_data, progress)
-        except GitProtocolError as e:
-            raise parse_git_error(self.transport.external_url(), e)
+            for ref, msg in list(result.ref_status.items()):
+                if msg:
+                    result.ref_status[ref] = RemoteGitError(msg=msg)
+            if progress_reporter:
+                for error in progress_reporter.errors:
+                    m = _LOCK_REF_ERROR_MATCHER.match(error)
+                    if m:
+                        result.ref_status[m.group(1)] = LockContention(m.group(1), m.group(2))
+            return result
         except HangupException as e:
             raise parse_git_hangup(self.transport.external_url(), e)
+        except GitProtocolError as e:
+            raise parse_git_error(self.transport.external_url(), e)
         finally:
             if pb is not None:
                 pb.finished()
@@ -496,12 +539,12 @@ class RemoteGitDir(GitDir):
         refname = self._get_selected_ref(name, ref)
         if refname != b'HEAD' and refname in self.get_refs_container():
             raise AlreadyBranchError(self.user_url)
-        ref_chain, unused_sha = self.get_refs_container().follow(
+        ref_chain, sha = self.get_refs_container().follow(
             self._get_selected_ref(name))
-        if ref_chain and ref_chain[0] == b'HEAD':
+        if ref_chain and ref_chain[0] == b'HEAD' and len(ref_chain) > 1:
             refname = ref_chain[1]
         repo = self.open_repository()
-        return RemoteGitBranch(self, repo, refname)
+        return RemoteGitBranch(self, repo, refname, sha)
 
     def destroy_branch(self, name=None):
         refname = self._get_selected_ref(name)
@@ -513,9 +556,12 @@ class RemoteGitDir(GitDir):
             ret[refname] = dulwich.client.ZERO_SHA
             return ret
 
-        def generate_pack_data(have, want, ofs_delta=False):
+        def generate_pack_data(have, want, ofs_delta=False, progress=None):
             return pack_objects_to_data([])
-        self.send_pack(get_changed_refs, generate_pack_data)
+        result = self.send_pack(get_changed_refs, generate_pack_data)
+        error = result.ref_status.get(refname)
+        if error:
+            raise error
 
     @property
     def user_url(self):
@@ -537,11 +583,25 @@ class RemoteGitDir(GitDir):
         return RemoteGitRepository(self)
 
     def get_branch_reference(self, name=None):
-        ref = branch_name_to_ref(name)
-        val = self.get_refs_container().read_ref(ref)
-        if val.startswith(SYMREF):
-            return val[len(SYMREF):]
-        return None
+        ref = self._get_selected_ref(name)
+        try:
+            ref_chain, unused_sha = self.get_refs_container().follow(ref)
+        except SymrefLoop:
+            raise BranchReferenceLoop(self)
+        if len(ref_chain) == 1:
+            return None
+        target_ref = ref_chain[1]
+        from .refs import ref_to_branch_name
+        try:
+            branch_name = ref_to_branch_name(target_ref)
+        except ValueError:
+            params = {'ref': urlutils.quote(target_ref.decode('utf-8'), '')}
+        else:
+            if branch_name != '':
+                params = {'branch': urlutils.quote(branch_name, '')}
+            else:
+                params = {}
+        return urlutils.join_segment_parameters(self.user_url.rstrip('/'), params)
 
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False, ref=None, possible_transports=None,
@@ -549,14 +609,16 @@ class RemoteGitDir(GitDir):
         repo = self.open_repository()
         ref = self._get_selected_ref(name, ref)
         try:
-            if not nascent_ok and ref not in self.get_refs_container():
-                raise NotBranchError(
-                    self.root_transport.base, controldir=self)
+            ref_chain, sha = self.get_refs_container().follow(ref)
+        except SymrefLoop:
+            raise BranchReferenceLoop(self)
         except NotGitRepository:
             raise NotBranchError(self.root_transport.base,
                                  controldir=self)
-        ref_chain, unused_sha = self.get_refs_container().follow(ref)
-        return RemoteGitBranch(self, repo, ref_chain[-1])
+        if not nascent_ok and sha is None:
+            raise NotBranchError(
+                self.root_transport.base, controldir=self)
+        return RemoteGitBranch(self, repo, ref_chain[-1], sha)
 
     def open_workingtree(self, recommend_upgrade=False):
         raise NotLocalUrl(self.transport.base)
@@ -585,6 +647,9 @@ class RemoteGitDir(GitDir):
             # No revision supplied by the user, default to the branch
             # revision
             revision_id = source.last_revision()
+        else:
+            if not source.repository.has_revision(revision_id):
+                raise NoSuchRevision(source, revision_id)
 
         push_result = GitPushResult()
         push_result.workingtree_updated = None
@@ -594,11 +659,16 @@ class RemoteGitDir(GitDir):
         push_result.branch_push_result = None
         repo = self.find_repository()
         refname = self._get_selected_ref(name)
-        ref_chain, old_sha = self.get_refs_container().follow(refname)
-        if ref_chain:
-            actual_refname = ref_chain[-1]
-        else:
+        try:
+            ref_chain, old_sha = self.get_refs_container().follow(refname)
+        except NotBranchError:
             actual_refname = refname
+            old_sha = None
+        else:
+            if ref_chain:
+                actual_refname = ref_chain[-1]
+            else:
+                actual_refname = refname
         if isinstance(source, GitBranch) and lossy:
             raise errors.LossyPushToSameVCS(source.controldir, self)
         source_store = get_object_store(source.repository)
@@ -617,13 +687,14 @@ class RemoteGitDir(GitDir):
                 except errors.NoSuchRevision:
                     raise errors.NoRoundtrippingSupport(
                         source, self.open_branch(name=name, nascent_ok=True))
+            old_sha = remote_refs.get(actual_refname)
             if not overwrite:
                 if remote_divergence(old_sha, new_sha, source_store):
                     raise DivergedBranches(
                         source, self.open_branch(name, nascent_ok=True))
             ret[actual_refname] = new_sha
             if fetch_tags:
-                for tagname, revid in viewitems(source.tags.get_tag_dict()):
+                for tagname, revid in source.tags.get_tag_dict().items():
                     if tag_selector and not tag_selector(tagname):
                         continue
                     if lossy:
@@ -637,6 +708,9 @@ class RemoteGitDir(GitDir):
                             new_sha = repo.lookup_bzr_revision_id(revid)[0]
                         except errors.NoSuchRevision:
                             continue
+                        else:
+                            if not source.repository.has_revision(revid):
+                                continue
                     ret[tag_name_to_ref(tagname)] = new_sha
             return ret
         with source_store.lock_read():
@@ -658,7 +732,15 @@ class RemoteGitDir(GitDir):
                 else:
                     return source_store.generate_pack_data(
                         have, want, progress=progress, ofs_delta=ofs_delta)
-            new_refs = self.send_pack(get_changed_refs, generate_pack_data)
+            dw_result = self.send_pack(get_changed_refs, generate_pack_data)
+            new_refs = dw_result.refs
+            error = dw_result.ref_status.get(actual_refname)
+            if error:
+                raise error
+            for ref, error in dw_result.ref_status.items():
+                if error:
+                    trace.warning('unable to open ref %s: %s',
+                                  ref, error)
         push_result.new_revid = repo.lookup_foreign_revision_id(
             new_refs[actual_refname])
         if old_sha is not None:
@@ -698,7 +780,7 @@ class EmptyObjectStoreIterator(dict):
 class TemporaryPackIterator(Pack):
 
     def __init__(self, path, resolve_ext_ref):
-        super(TemporaryPackIterator, self).__init__(
+        super().__init__(
             path, resolve_ext_ref=resolve_ext_ref)
         self._idx_load = lambda: self._idx_load_or_generate(self._idx_path)
 
@@ -727,7 +809,20 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         url.user = url.quoted_user = None
         url.password = url.quoted_password = None
         url = urlutils.strip_segment_parameters(str(url))
-        super(BzrGitHttpClient, self).__init__(url, *args, **kwargs)
+        super().__init__(url, *args, **kwargs)
+
+    def archive(
+        self,
+        path,
+        committish,
+        write_data,
+        progress=None,
+        write_error=None,
+        format=None,
+        subdirs=None,
+        prefix=None,
+    ):
+        raise GitSmartRemoteNotSupported(self.archive, self)
 
     def _http_request(self, url, headers=None, data=None,
                       allow_compression=False):
@@ -736,7 +831,6 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         :param url: Request URL.
         :param headers: Optional custom headers to override defaults.
         :param data: Request data.
-        :param allow_compression: Allow GZipped communication.
         :return: Tuple (`response`, `read`), where response is an `urllib3`
             response object with additional `content_type` and
             `redirect_location` properties, and `read` is a consumable read
@@ -745,10 +839,6 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
         if is_github_url(url):
             headers['User-agent'] = user_agent_for_github()
         headers["Pragma"] = "no-cache"
-        if allow_compression:
-            headers["Accept-Encoding"] = "gzip"
-        else:
-            headers["Accept-Encoding"] = "identity"
 
         response = self.transport.request(
             ('GET' if data is None else 'POST'),
@@ -762,17 +852,9 @@ class BzrGitHttpClient(dulwich.client.HttpGitClient):
             raise GitProtocolError("unexpected http resp %d for %s" %
                                    (response.status, url))
 
-        # TODO: Optimization available by adding `preload_content=False` to the
-        # request and just passing the `read` method on instead of going via
-        # `BytesIO`, if we can guarantee that the entire response is consumed
-        # before issuing the next to still allow for connection reuse from the
-        # pool.
-        if response.getheader("Content-Encoding") == "gzip":
-            read = gzip.GzipFile(fileobj=BytesIO(response.read())).read
-        else:
-            read = response.read
+        read = response.read
 
-        class WrapResponse(object):
+        class WrapResponse:
 
             def __init__(self, response):
                 self._response = response
@@ -801,7 +883,7 @@ class RemoteGitControlDirFormat(GitControlDirFormat):
 
     @classmethod
     def _known_formats(self):
-        return set([RemoteGitControlDirFormat()])
+        return {RemoteGitControlDirFormat()}
 
     def get_branch_format(self):
         return RemoteGitBranchFormat()
@@ -852,7 +934,7 @@ class RemoteGitControlDirFormat(GitControlDirFormat):
 
 class GitRemoteRevisionTree(RevisionTree):
 
-    def archive(self, format, name, root=None, subdir=None, force_mtime=None):
+    def archive(self, format, name, root=None, subdir=None, force_mtime=None, recurse_nested=False):
         """Create an archive of this tree.
 
         :param format: Format name (e.g. 'tar')
@@ -861,9 +943,13 @@ class GitRemoteRevisionTree(RevisionTree):
         :param subdir: Subdirectory to export (or None)
         :return: Iterator over archive chunks
         """
+        if recurse_nested:
+            # TODO(jelmer): Parse .gitmodules from archive afterwards?
+            raise NotImplementedError('recurse_nested is not yet supported')
         commit = self._repository.lookup_bzr_revision_id(
             self.get_revision_id())[0]
-        f = tempfile.SpooledTemporaryFile()
+        from tempfile import SpooledTemporaryFile
+        f = SpooledTemporaryFile(max_size=PACK_SPOOL_FILE_MAX_SIZE, prefix='incoming-')
         # git-upload-archive(1) generaly only supports refs. So let's see if we
         # can find one.
         reverse_refs = {
@@ -919,6 +1005,7 @@ class RemoteGitRepository(GitRepository):
 
     def fetch_objects(self, determine_wants, graph_walker, resolve_ext_ref,
                       progress=None):
+        import tempfile
         fd, path = tempfile.mkstemp(suffix=".pack")
         try:
             self.fetch_pack(determine_wants, graph_walker,
@@ -975,16 +1062,20 @@ class RemoteGitTagDict(GitTags):
             ret[ref] = sha
             return ret
 
-        def generate_pack_data(have, want, ofs_delta=False):
+        def generate_pack_data(have, want, ofs_delta=False, progress=None):
             return pack_objects_to_data([])
-        self.repository.send_pack(get_changed_refs, generate_pack_data)
+        result = self.repository.send_pack(
+            get_changed_refs, generate_pack_data)
+        error = result.ref_status.get(ref)
+        if error:
+            raise error
 
 
 class RemoteGitBranch(GitBranch):
 
-    def __init__(self, controldir, repository, name):
-        self._sha = None
-        super(RemoteGitBranch, self).__init__(controldir, repository, name,
+    def __init__(self, controldir, repository, ref, sha):
+        self._sha = sha
+        super().__init__(controldir, repository, ref,
                                               RemoteGitBranchFormat())
 
     def last_revision_info(self):
@@ -1006,14 +1097,6 @@ class RemoteGitBranch(GitBranch):
 
     @property
     def head(self):
-        if self._sha is not None:
-            return self._sha
-        refs = self.controldir.get_refs_container()
-        name = branch_name_to_ref(self.name)
-        try:
-            self._sha = refs[name]
-        except KeyError:
-            raise NoSuchRef(name, self.repository.user_url, refs)
         return self._sha
 
     def _synchronize_history(self, destination, revision_id):
@@ -1047,7 +1130,7 @@ class RemoteGitBranch(GitBranch):
             if peeled is None:
                 # Let's just hope it's a commit
                 peeled = unpeeled
-            if not isinstance(tag_name, text_type):
+            if not isinstance(tag_name, str):
                 raise TypeError(tag_name)
             yield (ref_name, tag_name, peeled, unpeeled)
 
@@ -1059,9 +1142,13 @@ class RemoteGitBranch(GitBranch):
         sha = self.lookup_bzr_revision_id(revision_id)[0]
         def get_changed_refs(old_refs):
             return {self.ref: sha}
-        def generate_pack_data(have, want, ofs_delta=False):
+        def generate_pack_data(have, want, ofs_delta=False, progress=None):
             return pack_objects_to_data([])
-        self.repository.send_pack(get_changed_refs, generate_pack_data)
+        result = self.repository.send_pack(
+            get_changed_refs, generate_pack_data)
+        error = result.ref_status.get(self.ref)
+        if error:
+            raise error
         self._sha = sha
 
 
