@@ -15,9 +15,23 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""An adapter between a Git Repository and a Bazaar Branch"""
+"""An adapter between a Git Repository and a Breezy one."""
+
+from io import BytesIO
+
+from dulwich.errors import (
+    NotCommitError,
+    )
+from dulwich.objects import (
+    Commit,
+    ZERO_SHA,
+    )
+from dulwich.object_store import (
+    peel_sha,
+    tree_lookup_path,
+    )
+
 
-from __future__ import absolute_import
 
 from .. import (
     check,
@@ -34,20 +48,14 @@ from ..decorators import only_raises
 from ..foreign import (
     ForeignRepository,
     )
-from ..sixish import (
-    viewitems,
-    viewvalues,
-    )
 
-from .commit import (
-    GitCommitBuilder,
-    )
 from .filegraph import (
     GitFileLastChangeScanner,
     GitFileParentProvider,
     )
 from .mapping import (
     default_mapping,
+    encode_git_path,
     foreign_vcs_git,
     mapping_registry,
     )
@@ -56,18 +64,6 @@ from .tree import (
     )
 
 
-from dulwich.errors import (
-    NotCommitError,
-    )
-from dulwich.objects import (
-    Commit,
-    ZERO_SHA,
-    )
-from dulwich.object_store import (
-    tree_lookup_path,
-    )
-
-
 class GitCheck(check.Check):
 
     def __init__(self, repository, check_repo=True):
@@ -94,7 +90,7 @@ class GitCheck(check.Check):
                     self.problems.append((sha, e))
 
     def _report_repo_results(self, verbose):
-        trace.note('checked repository {0} format {1}'.format(
+        trace.note('checked repository {} format {}'.format(
             self.repository.user_url,
             self.repository._format))
         trace.note('%6d objects', self.object_count)
@@ -106,38 +102,29 @@ class GitCheck(check.Check):
             self._report_repo_results(verbose)
 
 
-_optimisers_loaded = False
-
-
-def lazy_load_optimisers():
-    global _optimisers_loaded
-    if _optimisers_loaded:
-        return
-    from . import interrepo
-    for optimiser in [interrepo.InterRemoteGitNonGitRepository,
-                      interrepo.InterLocalGitNonGitRepository,
-                      interrepo.InterLocalGitLocalGitRepository,
-                      interrepo.InterRemoteGitLocalGitRepository,
-                      interrepo.InterToLocalGitRepository,
-                      interrepo.InterToRemoteGitRepository,
-                      ]:
-        repository.InterRepository.register_optimiser(optimiser)
-    _optimisers_loaded = True
+for optimiser in ['InterRemoteGitNonGitRepository',
+                  'InterLocalGitNonGitRepository',
+                  'InterLocalGitLocalGitRepository',
+                  'InterLocalGitRemoteGitRepository',
+                  'InterRemoteGitLocalGitRepository',
+                  'InterToLocalGitRepository',
+                  'InterToRemoteGitRepository',
+                  ]:
+    repository.InterRepository.register_lazy_optimiser(
+        'breezy.git.interrepo', optimiser)
 
 
 class GitRepository(ForeignRepository):
     """An adapter to git repositories for bzr."""
 
-    _serializer = None
     vcs = foreign_vcs_git
     chk_bytes = None
 
     def __init__(self, gitdir):
         self._transport = gitdir.root_transport
-        super(GitRepository, self).__init__(GitRepositoryFormat(),
+        super().__init__(GitRepositoryFormat(),
                                             gitdir, control_files=None)
         self.base = gitdir.root_transport.base
-        lazy_load_optimisers()
         self._lock_mode = None
         self._lock_count = 0
 
@@ -200,6 +187,8 @@ class GitRepository(ForeignRepository):
             transaction = self._transaction
             self._transaction = None
             transaction.finish()
+            if hasattr(self, '_git'):
+                self._git.close()
 
     def is_write_locked(self):
         return (self._lock_mode == 'w')
@@ -228,7 +217,7 @@ class GitRepository(ForeignRepository):
         return default_mapping
 
     def make_working_trees(self):
-        return not self._git.get_config().get_boolean(("core", ), "bare")
+        raise NotImplementedError(self.make_working_trees)
 
     def revision_graph_can_have_wrong_parents(self):
         return False
@@ -265,12 +254,20 @@ class LocalGitRepository(GitRepository):
         :param lossy: Whether to discard data that can not be natively
             represented, when pushing to a foreign VCS
         """
+        from .commit import (
+            GitCommitBuilder,
+            )
         builder = GitCommitBuilder(
             self, parents, config, timestamp, timezone, committer, revprops,
             revision_id, lossy)
         self.start_write_group()
         return builder
 
+    def _write_git_config(self, cs):
+        f = BytesIO()
+        cs.write_to_file(f)
+        self._git._put_named_file('config', f.getvalue())
+
     def get_file_graph(self):
         return _mod_graph.Graph(GitFileParentProvider(
             self._file_change_scanner))
@@ -298,7 +295,7 @@ class LocalGitRepository(GitRepository):
         for (file_id, revision_id, identifier) in desired_files:
             per_revision.setdefault(revision_id, []).append(
                 (file_id, identifier))
-        for revid, files in viewitems(per_revision):
+        for revid, files in per_revision.items():
             try:
                 (commit_id, mapping) = self.lookup_bzr_revision_id(revid)
             except errors.NoSuchRevision:
@@ -314,12 +311,10 @@ class LocalGitRepository(GitRepository):
                 except ValueError:
                     raise errors.RevisionNotPresent((fileid, revid), self)
                 try:
-                    obj = tree_lookup_path(
+                    mode, item_id = tree_lookup_path(
                         self._git.object_store.__getitem__, root_tree,
-                        path.encode('utf-8'))
-                    if isinstance(obj, tuple):
-                        (mode, item_id) = obj
-                        obj = self._git.object_store[item_id]
+                        encode_git_path(path))
+                    obj = self._git.object_store[item_id]
                 except KeyError:
                     raise errors.RevisionNotPresent((fileid, revid), self)
                 else:
@@ -332,7 +327,7 @@ class LocalGitRepository(GitRepository):
 
     def gather_stats(self, revid=None, committers=None):
         """See Repository.gather_stats()."""
-        result = super(LocalGitRepository, self).gather_stats(
+        result = super().gather_stats(
             revid, committers)
         revs = []
         for sha in self._git.object_store:
@@ -410,7 +405,7 @@ class LocalGitRepository(GitRepository):
                     this_parent_map[revid] = parents
             parent_map.update(this_parent_map)
             pending = set()
-            for values in viewvalues(this_parent_map):
+            for values in this_parent_map.values():
                 pending.update(values)
             pending = pending.difference(parent_map)
         return _mod_graph.KnownGraph(parent_map)
@@ -447,10 +442,10 @@ class LocalGitRepository(GitRepository):
             mapping = self.get_mapping()
         if foreign_revid == ZERO_SHA:
             return _mod_revision.NULL_REVISION
-        commit = self._git.object_store.peel_sha(foreign_revid)
-        if not isinstance(commit, Commit):
-            raise NotCommitError(commit.id)
-        revid = mapping.get_revision_id(commit)
+        unpeeled, peeled = peel_sha(self._git.object_store, foreign_revid)
+        if not isinstance(peeled, Commit):
+            raise NotCommitError(peeled.id)
+        revid = mapping.get_revision_id(peeled)
         # FIXME: check testament before doing this?
         return revid
 
@@ -559,48 +554,11 @@ class LocalGitRepository(GitRepository):
             raise ValueError('invalid revision id %s' % revision_id)
         return GitRevisionTree(self, revision_id)
 
-    def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        """Produce a generator of revision deltas.
-
-        Note that the input is a sequence of REVISIONS, not revision_ids.
-        Trees will be held in memory until the generator exits.
-        Each delta is relative to the revision's lefthand predecessor.
-
-        :param specific_fileids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        # Get the revision-ids of interest
-        required_trees = set()
-        for revision in revisions:
-            required_trees.add(revision.revision_id)
-            required_trees.update(revision.parent_ids[:1])
-
-        trees = dict((t.get_revision_id(), t) for
-                     t in self.revision_trees(required_trees))
-
-        # Calculate the deltas
-        for revision in revisions:
-            if not revision.parent_ids:
-                old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
-            else:
-                old_tree = trees[revision.parent_ids[0]]
-            new_tree = trees[revision.revision_id]
-            if specific_fileids is not None:
-                specific_files = [new_tree.id2path(
-                    fid) for fid in specific_fileids]
-            else:
-                specific_files = None
-            yield new_tree.changes_from(
-                old_tree, specific_files=specific_files)
-
     def set_make_working_trees(self, trees):
         raise errors.UnsupportedOperation(self.set_make_working_trees, self)
 
-    def fetch_objects(self, determine_wants, graph_walker, resolve_ext_ref,
-                      progress=None, limit=None):
-        return self._git.fetch_objects(determine_wants, graph_walker, progress,
-                                       limit=limit)
+    def make_working_trees(self):
+        return not self._git.get_config().get_boolean(("core", ), "bare")
 
 
 class GitRepositoryFormat(repository.RepositoryFormat):
@@ -623,6 +581,9 @@ class GitRepositoryFormat(repository.RepositoryFormat):
     supports_overriding_transport = False
     supports_custom_revision_properties = False
     records_per_file_revision = False
+    supports_multiple_authors = False
+    supports_ghosts = False
+    supports_chks = False
 
     @property
     def _matchingcontroldir(self):
