@@ -1,5 +1,4 @@
 # Copyright (C) 2007-2018 Jelmer Vernooij <jelmer@jelmer.uk>
-# -*- encoding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +16,6 @@
 
 """Tests for mapping."""
 
-from __future__ import absolute_import
-
 from ...revision import (
     Revision,
     )
@@ -34,15 +31,14 @@ from dulwich.tests.utils import (
     )
 
 from .. import tests
-from ..errors import (
-    UnknownCommitExtra,
-    UnknownMercurialCommitExtra,
-    )
 from ..mapping import (
     BzrGitMappingv1,
     escape_file_id,
     fix_person_identifier,
     unescape_file_id,
+    UnknownCommitExtra,
+    UnknownCommitEncoding,
+    UnknownMercurialCommitExtra,
     )
 
 
@@ -71,7 +67,7 @@ class TestRevidConversionV1(tests.TestCase):
     def test_generate_file_id(self):
         mapping = BzrGitMappingv1()
         self.assertIsInstance(mapping.generate_file_id("la"), bytes)
-        self.assertIsInstance(mapping.generate_file_id(u"é"), bytes)
+        self.assertIsInstance(mapping.generate_file_id("é"), bytes)
 
 
 class FileidTests(tests.TestCase):
@@ -111,19 +107,36 @@ class TestImportCommit(tests.TestCase):
         c.author_timezone = 60 * 3
         c.author = b"Author"
         mapping = BzrGitMappingv1()
-        rev, roundtrip_revid, verifiers = mapping.import_commit(c,
-                                                                mapping.revision_id_foreign_to_bzr)
+        rev, roundtrip_revid, verifiers = mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr)
         self.assertEqual(None, roundtrip_revid)
         self.assertEqual({}, verifiers)
-        self.assertEqual(u"Some message", rev.message)
-        self.assertEqual(u"Committer", rev.committer)
-        self.assertEqual(u"Author", rev.properties[u'author'])
+        self.assertEqual("Some message", rev.message)
+        self.assertEqual("Committer", rev.committer)
+        self.assertEqual("Author", rev.properties['author'])
         self.assertEqual(300, rev.timezone)
         self.assertEqual([], rev.parent_ids)
-        self.assertEqual("5", rev.properties[u'author-timestamp'])
-        self.assertEqual("180", rev.properties[u'author-timezone'])
+        self.assertEqual("5", rev.properties['author-timestamp'])
+        self.assertEqual("180", rev.properties['author-timezone'])
         self.assertEqual(b"git-v1:" + c.id, rev.revision_id)
 
+    def test_unknown_encoding(self):
+        c = Commit()
+        c.tree = b"cc9462f7f8263ef5adfbeff2fb936bb36b504cba"
+        c.message = b"Some message"
+        c.committer = b"Committer"
+        c.commit_time = 4
+        c.author_time = 5
+        c.commit_timezone = 60 * 5
+        c.author_timezone = 60 * 3
+        c.author = "Authér".encode("iso8859-1")
+        c.encoding = b"Unknown"
+        mapping = BzrGitMappingv1()
+        e = self.assertRaises(
+            UnknownCommitEncoding, mapping.import_commit,
+            c, mapping.revision_id_foreign_to_bzr)
+        self.assertEqual(e.encoding, "Unknown")
+
     def test_explicit_encoding(self):
         c = Commit()
         c.tree = b"cc9462f7f8263ef5adfbeff2fb936bb36b504cba"
@@ -133,16 +146,36 @@ class TestImportCommit(tests.TestCase):
         c.author_time = 5
         c.commit_timezone = 60 * 5
         c.author_timezone = 60 * 3
-        c.author = u"Authér".encode("iso8859-1")
+        c.author = "Authér".encode("iso8859-1")
         c.encoding = b"iso8859-1"
         mapping = BzrGitMappingv1()
-        rev, roundtrip_revid, verifiers = mapping.import_commit(c,
-                                                                mapping.revision_id_foreign_to_bzr)
+        rev, roundtrip_revid, verifiers = mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr)
         self.assertEqual(None, roundtrip_revid)
         self.assertEqual({}, verifiers)
-        self.assertEqual(u"Authér", rev.properties[u'author'])
-        self.assertEqual("iso8859-1", rev.properties[u"git-explicit-encoding"])
-        self.assertTrue(u"git-implicit-encoding" not in rev.properties)
+        self.assertEqual("Authér", rev.properties['author'])
+        self.assertEqual("iso8859-1", rev.properties["git-explicit-encoding"])
+        self.assertTrue("git-implicit-encoding" not in rev.properties)
+
+    def test_explicit_encoding_false(self):
+        c = Commit()
+        c.tree = b"cc9462f7f8263ef5adfbeff2fb936bb36b504cba"
+        c.message = b"Some message"
+        c.committer = b"Committer"
+        c.commit_time = 4
+        c.author_time = 5
+        c.commit_timezone = 60 * 5
+        c.author_timezone = 60 * 3
+        c.author = "Authér".encode()
+        c.encoding = b"false"
+        mapping = BzrGitMappingv1()
+        rev, roundtrip_revid, verifiers = mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr)
+        self.assertEqual(None, roundtrip_revid)
+        self.assertEqual({}, verifiers)
+        self.assertEqual("Authér", rev.properties['author'])
+        self.assertEqual("false", rev.properties["git-explicit-encoding"])
+        self.assertTrue("git-implicit-encoding" not in rev.properties)
 
     def test_implicit_encoding_fallback(self):
         c = Commit()
@@ -153,15 +186,15 @@ class TestImportCommit(tests.TestCase):
         c.author_time = 5
         c.commit_timezone = 60 * 5
         c.author_timezone = 60 * 3
-        c.author = u"Authér".encode("latin1")
+        c.author = "Authér".encode("latin1")
         mapping = BzrGitMappingv1()
-        rev, roundtrip_revid, verifiers = mapping.import_commit(c,
-                                                                mapping.revision_id_foreign_to_bzr)
+        rev, roundtrip_revid, verifiers = mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr)
         self.assertEqual(None, roundtrip_revid)
         self.assertEqual({}, verifiers)
-        self.assertEqual(u"Authér", rev.properties[u'author'])
-        self.assertEqual("latin1", rev.properties[u"git-implicit-encoding"])
-        self.assertTrue(u"git-explicit-encoding" not in rev.properties)
+        self.assertEqual("Authér", rev.properties['author'])
+        self.assertEqual("latin1", rev.properties["git-implicit-encoding"])
+        self.assertTrue("git-explicit-encoding" not in rev.properties)
 
     def test_implicit_encoding_utf8(self):
         c = Commit()
@@ -172,15 +205,15 @@ class TestImportCommit(tests.TestCase):
         c.author_time = 5
         c.commit_timezone = 60 * 5
         c.author_timezone = 60 * 3
-        c.author = u"Authér".encode("utf-8")
+        c.author = "Authér".encode()
         mapping = BzrGitMappingv1()
-        rev, roundtrip_revid, verifiers = mapping.import_commit(c,
-                                                                mapping.revision_id_foreign_to_bzr)
+        rev, roundtrip_revid, verifiers = mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr)
         self.assertEqual(None, roundtrip_revid)
         self.assertEqual({}, verifiers)
-        self.assertEqual(u"Authér", rev.properties[u'author'])
-        self.assertTrue(u"git-explicit-encoding" not in rev.properties)
-        self.assertTrue(u"git-implicit-encoding" not in rev.properties)
+        self.assertEqual("Authér", rev.properties['author'])
+        self.assertTrue("git-explicit-encoding" not in rev.properties)
+        self.assertTrue("git-implicit-encoding" not in rev.properties)
 
     def test_unknown_extra(self):
         c = Commit()
@@ -196,6 +229,7 @@ class TestImportCommit(tests.TestCase):
         mapping = BzrGitMappingv1()
         self.assertRaises(UnknownCommitExtra, mapping.import_commit, c,
                           mapping.revision_id_foreign_to_bzr)
+        mapping.import_commit(c, mapping.revision_id_foreign_to_bzr, strict=False)
 
     def test_mergetag(self):
         c = Commit()
@@ -218,7 +252,7 @@ class TestImportCommit(tests.TestCase):
         rev, roundtrip_revid, verifiers = mapping.import_commit(
             c, mapping.revision_id_foreign_to_bzr)
         self.assertEqual(
-            rev.properties[u'git-mergetag-0'], tag.as_raw_string())
+            rev.properties['git-mergetag-0'].encode('utf-8'), tag.as_raw_string())
 
     def test_unknown_hg_fields(self):
         c = Commit()
@@ -235,6 +269,8 @@ class TestImportCommit(tests.TestCase):
         self.assertRaises(
             UnknownMercurialCommitExtra,
             mapping.import_commit, c, mapping.revision_id_foreign_to_bzr)
+        mapping.import_commit(
+            c, mapping.revision_id_foreign_to_bzr, strict=False)
         self.assertEqual(
             mapping.revision_id_foreign_to_bzr(c.id),
             mapping.get_revision_id(c))
@@ -258,7 +294,7 @@ class TestImportCommit(tests.TestCase):
 class RoundtripRevisionsFromBazaar(tests.TestCase):
 
     def setUp(self):
-        super(RoundtripRevisionsFromBazaar, self).setUp()
+        super().setUp()
         self.mapping = BzrGitMappingv1()
         self._parent_map = {}
         self._lookup_parent = self._parent_map.__getitem__
@@ -267,7 +303,7 @@ class RoundtripRevisionsFromBazaar(tests.TestCase):
         commit = self.mapping.export_commit(orig_rev, b"mysha",
                                             self._lookup_parent, True, b"testamentsha")
         rev, roundtrip_revid, verifiers = self.mapping.import_commit(
-            commit, self.mapping.revision_id_foreign_to_bzr)
+            commit, self.mapping.revision_id_foreign_to_bzr, strict=True)
         self.assertEqual(rev.revision_id,
                          self.mapping.revision_id_foreign_to_bzr(commit.id))
         if self.mapping.roundtripping:
@@ -281,6 +317,7 @@ class RoundtripRevisionsFromBazaar(tests.TestCase):
             self.assertEqual(list(orig_rev.parent_ids), list(rev.parent_ids))
         else:
             self.assertEqual({}, verifiers)
+        return commit
 
     def test_simple_commit(self):
         r = Revision(self.mapping.revision_id_foreign_to_bzr(
@@ -305,7 +342,7 @@ class RoundtripRevisionsFromBazaar(tests.TestCase):
 
     def test_ghost_parent(self):
         r = Revision(b"myrevid")
-        r.message = u"MyCommitMessage"
+        r.message = "MyCommitMessage"
         r.parent_ids = [b"iamaghost"]
         r.committer = "Jelmer Vernooij <jelmer@apache.org>"
         r.timestamp = 453543543
@@ -315,19 +352,47 @@ class RoundtripRevisionsFromBazaar(tests.TestCase):
 
     def test_custom_property(self):
         r = Revision(b"myrevid")
-        r.message = u"MyCommitMessage"
+        r.message = "MyCommitMessage"
         r.parent_ids = []
-        r.properties = {u"fool": "bar"}
+        r.properties = {"fool": "bar"}
         r.committer = "Jelmer Vernooij <jelmer@apache.org>"
         r.timestamp = 453543543
         r.timezone = 0
         self.assertRoundtripRevision(r)
 
+    def test_multiple_authors(self):
+        r = Revision(b"myrevid")
+        r.message = "MyCommitMessage"
+        r.parent_ids = []
+        r.properties = {
+            "authors":
+                "Jelmer Vernooij <jelmer@jelmer.uk>\n"
+                "Alex <alexa@example.com>"}
+        r.committer = "Jelmer Vernooij <jelmer@apache.org>"
+        r.timestamp = 453543543
+        r.timezone = 0
+        c = self.assertRoundtripRevision(r)
+        self.assertEqual(c.author, b'Jelmer Vernooij <jelmer@jelmer.uk>')
+
+    def test_multiple_authors_comma(self):
+        r = Revision(b"myrevid")
+        r.message = "MyCommitMessage"
+        r.parent_ids = []
+        r.properties = {
+            "authors":
+                "Jelmer Vernooij <jelmer@jelmer.uk>, "
+                "Alex <alexa@example.com>"}
+        r.committer = "Jelmer Vernooij <jelmer@apache.org>"
+        r.timestamp = 453543543
+        r.timezone = 0
+        c = self.assertRoundtripRevision(r)
+        self.assertEqual(c.author, b'Jelmer Vernooij <jelmer@jelmer.uk>')
+
 
 class RoundtripRevisionsFromGit(tests.TestCase):
 
     def setUp(self):
-        super(RoundtripRevisionsFromGit, self).setUp()
+        super().setUp()
         self.mapping = BzrGitMappingv1()
 
     def assertRoundtripTree(self, tree):
@@ -338,7 +403,7 @@ class RoundtripRevisionsFromGit(tests.TestCase):
 
     def assertRoundtripCommit(self, commit1):
         rev, roundtrip_revid, verifiers = self.mapping.import_commit(
-            commit1, self.mapping.revision_id_foreign_to_bzr)
+            commit1, self.mapping.revision_id_foreign_to_bzr, strict=True)
         commit2 = self.mapping.export_commit(rev, "12341212121212", None,
                                              True, None)
         self.assertEqual(commit1.committer, commit2.committer)
@@ -442,9 +507,14 @@ class FixPersonIdentifierTests(tests.TestCase):
                          fix_person_identifier(b"bar@blah.nl"))
 
     def test_fix(self):
-        self.assertEqual(b"person <bar@blah.nl>",
-                         fix_person_identifier(b"somebody <person <bar@blah.nl>>"))
-        self.assertEqual(b"person <bar@blah.nl>",
-                         fix_person_identifier(b"person<bar@blah.nl>"))
+        self.assertEqual(
+            b"person <bar@blah.nl>",
+            fix_person_identifier(b"somebody <person <bar@blah.nl>>"))
+        self.assertEqual(
+            b"person <bar@blah.nl>",
+            fix_person_identifier(b"person<bar@blah.nl>"))
+        self.assertEqual(
+            b'Rohan Garg <rohangarg@kubuntu.org>',
+            fix_person_identifier(b'Rohan Garg <rohangarg@kubuntu.org'))
         self.assertRaises(ValueError,
                           fix_person_identifier, b"person >bar@blah.nl<")
