@@ -17,12 +17,15 @@
 
 """Tests for Git working trees."""
 
-from __future__ import absolute_import
-
 import os
 import stat
 
+from dulwich import __version__ as dulwich_version
+from dulwich.diff_tree import RenameDetector, tree_changes
 from dulwich.index import IndexEntry
+from dulwich.object_store import (
+    OverlayObjectStore,
+    )
 from dulwich.objects import (
     S_IFGITLINK,
     Blob,
@@ -35,12 +38,11 @@ from ... import (
     workingtree as _mod_workingtree,
     )
 from ...delta import TreeDelta
-from ...tree import TreeChange
+from ...bzr.inventorytree import InventoryTreeChange as TreeChange
 from ..mapping import (
     default_mapping,
     )
 from ..tree import (
-    changes_between_git_tree_and_working_copy,
     tree_delta_from_git_changes,
     )
 from ..workingtree import (
@@ -52,10 +54,26 @@ from ...tests import (
     )
 
 
+def changes_between_git_tree_and_working_copy(source_store, from_tree_sha, target,
+                                              want_unchanged=False,
+                                              want_unversioned=False,
+                                              rename_detector=None,
+                                              include_trees=True):
+    """Determine the changes between a git tree and a working tree with index.
+
+    """
+    to_tree_sha, extras = target.git_snapshot(want_unversioned=want_unversioned)
+    store = OverlayObjectStore([source_store, target.store])
+    return tree_changes(
+        store, from_tree_sha, to_tree_sha, include_trees=include_trees,
+        rename_detector=rename_detector,
+        want_unchanged=want_unchanged, change_type_same=True), extras
+
+
 class GitWorkingTreeTests(TestCaseWithTransport):
 
     def setUp(self):
-        super(GitWorkingTreeTests, self).setUp()
+        super().setUp()
         self.tree = self.make_branch_and_tree('.', format="git")
 
     def test_conflict_list(self):
@@ -67,8 +85,8 @@ class GitWorkingTreeTests(TestCaseWithTransport):
         self.build_tree(['conflicted'])
         self.tree.add(['conflicted'])
         with self.tree.lock_tree_write():
-            self.tree.index[b'conflicted'] = self.tree.index[b'conflicted'][:9] + \
-                (FLAG_STAGEMASK, )
+            self.tree.index[b'conflicted'] = self.tree.index[b'conflicted']._replace(
+                flags=FLAG_STAGEMASK)
             self.tree._index_dirty = True
         conflicts = self.tree.conflicts()
         self.assertEqual(1, len(conflicts))
@@ -118,7 +136,7 @@ class GitWorkingTreeTests(TestCaseWithTransport):
 class GitWorkingTreeFileTests(TestCaseWithTransport):
 
     def setUp(self):
-        super(GitWorkingTreeFileTests, self).setUp()
+        super().setUp()
         self.tree = self.make_branch_and_tree('actual', format="git")
         self.build_tree_contents(
             [('linked/',), ('linked/.git', 'gitdir: ../actual/.git')])
@@ -141,9 +159,15 @@ class TreeDeltaFromGitChangesTests(TestCase):
 
     def test_missing(self):
         delta = TreeDelta()
-        delta.removed.append(TreeChange(b'git:a', ('a', 'a'), False, (True, True), (b'TREE_ROOT', b'TREE_ROOT'), ('a', 'a'), ('file', None), (True, False)))
-        changes = [((b'a', b'a'), (stat.S_IFREG | 0o755, 0),
-                    (b'a' * 40, b'a' * 40))]
+        delta.removed.append(
+            TreeChange(
+                b'git:a', ('a', 'a'), False, (True, True),
+                (b'TREE_ROOT', b'TREE_ROOT'), ('a', 'a'), ('file', None),
+                (True, False)))
+        changes = [
+            ('remove',
+                (b'a', stat.S_IFREG | 0o755, b'a' * 40),
+                (b'a', 0, b'a' * 40))]
         self.assertEqual(
             delta,
             tree_delta_from_git_changes(changes, (default_mapping, default_mapping)))
@@ -152,13 +176,13 @@ class TreeDeltaFromGitChangesTests(TestCase):
 class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
 
     def setUp(self):
-        super(ChangesBetweenGitTreeAndWorkingCopyTests, self).setUp()
+        super().setUp()
         self.wt = self.make_branch_and_tree('.', format='git')
         self.store = self.wt.branch.repository._git.object_store
 
     def expectDelta(self, expected_changes,
                     expected_extras=None, want_unversioned=False,
-                    tree_id=None):
+                    tree_id=None, rename_detector=None):
         if tree_id is None:
             try:
                 tree_id = self.store[self.wt.branch.repository._git.head()].tree
@@ -166,7 +190,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
                 tree_id = None
         with self.wt.lock_read():
             changes, extras = changes_between_git_tree_and_working_copy(
-                self.store, tree_id, self.wt, want_unversioned=want_unversioned)
+                self.store, tree_id, self.wt, want_unversioned=want_unversioned,
+                rename_detector=rename_detector)
             self.assertEqual(expected_changes, list(changes))
         if expected_extras is None:
             expected_extras = set()
@@ -174,7 +199,7 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
 
     def test_empty(self):
         self.expectDelta(
-            [((None, b''), (None, stat.S_IFDIR), (None, Tree().id))])
+            [('add', (None, None, None), (b'', stat.S_IFDIR, Tree().id))])
 
     def test_added_file(self):
         self.build_tree(['a'])
@@ -183,20 +208,74 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         t = Tree()
         t.add(b"a", stat.S_IFREG | 0o644, a.id)
         self.expectDelta(
-            [((None, b''), (None, stat.S_IFDIR), (None, t.id)),
-             ((None, b'a'), (None, stat.S_IFREG | 0o644), (None, a.id))])
+            [('add', (None, None, None), (b'', stat.S_IFDIR, t.id)),
+             ('add', (None, None, None), (b'a', stat.S_IFREG | 0o644, a.id))])
+
+    def test_renamed_file(self):
+        self.build_tree(['a'])
+        self.wt.add(['a'])
+        self.wt.rename_one('a', 'b')
+        a = Blob.from_string(b'contents of a\n')
+        self.store.add_object(a)
+        oldt = Tree()
+        oldt.add(b"a", stat.S_IFREG | 0o644, a.id)
+        self.store.add_object(oldt)
+        newt = Tree()
+        newt.add(b"b", stat.S_IFREG | 0o644, a.id)
+        self.store.add_object(newt)
+        self.expectDelta(
+            [('modify', (b'', stat.S_IFDIR, oldt.id), (b'', stat.S_IFDIR, newt.id)),
+             ('delete', (b'a', stat.S_IFREG | 0o644, a.id), (None, None, None)),
+             ('add', (None, None, None), (b'b', stat.S_IFREG | 0o644, a.id)),
+             ],
+            tree_id=oldt.id)
+        if dulwich_version >= (0, 19, 15):
+            self.expectDelta(
+                [('modify', (b'', stat.S_IFDIR, oldt.id), (b'', stat.S_IFDIR, newt.id)),
+                 ('rename', (b'a', stat.S_IFREG | 0o644, a.id), (b'b', stat.S_IFREG | 0o644, a.id))],
+                tree_id=oldt.id, rename_detector=RenameDetector(self.store))
+
+    def test_copied_file(self):
+        self.build_tree(['a'])
+        self.wt.add(['a'])
+        self.wt.copy_one('a', 'b')
+        a = Blob.from_string(b'contents of a\n')
+        self.store.add_object(a)
+        oldt = Tree()
+        oldt.add(b"a", stat.S_IFREG | 0o644, a.id)
+        self.store.add_object(oldt)
+        newt = Tree()
+        newt.add(b"a", stat.S_IFREG | 0o644, a.id)
+        newt.add(b"b", stat.S_IFREG | 0o644, a.id)
+        self.store.add_object(newt)
+        self.expectDelta(
+            [('modify', (b'', stat.S_IFDIR, oldt.id), (b'', stat.S_IFDIR, newt.id)),
+             ('add', (None, None, None), (b'b', stat.S_IFREG | 0o644, a.id)),
+             ],
+            tree_id=oldt.id)
+
+        if dulwich_version >= (0, 19, 15):
+            self.expectDelta(
+                [('modify', (b'', stat.S_IFDIR, oldt.id), (b'', stat.S_IFDIR, newt.id)),
+                 ('copy', (b'a', stat.S_IFREG | 0o644, a.id), (b'b', stat.S_IFREG | 0o644, a.id))],
+                tree_id=oldt.id, rename_detector=RenameDetector(self.store, find_copies_harder=True))
+            self.expectDelta(
+                [('modify', (b'', stat.S_IFDIR, oldt.id), (b'', stat.S_IFDIR, newt.id)),
+                 ('add', (None, None, None), (b'b', stat.S_IFREG | 0o644, a.id)),
+                 ],
+                tree_id=oldt.id, rename_detector=RenameDetector(self.store, find_copies_harder=False))
 
     def test_added_unknown_file(self):
         self.build_tree(['a'])
         t = Tree()
         self.expectDelta(
-            [((None, b''), (None, stat.S_IFDIR), (None, t.id))])
+            [('add', (None, None, None), (b'', stat.S_IFDIR, t.id))])
         a = Blob.from_string(b'contents of a\n')
         t = Tree()
         t.add(b"a", stat.S_IFREG | 0o644, a.id)
         self.expectDelta(
-            [((None, b''), (None, stat.S_IFDIR), (None, t.id)),
-             ((None, b'a'), (None, stat.S_IFREG | 0o644), (None, a.id))],
+            [('add', (None, None, None), (b'', stat.S_IFDIR, t.id)),
+             ('add', (None, None, None), (b'a', stat.S_IFREG | 0o644, a.id))],
             [b'a'],
             want_unversioned=True)
 
@@ -208,8 +287,8 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         t = Tree()
         t.add(b"a", 0, ZERO_SHA)
         self.expectDelta(
-            [((None, b''), (None, stat.S_IFDIR), (None, t.id)),
-             ((None, b'a'), (None, 0), (None, ZERO_SHA))],
+            [('add', (None, None, None), (b'', stat.S_IFDIR, t.id)),
+             ('add', (None, None, None), (b'a', 0, ZERO_SHA))],
             [])
 
     def test_missing_versioned_file(self):
@@ -223,8 +302,12 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newt = Tree()
         newt.add(b"a", 0, ZERO_SHA)
         self.expectDelta(
-            [((b'', b''), (stat.S_IFDIR, stat.S_IFDIR), (oldt.id, newt.id)),
-             ((b'a', b'a'), (stat.S_IFREG | 0o644, 0), (a.id, ZERO_SHA))])
+            [('modify',
+                (b'', stat.S_IFDIR, oldt.id),
+                (b'', stat.S_IFDIR, newt.id)),
+             ('modify',
+                 (b'a', stat.S_IFREG | 0o644, a.id),
+                 (b'a', 0, ZERO_SHA))])
 
     def test_versioned_replace_by_dir(self):
         self.build_tree(['a'])
@@ -239,16 +322,20 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newa = Tree()
         newt.add(b"a", stat.S_IFDIR, newa.id)
         self.expectDelta([
-            ((b'', b''),
-             (stat.S_IFDIR, stat.S_IFDIR),
-             (oldt.id, newt.id)),
-            ((b'a', b'a'), (stat.S_IFREG | 0o644, stat.S_IFDIR), (olda.id, newa.id))
+            ('modify',
+                (b'', stat.S_IFDIR, oldt.id),
+                (b'', stat.S_IFDIR, newt.id)),
+            ('modify',
+                (b'a', stat.S_IFREG | 0o644, olda.id),
+                (b'a', stat.S_IFDIR, newa.id))
             ], want_unversioned=False)
         self.expectDelta([
-            ((b'', b''),
-             (stat.S_IFDIR, stat.S_IFDIR),
-             (oldt.id, newt.id)),
-            ((b'a', b'a'), (stat.S_IFREG | 0o644, stat.S_IFDIR), (olda.id, newa.id))
+            ('modify',
+                (b'', stat.S_IFDIR, oldt.id),
+                (b'', stat.S_IFDIR, newt.id)),
+            ('modify',
+                (b'a', stat.S_IFREG | 0o644, olda.id),
+                (b'a', stat.S_IFDIR, newa.id)),
             ], want_unversioned=True)
 
     def test_extra(self):
@@ -257,10 +344,12 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         newt = Tree()
         newt.add(b"a", stat.S_IFREG | 0o644, newa.id)
         self.expectDelta([
-            ((None, b''),
-             (None, stat.S_IFDIR),
-             (None, newt.id)),
-            ((None, b'a'), (None, stat.S_IFREG | 0o644), (None, newa.id))
+            ('add',
+                (None, None, None),
+                (b'', stat.S_IFDIR, newt.id)),
+            ('add',
+                (None, None, None),
+                (b'a', stat.S_IFREG | 0o644, newa.id)),
             ], [b'a'], want_unversioned=True)
 
     def test_submodule(self):
@@ -269,7 +358,7 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         self.build_tree_contents([('a/.git/HEAD', a.id)])
         with self.wt.lock_tree_write():
             (index, index_path) = self.wt._lookup_index(b'a')
-            index[b'a'] = IndexEntry(0, 0, 0, 0, S_IFGITLINK, 0, 0, 0, a.id, 0)
+            index[b'a'] = IndexEntry(0, 0, 0, 0, S_IFGITLINK, 0, 0, 0, a.id, 0, 0)
             self.wt._index_dirty = True
         t = Tree()
         t.add(b"a", S_IFGITLINK, a.id)
@@ -280,7 +369,7 @@ class ChangesBetweenGitTreeAndWorkingCopyTests(TestCaseWithTransport):
         a = Blob.from_string(b'irrelevant\n')
         with self.wt.lock_tree_write():
             (index, index_path) = self.wt._lookup_index(b'a')
-            index[b'a'] = IndexEntry(0, 0, 0, 0, S_IFGITLINK, 0, 0, 0, a.id, 0)
+            index[b'a'] = IndexEntry(0, 0, 0, 0, S_IFGITLINK, 0, 0, 0, a.id, 0, 0)
             self.wt._index_dirty = True
         os.mkdir(self.wt.abspath('a'))
         t = Tree()
