@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
-
 import time
 
 from . import (
@@ -25,10 +23,6 @@ from . import (
     revision,
     trace,
     )
-from .sixish import (
-    viewitems,
-    viewvalues,
-    )
 
 STEP_UNIQUE_SEARCHER_EVERY = 5
 
@@ -55,7 +49,7 @@ STEP_UNIQUE_SEARCHER_EVERY = 5
 # 2. Since len(['D', 'E']) > 1, find_lca('D', 'E') => ['A']
 
 
-class DictParentsProvider(object):
+class DictParentsProvider:
     """A parents provider for Graph objects."""
 
     def __init__(self, ancestry):
@@ -72,10 +66,10 @@ class DictParentsProvider(object):
     def get_parent_map(self, keys):
         """See StackedParentsProvider.get_parent_map"""
         ancestry = self.ancestry
-        return dict([(k, ancestry[k]) for k in keys if k in ancestry])
+        return {k: ancestry[k] for k in keys if k in ancestry}
 
 
-class StackedParentsProvider(object):
+class StackedParentsProvider:
     """A parents provider which stacks (or unions) multiple providers.
 
     The providers are queries in the order of the provided parent_providers.
@@ -85,7 +79,7 @@ class StackedParentsProvider(object):
         self._parent_providers = parent_providers
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self._parent_providers)
+        return "{}({!r})".format(self.__class__.__name__, self._parent_providers)
 
     def get_parent_map(self, keys):
         """Get a mapping of keys => parents
@@ -130,7 +124,7 @@ class StackedParentsProvider(object):
         return found
 
 
-class CachingParentsProvider(object):
+class CachingParentsProvider:
     """A parents provider which will cache the revision => parents as a dict.
 
     This is useful for providers which have an expensive look up.
@@ -159,7 +153,7 @@ class CachingParentsProvider(object):
         self.enable_cache(True)
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self._real_provider)
+        return "{}({!r})".format(self.__class__.__name__, self._real_provider)
 
     def enable_cache(self, cache_misses=True):
         """Enable cache."""
@@ -190,7 +184,7 @@ class CachingParentsProvider(object):
         cache = self._cache
         if cache is None:
             return {}
-        return dict([(key, cache[key]) for key in keys if key in cache])
+        return {key: cache[key] for key in keys if key in cache}
 
     def get_parent_map(self, keys):
         """See StackedParentsProvider.get_parent_map."""
@@ -198,7 +192,7 @@ class CachingParentsProvider(object):
         if cache is None:
             cache = self._get_parent_map(keys)
         else:
-            needed_revisions = set(key for key in keys if key not in cache)
+            needed_revisions = {key for key in keys if key not in cache}
             # Do not ask for negatively cached keys
             needed_revisions.difference_update(self.missing_keys)
             if needed_revisions:
@@ -221,7 +215,7 @@ class CachingParentsProvider(object):
             self.missing_keys.add(key)
 
 
-class CallableToParentsProviderAdapter(object):
+class CallableToParentsProviderAdapter:
     """A parents provider that adapts any callable to the parents provider API.
 
     i.e. it accepts calls to self.get_parent_map and relays them to the
@@ -232,13 +226,13 @@ class CallableToParentsProviderAdapter(object):
         self.callable = a_callable
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.callable)
+        return "{}({!r})".format(self.__class__.__name__, self.callable)
 
     def get_parent_map(self, keys):
         return self.callable(keys)
 
 
-class Graph(object):
+class Graph:
     """Provide incremental access to revision graphs.
 
     This is the generic implementation; it is intended to be subclassed to
@@ -343,7 +337,7 @@ class Graph(object):
         """
         parent_map = self._parents_provider.get_parent_map(keys)
         parent_child = {}
-        for child, parents in sorted(viewitems(parent_map)):
+        for child, parents in sorted(parent_map.items()):
             for parent in parents:
                 parent_child.setdefault(parent, []).append(child)
         return parent_child
@@ -654,7 +648,7 @@ class Graph(object):
         # TODO: it might be possible to collapse searchers faster when they
         #       only have *some* search tips in common.
         next_unique_searchers = []
-        for searchers in viewvalues(unique_search_tips):
+        for searchers in unique_search_tips.values():
             if len(searchers) == 1:
                 # Searching unique tips, go for it
                 next_unique_searchers.append(searchers[0])
@@ -728,7 +722,7 @@ class Graph(object):
                                  all_unique_searcher._iterations)
             unique_tip_searchers = next_unique_searchers
 
-    def get_parent_map(self, revisions):
+    def get_parent_map(self, revisions):   # type: ignore
         """Get a map of key:parent_list for revisions.
 
         This implementation delegates to get_parents, for old parent_providers
@@ -839,11 +833,11 @@ class Graph(object):
                 return {revision.NULL_REVISION}
         if len(candidate_heads) < 2:
             return candidate_heads
-        searchers = dict((c, self._make_breadth_first_searcher([c]))
-                         for c in candidate_heads)
+        searchers = {c: self._make_breadth_first_searcher([c])
+                         for c in candidate_heads}
         active_searchers = dict(searchers)
         # skip over the actual candidate for each searcher
-        for searcher in viewvalues(active_searchers):
+        for searcher in active_searchers.values():
             next(searcher)
         # The common walker finds nodes that are common to two or more of the
         # input keys, so that we don't access all history when a currently
@@ -886,11 +880,11 @@ class Graph(object):
                     # some searcher has encountered our known common nodes:
                     # just stop it
                     ancestor_set = {ancestor}
-                    for searcher in viewvalues(searchers):
+                    for searcher in searchers.values():
                         searcher.stop_searching_any(ancestor_set)
                 else:
                     # or it may have been just reached by all the searchers:
-                    for searcher in viewvalues(searchers):
+                    for searcher in searchers.values():
                         if ancestor not in searcher.seen:
                             break
                     else:
@@ -898,7 +892,7 @@ class Graph(object):
                         # making it be known as a descendant of all candidates,
                         # so we can stop searching it, and any seen ancestors
                         new_common.add(ancestor)
-                        for searcher in viewvalues(searchers):
+                        for searcher in searchers.values():
                             seen_ancestors =\
                                 searcher.find_seen_ancestors([ancestor])
                             searcher.stop_searching_any(seen_ancestors)
@@ -1021,7 +1015,7 @@ class Graph(object):
             processed.update(pending)
             next_map = self.get_parent_map(pending)
             next_pending = set()
-            for item in viewitems(next_map):
+            for item in next_map.items():
                 yield item
                 next_pending.update(p for p in item[1] if p not in processed)
             ghosts = pending.difference(next_map)
@@ -1259,7 +1253,7 @@ class Graph(object):
         # for revision in revisions.intersection(descendants):
         # simple_ancestors.difference_update(descendants[revision])
         # return simple_ancestors
-        for revision, parent_ids in viewitems(parent_map):
+        for revision, parent_ids in parent_map.items():
             if parent_ids is None:
                 continue
             for parent_id in parent_ids:
@@ -1271,7 +1265,7 @@ class Graph(object):
         return simple_ancestors
 
 
-class HeadsCache(object):
+class HeadsCache:
     """A cache of results for graph heads calls."""
 
     def __init__(self, graph):
@@ -1299,7 +1293,7 @@ class HeadsCache(object):
             return set(heads)
 
 
-class FrozenHeadsCache(object):
+class FrozenHeadsCache:
     """Cache heads() calls, assuming the caller won't modify them."""
 
     def __init__(self, graph):
@@ -1329,7 +1323,7 @@ class FrozenHeadsCache(object):
         self._heads[frozenset(keys)] = frozenset(heads)
 
 
-class _BreadthFirstSearcher(object):
+class _BreadthFirstSearcher:
     """Parallel search breadth-first the ancestry of revisions.
 
     This class implements the iterator protocol, but additionally
@@ -1354,7 +1348,7 @@ class _BreadthFirstSearcher(object):
             prefix = "searching"
         else:
             prefix = "starting"
-        search = '%s=%r' % (prefix, list(self._next_query))
+        search = '{}={!r}'.format(prefix, list(self._next_query))
         return ('_BreadthFirstSearcher(iterations=%d, %s,'
                 ' seen=%r)' % (self._iterations, search, list(self.seen)))
 
@@ -1466,7 +1460,7 @@ class _BreadthFirstSearcher(object):
         seen.update(revisions)
         parent_map = self._parents_provider.get_parent_map(revisions)
         found_revisions.update(parent_map)
-        for rev_id, parents in viewitems(parent_map):
+        for rev_id, parents in parent_map.items():
             if parents is None:
                 continue
             new_found_parents = [p for p in parents if p not in seen]
@@ -1509,7 +1503,7 @@ class _BreadthFirstSearcher(object):
             all_parents = []
             # We don't care if it is a ghost, since it can't be seen if it is
             # a ghost
-            for parent_ids in viewvalues(parent_map):
+            for parent_ids in parent_map.values():
                 all_parents.extend(parent_ids)
             next_pending = all_seen.intersection(
                 all_parents).difference(seen_ancestors)
@@ -1555,14 +1549,14 @@ class _BreadthFirstSearcher(object):
                     stop_rev_references[parent_id] += 1
             # if only the stopped revisions reference it, the ref count will be
             # 0 after this loop
-            for parents in viewvalues(self._current_parents):
+            for parents in self._current_parents.values():
                 for parent_id in parents:
                     try:
                         stop_rev_references[parent_id] -= 1
                     except KeyError:
                         pass
             stop_parents = set()
-            for rev_id, refs in viewitems(stop_rev_references):
+            for rev_id, refs in stop_rev_references.items():
                 if refs == 0:
                     stop_parents.add(rev_id)
             self._next_query.difference_update(stop_parents)
@@ -1598,7 +1592,7 @@ class _BreadthFirstSearcher(object):
 def invert_parent_map(parent_map):
     """Given a map from child => parents, create a map of parent=>children"""
     child_map = {}
-    for child, parents in viewitems(parent_map):
+    for child, parents in parent_map.items():
         for p in parents:
             # Any given parent is likely to have only a small handful
             # of children, many will have only one. So we avoid mem overhead of
@@ -1650,7 +1644,7 @@ def collapse_linear_regions(parent_map):
     # Will not have any nodes removed, even though you do have an
     # 'uninteresting' linear D->B and E->C
     children = {}
-    for child, parents in viewitems(parent_map):
+    for child, parents in parent_map.items():
         children.setdefault(child, [])
         for p in parents:
             children.setdefault(p, []).append(child)
@@ -1682,7 +1676,7 @@ def collapse_linear_regions(parent_map):
     return result
 
 
-class GraphThunkIdsToKeys(object):
+class GraphThunkIdsToKeys:
     """Forwards calls about 'ids' to be about keys internally."""
 
     def __init__(self, graph):
