@@ -47,7 +47,7 @@ class _RevisionNotLinear(Exception):
     """Raised when a revision is not on left-hand history."""
 
 
-class GrepOptions(object):
+class GrepOptions:
     """Container to pass around grep options.
 
     This class is used as a container to pass around user option and
@@ -164,7 +164,7 @@ def is_fixed_string(s):
     return False
 
 
-class _GrepDiffOutputter(object):
+class _GrepDiffOutputter:
     """Precalculate formatting based on options given for diff grep.
     """
 
@@ -331,14 +331,14 @@ def grep_diff(opts):
                     display_file = True
                 elif diff_pattern.search(line):
                     if display_revno:
-                        writerevno("=== revno:%s ===" % (revno,))
+                        writerevno("=== revno:{} ===".format(revno))
                         display_revno = False
                     if display_file:
                         writefileheader(
-                            "  %s" % (file_header.decode(file_encoding, 'replace'),))
+                            "  {}".format(file_header.decode(file_encoding, 'replace')))
                         display_file = False
                     line = line.decode(file_encoding, 'replace')
-                    writeline("    %s" % (line,))
+                    writeline("    {}".format(line))
 
 
 def versioned_grep(opts):
@@ -563,7 +563,7 @@ def _file_grep_list_only_wtree(file, path, opts, path_prefix=None):
         opts.outputter.get_writer(path, None, None)()
 
 
-class _Outputter(object):
+class _Outputter:
     """Precalculate formatting based on options given
 
     The idea here is to do this work only once per run, and finally return a
