@@ -19,8 +19,6 @@
 See MemoryTree for more details.
 """
 
-from __future__ import absolute_import
-
 import os
 import stat
 
@@ -28,6 +26,7 @@ from . import (
     errors,
     lock,
     revision as _mod_revision,
+    transport as _mod_transport,
     )
 from .bzr.inventory import Inventory
 from .bzr.inventorytree import MutableInventoryTree
@@ -64,7 +63,7 @@ class MemoryTree(MutableInventoryTree):
         # Memory tree doesn't have any control filenames
         return False
 
-    def _add(self, files, ids, kinds):
+    def _add(self, files, kinds, ids):
         """See MutableTree._add."""
         with self.lock_tree_write():
             for f, file_id, kind in zip(files, ids, kinds):
@@ -90,8 +89,7 @@ class MemoryTree(MutableInventoryTree):
     @staticmethod
     def create_on_branch(branch):
         """Create a MemoryTree for branch, using the last-revision of branch."""
-        revision_id = _mod_revision.ensure_null(branch.last_revision())
-        return MemoryTree(branch, revision_id)
+        return MemoryTree(branch, branch.last_revision())
 
     def _gather_kinds(self, files, kinds):
         """See MutableTree._gather_kinds.
@@ -164,7 +162,7 @@ class MemoryTree(MutableInventoryTree):
 
     def mkdir(self, path, file_id=None):
         """See MutableTree.mkdir()."""
-        self.add(path, file_id, 'directory')
+        self.add(path, 'directory', file_id)
         if file_id is None:
             file_id = self.path2id(path)
         self._file_transport.mkdir(path)
@@ -285,7 +283,7 @@ class MemoryTree(MutableInventoryTree):
             for path in paths:
                 file_id = self.path2id(path)
                 if file_id is None:
-                    raise errors.NoSuchFile(path)
+                    raise _mod_transport.NoSuchFile(path)
                 file_ids.add(file_id)
             for file_id in file_ids:
                 if self._inventory.has_id(file_id):
