@@ -26,14 +26,12 @@ from breezy import (
     diff,
     email_message,
     gpg,
-    hooks,
-    registry,
     revision as _mod_revision,
-    rio,
     timestamp,
     trace,
     )
 from breezy.bzr import (
+    rio,
     testament,
     )
 from breezy.bzr.bundle import (
@@ -42,6 +40,8 @@ from breezy.bzr.bundle import (
 """)
 from . import (
     errors,
+    hooks,
+    registry,
     )
 
 
@@ -55,7 +55,7 @@ class IllegalMergeDirectivePayload(errors.BzrError):
         self.start = start
 
 
-class MergeRequestBodyParams(object):
+class MergeRequestBodyParams:
     """Parameter object for the merge_request_body hook."""
 
     def __init__(self, body, orig_body, directive, to, basename, subject,
@@ -84,7 +84,7 @@ class MergeDirectiveHooks(hooks.Hooks):
             " provided to the next.", (1, 15, 0))
 
 
-class BaseMergeDirective(object):
+class BaseMergeDirective:
     """A request to perform a merge into a branch.
 
     This is the base class that all merge directive implementations
@@ -208,7 +208,6 @@ class BaseMergeDirective(object):
             patch = None
         else:
             submit_revision_id = submit_branch.last_revision()
-            submit_revision_id = _mod_revision.ensure_null(submit_revision_id)
             repository.fetch(submit_branch.repository, submit_revision_id)
             graph = repository.get_graph()
             ancestor_id = graph.find_unique_lca(revision_id,
@@ -243,7 +242,7 @@ class BaseMergeDirective(object):
             except errors.NoSuchRevision:
                 revno = ['merge']
         nick = re.sub('(\\W+)', '-', branch.nick).strip('-')
-        return '%s-%s' % (nick, '.'.join(str(n) for n in revno))
+        return '{}-{}'.format(nick, '.'.join(str(n) for n in revno))
 
     @staticmethod
     def _generate_diff(repository, revision_id, ancestor_id):
@@ -311,8 +310,8 @@ class BaseMergeDirective(object):
                     except errors.NotBranchError:
                         raise errors.TargetNotBranch(self.target_branch)
                     missing_revisions = []
-                    bundle_revisions = set(r.revision_id for r in
-                                           info.real_revisions)
+                    bundle_revisions = {r.revision_id for r in
+                                           info.real_revisions}
                     for revision in info.real_revisions:
                         for parent_id in revision.parent_ids:
                             if (parent_id not in bundle_revisions
@@ -620,7 +619,6 @@ class MergeDirective2(BaseMergeDirective):
             if submit_branch.get_public_branch() is not None:
                 target_branch = submit_branch.get_public_branch()
             submit_revision_id = submit_branch.last_revision()
-            submit_revision_id = _mod_revision.ensure_null(submit_revision_id)
             graph = repository.get_graph(submit_branch.repository)
             ancestor_id = graph.find_unique_lca(revision_id,
                                                 submit_revision_id)
