@@ -15,8 +15,6 @@
 
 """An abstraction of a repository providing just the bits importing needs."""
 
-from __future__ import absolute_import
-
 from io import BytesIO
 
 from ... import (
@@ -25,12 +23,13 @@ from ... import (
     osutils,
     revision as _mod_revision,
     )
+from ...bzr.inventorytree import InventoryTreeChange
 from ...bzr import (
     inventory,
     )
 
 
-class _TreeShim(object):
+class _TreeShim:
     """Fake a Tree implementation.
 
     This implements just enough of the tree api to make commit builder happy.
@@ -41,12 +40,12 @@ class _TreeShim(object):
         self._content_provider = content_provider
         self._basis_inv = basis_inv
         self._inv_delta = inv_delta
-        self._new_info_by_id = dict([(file_id, (new_path, ie))
-                                    for _, new_path, file_id, ie in inv_delta])
+        self._new_info_by_id = {file_id: (new_path, ie)
+                                    for _, new_path, file_id, ie in inv_delta}
         self._new_info_by_path = {new_path: ie
                                   for _, new_path, file_id, ie in inv_delta}
 
-    def id2path(self, file_id):
+    def id2path(self, file_id, recurse='down'):
         if file_id in self._new_info_by_id:
             new_path = self._new_info_by_id[file_id][0]
             if new_path is None:
@@ -118,53 +117,57 @@ class _TreeShim(object):
                 old_ie = None
                 if ie is None:
                     raise AssertionError('How is both old and new None?')
-                    change = (file_id,
-                              (old_path, new_path),
-                              False,
-                              (False, False),
-                              (None, None),
-                              (None, None),
-                              (None, None),
-                              (None, None),
-                              )
-                change = (file_id,
-                          (old_path, new_path),
-                          True,
-                          (False, True),
-                          (None, ie.parent_id),
-                          (None, ie.name),
-                          (None, ie.kind),
-                          (None, ie.executable),
-                          )
+                    change = InventoryTreeChange(
+                        file_id,
+                        (old_path, new_path),
+                        False,
+                        (False, False),
+                        (None, None),
+                        (None, None),
+                        (None, None),
+                        (None, None),
+                        )
+                change = InventoryTreeChange(
+                    file_id,
+                    (old_path, new_path),
+                    True,
+                    (False, True),
+                    (None, ie.parent_id),
+                    (None, ie.name),
+                    (None, ie.kind),
+                    (None, ie.executable),
+                    )
             else:
                 if ie is None:
-                    change = (file_id,
-                              (old_path, new_path),
-                              True,
-                              (True, False),
-                              (old_ie.parent_id, None),
-                              (old_ie.name, None),
-                              (old_ie.kind, None),
-                              (old_ie.executable, None),
-                              )
+                    change = InventoryTreeChange(
+                        file_id,
+                        (old_path, new_path),
+                        True,
+                        (True, False),
+                        (old_ie.parent_id, None),
+                        (old_ie.name, None),
+                        (old_ie.kind, None),
+                        (old_ie.executable, None),
+                        )
                 else:
                     content_modified = (ie.text_sha1 != old_ie.text_sha1 or
                                         ie.text_size != old_ie.text_size)
                     # TODO: ie.kind != old_ie.kind
                     # TODO: symlinks changing targets, content_modified?
-                    change = (file_id,
-                              (old_path, new_path),
-                              content_modified,
-                              (True, True),
-                              (old_ie.parent_id, ie.parent_id),
-                              (old_ie.name, ie.name),
-                              (old_ie.kind, ie.kind),
-                              (old_ie.executable, ie.executable),
-                              )
+                    change = InventoryTreeChange(
+                        file_id,
+                        (old_path, new_path),
+                        content_modified,
+                        (True, True),
+                        (old_ie.parent_id, ie.parent_id),
+                        (old_ie.name, ie.name),
+                        (old_ie.kind, ie.kind),
+                        (old_ie.executable, ie.executable),
+                        )
             yield change
 
 
-class RevisionStore(object):
+class RevisionStore:
 
     def __init__(self, repo):
         """An object responsible for loading revisions into a repository.
@@ -222,17 +225,10 @@ class RevisionStore(object):
         """Get a stored inventory."""
         return self.repo.get_inventory(revision_id)
 
-    def get_file_text(self, revision_id, file_id):
-        """Get the text stored for a file in a given revision."""
-        revtree = self.repo.revision_tree(revision_id)
-        path = revtree.id2path(file_id)
-        return revtree.get_file_text(path)
-
-    def get_file_lines(self, revision_id, file_id):
+    def get_file_lines(self, revision_id, path):
         """Get the lines stored for a file in a given revision."""
         revtree = self.repo.revision_tree(revision_id)
-        path = revtree.id2path(file_id)
-        return osutils.split_lines(revtree.get_file_text(path))
+        return revtree.get_file_lines(path)
 
     def start_new_revision(self, revision, parents, parent_invs):
         """Init the metadata needed for get_parents_and_revision_for_entry().
@@ -250,10 +246,11 @@ class RevisionStore(object):
         # We can't use self.repo.get_commit_builder() here because it starts a
         # new write group. We want one write group around a batch of imports
         # where the default batch size is currently 10000. IGC 20090312
-        self._commit_builder = self.repo._commit_builder_class(self.repo,
-                                                               parents, config, timestamp=revision.timestamp,
-                                                               timezone=revision.timezone, committer=revision.committer,
-                                                               revprops=revision.properties, revision_id=revision.revision_id)
+        self._commit_builder = self.repo.get_commit_builder(
+            self.repo,
+            parents, config, timestamp=revision.timestamp,
+            timezone=revision.timezone, committer=revision.committer,
+            revprops=revision.properties, revision_id=revision.revision_id)
 
     def get_parents_and_revision_for_entry(self, ie):
         """Get the parents and revision for an inventory entry.
@@ -393,10 +390,3 @@ class RevisionStore(object):
             raise AssertionError('signatures not guaranteed yet')
             self.repo.add_signature_text(rev.revision_id, signature)
         return builder.revision_tree().root_inventory
-
-    def get_file_lines(self, revision_id, file_id):
-        record = next(self.repo.texts.get_record_stream([(file_id, revision_id)],
-                                                        'unordered', True))
-        if record.storage_kind == 'absent':
-            raise errors.RevisionNotPresent(record.key, self.repo)
-        return osutils.split_lines(record.get_bytes_as('fulltext'))
