@@ -19,8 +19,6 @@
 Currently only tells the user that Monotone is not supported.
 """
 
-from __future__ import absolute_import
-
 from ... import version_info  # noqa: F401
 
 from ... import (
@@ -29,11 +27,13 @@ from ... import (
     )
 
 
-class MonotoneUnsupportedError(errors.UnsupportedFormatError):
+class MonotoneUnsupportedError(errors.UnsupportedVcs):
+
+    vcs = "mtn"
 
     _fmt = ('Monotone branches are not yet supported. '
-            'To convert monotone branches to Bazaar branches or vice versa, '
-            'use bzr-fastimport. See http://bazaar-vcs.org/BzrMigration.')
+            'To interoperate with Monotone branches, '
+            'use fastimport.')
 
 
 class MonotoneDirFormat(controldir.ControlDirFormat):
@@ -56,7 +56,7 @@ class MonotoneDirFormat(controldir.ControlDirFormat):
 
     def check_support_status(self, allow_unsupported, recommend_upgrade=True,
                              basedir=None):
-        raise MonotoneUnsupportedError(self)
+        raise MonotoneUnsupportedError(format=self)
 
     def open(self, transport):
         # Raise NotBranchError if there is nothing there
