@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
+from typing import List
 
 from ...bzr.xml_serializer import (
     Element,
@@ -36,7 +36,7 @@ class _Serializer_v4(XMLSerializer):
     v4 serialisation is no longer supported, only deserialisation.
     """
 
-    __slots__ = []
+    __slots__: List[str] = []
 
     def _pack_entry(self, ie):
         """Convert InventoryEntry to XML element"""
