@@ -14,35 +14,38 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
-
-
-from .lazy_import import lazy_import
-lazy_import(globals(), """
-import bisect
-import datetime
+from typing import List, Type, Optional
 
 from breezy import (
-    branch as _mod_branch,
-    cache_utf8,
     revision,
     workingtree,
     )
 from breezy.i18n import gettext
-""")
 
 from . import (
     errors,
     lazy_regex,
     registry,
+    revision as _mod_revision,
     trace,
     )
-from .sixish import (
-    text_type,
-    )
 
 
-class RevisionInfo(object):
+class InvalidRevisionSpec(errors.BzrError):
+
+    _fmt = ("Requested revision: '%(spec)s' does not exist in branch:"
+            " %(branch_url)s%(extra)s")
+
+    def __init__(self, spec, branch, extra=None):
+        errors.BzrError.__init__(self, branch=branch, spec=spec)
+        self.branch_url = getattr(branch, 'user_url', str(branch))
+        if extra:
+            self.extra = '\n' + str(extra)
+        else:
+            self.extra = ''
+
+
+class RevisionInfo:
     """The results of applying a revision specification to a branch."""
 
     help_txt = """The results of applying a revision specification to a branch.
@@ -109,7 +112,7 @@ class RevisionInfo(object):
         return tuple(self) == tuple(other)
 
     def __repr__(self):
-        return '<breezy.revisionspec.RevisionInfo object %s, %s for %r>' % (
+        return '<breezy.revisionspec.RevisionInfo object {}, {} for {!r}>'.format(
             self.revno, self.rev_id, self.branch)
 
     @staticmethod
@@ -121,7 +124,7 @@ class RevisionInfo(object):
         return RevisionInfo(branch, revno=None, rev_id=revision_id)
 
 
-class RevisionSpec(object):
+class RevisionSpec:
     """A parsed revision specification."""
 
     help_txt = """A parsed revision specification.
@@ -140,8 +143,8 @@ class RevisionSpec(object):
     (Equivalent to the old Branch method get_revision_info())
     """
 
-    prefix = None
-    dwim_catchable_exceptions = (errors.InvalidRevisionSpec,)
+    prefix: Optional[str] = None
+    dwim_catchable_exceptions: List[Type[Exception]] = [InvalidRevisionSpec]
     """Exceptions that RevisionSpec_dwim._match_on will catch.
 
     If the revspec is part of ``dwim_revspecs``, it may be tried with an
@@ -160,7 +163,7 @@ class RevisionSpec(object):
         """
         if spec is None:
             return RevisionSpec(None, _internal=True)
-        if not isinstance(spec, (str, text_type)):
+        if not isinstance(spec, str):
             raise TypeError("revision spec needs to be text")
         match = revspec_registry.get_prefix(spec)
         if match is not None:
@@ -201,9 +204,9 @@ class RevisionSpec(object):
             # special case - nothing supplied
             return info
         elif self.prefix:
-            raise errors.InvalidRevisionSpec(self.user_spec, branch)
+            raise InvalidRevisionSpec(self.user_spec, branch)
         else:
-            raise errors.InvalidRevisionSpec(self.spec, branch)
+            raise InvalidRevisionSpec(self.spec, branch)
 
     def in_history(self, branch):
         return self._match_on_and_check(branch, revs=None)
@@ -255,7 +258,7 @@ class RevisionSpec(object):
 
     def __repr__(self):
         # this is mostly for helping with testing
-        return '<%s %s>' % (self.__class__.__name__,
+        return '<{} {}>'.format(self.__class__.__name__,
                             self.user_spec)
 
     def needs_branch(self):
@@ -284,12 +287,12 @@ class RevisionSpec_dwim(RevisionSpec):
     is called so the string describing the revision is kept here until needed.
     """
 
-    help_txt = None
+    help_txt: str
 
     _revno_regex = lazy_regex.lazy_compile(r'^(?:(\d+(\.\d+)*)|-\d+)(:.*)?$')
 
     # The revspecs to try
-    _possible_revspecs = []
+    _possible_revspecs: List[Type[registry._ObjectGetter]] = []
 
     def _try_spectype(self, rstype, branch):
         rs = rstype(self.spec, _internal=True)
@@ -304,7 +307,7 @@ class RevisionSpec_dwim(RevisionSpec):
         if self._revno_regex.match(self.spec) is not None:
             try:
                 return self._try_spectype(RevisionSpec_revno, branch)
-            except RevisionSpec_revno.dwim_catchable_exceptions:
+            except tuple(RevisionSpec_revno.dwim_catchable_exceptions):
                 pass
 
         # Next see what has been registered
@@ -312,13 +315,13 @@ class RevisionSpec_dwim(RevisionSpec):
             rs_class = objgetter.get_obj()
             try:
                 return self._try_spectype(rs_class, branch)
-            except rs_class.dwim_catchable_exceptions:
+            except tuple(rs_class.dwim_catchable_exceptions):
                 pass
 
         # Well, I dunno what it is. Note that we don't try to keep track of the
         # first of last exception raised during the DWIM tries as none seems
         # really relevant.
-        raise errors.InvalidRevisionSpec(self.spec, branch)
+        raise InvalidRevisionSpec(self.spec, branch)
 
     @classmethod
     def append_possible_revspec(cls, revspec):
@@ -378,8 +381,9 @@ class RevisionSpec_revno(RevisionSpec):
 
         if revno_spec == '':
             if not branch_spec:
-                raise errors.InvalidRevisionSpec(self.user_spec,
-                                                 branch, 'cannot have an empty revno and no branch')
+                raise InvalidRevisionSpec(
+                    self.user_spec, branch,
+                    'cannot have an empty revno and no branch')
             revno = None
         else:
             try:
@@ -390,23 +394,24 @@ class RevisionSpec_revno(RevisionSpec):
                 # but the from_string method is a little primitive
                 # right now - RBC 20060928
                 try:
-                    match_revno = tuple((int(number)
-                                         for number in revno_spec.split('.')))
+                    match_revno = tuple(int(number)
+                                         for number in revno_spec.split('.'))
                 except ValueError as e:
-                    raise errors.InvalidRevisionSpec(self.user_spec, branch, e)
+                    raise InvalidRevisionSpec(self.user_spec, branch, e)
 
                 dotted = True
 
         if branch_spec:
+            from .branch import Branch
             # the user has overriden the branch to look in.
-            branch = _mod_branch.Branch.open(branch_spec)
+            branch = Branch.open(branch_spec)
 
         if dotted:
             try:
                 revision_id = branch.dotted_revno_to_revision_id(match_revno,
                                                                  _cache_reverse=True)
             except (errors.NoSuchRevision, errors.RevnoOutOfBounds):
-                raise errors.InvalidRevisionSpec(self.user_spec, branch)
+                raise InvalidRevisionSpec(self.user_spec, branch)
             else:
                 # there is no traditional 'revno' for dotted-decimal revnos.
                 # so for API compatibility we return None.
@@ -423,7 +428,7 @@ class RevisionSpec_revno(RevisionSpec):
             try:
                 revision_id = branch.get_rev_id(revno)
             except (errors.NoSuchRevision, errors.RevnoOutOfBounds):
-                raise errors.InvalidRevisionSpec(self.user_spec, branch)
+                raise InvalidRevisionSpec(self.user_spec, branch)
         return branch, revno, revision_id
 
     def _as_revision_id(self, context_branch):
@@ -471,8 +476,8 @@ class RevisionSpec_revid(RevisionIDSpec):
         # self.spec comes straight from parsing the command line arguments,
         # so we expect it to be a Unicode string. Switch it to the internal
         # representation.
-        if isinstance(self.spec, text_type):
-            return cache_utf8.encode(self.spec)
+        if isinstance(self.spec, str):
+            return self.spec.encode('utf-8')
         return self.spec
 
 
@@ -506,17 +511,18 @@ class RevisionSpec_last(RevisionSpec):
         try:
             offset = int(self.spec)
         except ValueError as e:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch, e)
+            raise InvalidRevisionSpec(self.user_spec, context_branch, e)
 
         if offset <= 0:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                             'you must supply a positive value')
+            raise InvalidRevisionSpec(
+                self.user_spec, context_branch,
+                'you must supply a positive value')
 
         revno = last_revno - offset + 1
         try:
             revision_id = context_branch.get_rev_id(revno)
         except (errors.NoSuchRevision, errors.RevnoOutOfBounds):
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch)
+            raise InvalidRevisionSpec(self.user_spec, context_branch)
         return revno, revision_id
 
     def _as_revision_id(self, context_branch):
@@ -553,8 +559,9 @@ class RevisionSpec_before(RevisionSpec):
     def _match_on(self, branch, revs):
         r = RevisionSpec.from_string(self.spec)._match_on(branch, revs)
         if r.revno == 0:
-            raise errors.InvalidRevisionSpec(self.user_spec, branch,
-                                             'cannot go before the null: revision')
+            raise InvalidRevisionSpec(
+                self.user_spec, branch,
+                'cannot go before the null: revision')
         if r.revno is None:
             # We need to use the repository history here
             rev = branch.repository.get_revision(r.rev_id)
@@ -568,27 +575,27 @@ class RevisionSpec_before(RevisionSpec):
             try:
                 revision_id = branch.get_rev_id(revno, revs)
             except (errors.NoSuchRevision, errors.RevnoOutOfBounds):
-                raise errors.InvalidRevisionSpec(self.user_spec,
-                                                 branch)
+                raise InvalidRevisionSpec(self.user_spec, branch)
         return RevisionInfo(branch, revno, revision_id)
 
     def _as_revision_id(self, context_branch):
         base_revision_id = RevisionSpec.from_string(
             self.spec)._as_revision_id(context_branch)
         if base_revision_id == revision.NULL_REVISION:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                             'cannot go before the null: revision')
+            raise InvalidRevisionSpec(
+                self.user_spec, context_branch,
+                'cannot go before the null: revision')
         context_repo = context_branch.repository
         with context_repo.lock_read():
             parent_map = context_repo.get_parent_map([base_revision_id])
         if base_revision_id not in parent_map:
             # Ghost, or unknown revision id
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                             'cannot find the matching revision')
+            raise InvalidRevisionSpec(
+                self.user_spec, context_branch, 'cannot find the matching revision')
         parents = parent_map[base_revision_id]
         if len(parents) < 1:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                             'No parents for revision.')
+            raise InvalidRevisionSpec(
+                self.user_spec, context_branch, 'No parents for revision.')
         return parents[0]
 
 
@@ -601,7 +608,7 @@ class RevisionSpec_tag(RevisionSpec):
     """
 
     prefix = 'tag:'
-    dwim_catchable_exceptions = (errors.NoSuchTag, errors.TagsNotSupported)
+    dwim_catchable_exceptions = [errors.NoSuchTag, errors.TagsNotSupported]
 
     def _match_on(self, branch, revs):
         # Can raise tags not supported, NoSuchTag, etc
@@ -612,7 +619,7 @@ class RevisionSpec_tag(RevisionSpec):
         return context_branch.tags.lookup_tag(self.spec)
 
 
-class _RevListToTimestamps(object):
+class _RevListToTimestamps:
     """This takes a list of revisions, and allows you to bisect by date"""
 
     __slots__ = ['branch']
@@ -623,11 +630,55 @@ class _RevListToTimestamps(object):
     def __getitem__(self, index):
         """Get the date of the index'd item"""
         r = self.branch.repository.get_revision(self.branch.get_rev_id(index))
-        # TODO: Handle timezone.
-        return datetime.datetime.fromtimestamp(r.timestamp)
+        return r.datetime()
 
-    def __len__(self):
-        return self.branch.revno()
+
+_date_regex = lazy_regex.lazy_compile(
+    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+    r'(,|T)?\s*'
+    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+    )
+
+
+def _parse_datespec(spec):
+    import datetime
+    #  XXX: This doesn't actually work
+    #  So the proper way of saying 'give me all entries for today' is:
+    #      -r date:yesterday..date:today
+    today = datetime.datetime.fromordinal(
+        datetime.date.today().toordinal())
+    if spec.lower() == 'yesterday':
+        return today - datetime.timedelta(days=1)
+    elif spec.lower() == 'today':
+        return today
+    elif spec.lower() == 'tomorrow':
+        return today + datetime.timedelta(days=1)
+    else:
+        m = _date_regex.match(spec)
+        if not m or (not m.group('date') and not m.group('time')):
+            raise ValueError
+
+        if m.group('date'):
+            year = int(m.group('year'))
+            month = int(m.group('month'))
+            day = int(m.group('day'))
+        else:
+            year = today.year
+            month = today.month
+            day = today.day
+
+        if m.group('time'):
+            hour = int(m.group('hour'))
+            minute = int(m.group('minute'))
+            if m.group('second'):
+                second = int(m.group('second'))
+            else:
+                second = 0
+        else:
+            hour, minute, second = 0, 0, 0
+
+        return datetime.datetime(year=year, month=month, day=day,
+                               hour=hour, minute=minute, second=second)
 
 
 class RevisionSpec_date(RevisionSpec):
@@ -651,11 +702,28 @@ class RevisionSpec_date(RevisionSpec):
                                    August 14th, 2006 at 5:10pm.
     """
     prefix = 'date:'
-    _date_regex = lazy_regex.lazy_compile(
-        r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-        r'(,|T)?\s*'
-        r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-        )
+
+    def _scan_backwards(self, branch, dt):
+        with branch.lock_read():
+            graph = branch.repository.get_graph()
+            last_match = None
+            for revid in graph.iter_lefthand_ancestry(
+                    branch.last_revision(), (_mod_revision.NULL_REVISION,)):
+                r = branch.repository.get_revision(revid)
+                if r.datetime() < dt:
+                    if last_match is None:
+                        raise InvalidRevisionSpec(self.user_spec, branch)
+                    return RevisionInfo(branch, None, last_match)
+                last_match = revid
+            return RevisionInfo(branch, None, last_match)
+
+    def _bisect_backwards(self, branch, dt, hi):
+        import bisect
+        with branch.lock_read():
+            rev = bisect.bisect(_RevListToTimestamps(branch), dt, 1, hi)
+        if rev == branch.revno():
+            raise InvalidRevisionSpec(self.user_spec, branch)
+        return RevisionInfo(branch, rev)
 
     def _match_on(self, branch, revs):
         """Spec for date revisions:
@@ -664,53 +732,16 @@ class RevisionSpec_date(RevisionSpec):
           matches the first entry after a given date (either at midnight or
           at a specified time).
         """
-        #  XXX: This doesn't actually work
-        #  So the proper way of saying 'give me all entries for today' is:
-        #      -r date:yesterday..date:today
-        today = datetime.datetime.fromordinal(
-            datetime.date.today().toordinal())
-        if self.spec.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif self.spec.lower() == 'today':
-            dt = today
-        elif self.spec.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
+        try:
+            dt = _parse_datespec(self.spec)
+        except ValueError:
+            raise InvalidRevisionSpec(
+                self.user_spec, branch, 'invalid date')
+        revno = branch.revno()
+        if revno is None:
+            return self._scan_backwards(branch, dt)
         else:
-            m = self._date_regex.match(self.spec)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise errors.InvalidRevisionSpec(self.user_spec,
-                                                 branch, 'invalid date')
-
-            try:
-                if m.group('date'):
-                    year = int(m.group('year'))
-                    month = int(m.group('month'))
-                    day = int(m.group('day'))
-                else:
-                    year = today.year
-                    month = today.month
-                    day = today.day
-
-                if m.group('time'):
-                    hour = int(m.group('hour'))
-                    minute = int(m.group('minute'))
-                    if m.group('second'):
-                        second = int(m.group('second'))
-                    else:
-                        second = 0
-                else:
-                    hour, minute, second = 0, 0, 0
-            except ValueError:
-                raise errors.InvalidRevisionSpec(self.user_spec,
-                                                 branch, 'invalid date')
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                                   hour=hour, minute=minute, second=second)
-        with branch.lock_read():
-            rev = bisect.bisect(_RevListToTimestamps(branch), dt, 1)
-        if rev == branch.revno():
-            raise errors.InvalidRevisionSpec(self.user_spec, branch)
-        return RevisionInfo(branch, rev)
+            return self._bisect_backwards(branch, dt, revno)
 
 
 class RevisionSpec_ancestor(RevisionSpec):
@@ -753,14 +784,14 @@ class RevisionSpec_ancestor(RevisionSpec):
         from .branch import Branch
 
         with branch.lock_read():
-            revision_a = revision.ensure_null(branch.last_revision())
+            revision_a = branch.last_revision()
             if revision_a == revision.NULL_REVISION:
                 raise errors.NoCommits(branch)
             if other_location == '':
                 other_location = branch.get_parent()
             other_branch = Branch.open(other_location)
             with other_branch.lock_read():
-                revision_b = revision.ensure_null(other_branch.last_revision())
+                revision_b = other_branch.last_revision()
                 if revision_b == revision.NULL_REVISION:
                     raise errors.NoCommits(other_branch)
                 graph = branch.repository.get_graph(other_branch.repository)
@@ -782,7 +813,7 @@ class RevisionSpec_branch(RevisionSpec):
       branch:/path/to/branch
     """
     prefix = 'branch:'
-    dwim_catchable_exceptions = (errors.NotBranchError,)
+    dwim_catchable_exceptions = [errors.NotBranchError]
 
     def _match_on(self, branch, revs):
         from .branch import Branch
@@ -804,7 +835,6 @@ class RevisionSpec_branch(RevisionSpec):
         from .branch import Branch
         other_branch = Branch.open(self.spec)
         last_revision = other_branch.last_revision()
-        last_revision = revision.ensure_null(last_revision)
         context_branch.fetch(other_branch, last_revision)
         if last_revision == revision.NULL_REVISION:
             raise errors.NoCommits(other_branch)
@@ -814,7 +844,6 @@ class RevisionSpec_branch(RevisionSpec):
         from .branch import Branch
         other_branch = Branch.open(self.spec)
         last_revision = other_branch.last_revision()
-        last_revision = revision.ensure_null(last_revision)
         if last_revision == revision.NULL_REVISION:
             raise errors.NoCommits(other_branch)
         return other_branch.repository.revision_tree(last_revision)
@@ -882,8 +911,9 @@ class RevisionSpec_annotate(RevisionIDSpec):
     """
 
     def _raise_invalid(self, numstring, context_branch):
-        raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                         'No such line: %s' % numstring)
+        raise InvalidRevisionSpec(
+            self.user_spec, context_branch,
+            'No such line: %s' % numstring)
 
     def _as_revision_id(self, context_branch):
         path, numstring = self.spec.rsplit(':', 1)
@@ -894,17 +924,18 @@ class RevisionSpec_annotate(RevisionIDSpec):
         tree, file_path = workingtree.WorkingTree.open_containing(path)
         with tree.lock_read():
             if not tree.has_filename(file_path):
-                raise errors.InvalidRevisionSpec(self.user_spec,
-                                                 context_branch, "File '%s' is not versioned." %
-                                                 file_path)
+                raise InvalidRevisionSpec(
+                    self.user_spec, context_branch,
+                    "File '%s' is not versioned." % file_path)
             revision_ids = [r for (r, l) in tree.annotate_iter(file_path)]
         try:
             revision_id = revision_ids[index]
         except IndexError:
             self._raise_invalid(numstring, context_branch)
         if revision_id == revision.CURRENT_REVISION:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
-                                             'Line %s has not been committed.' % numstring)
+            raise InvalidRevisionSpec(
+                self.user_spec, context_branch,
+                'Line %s has not been committed.' % numstring)
         return revision_id
 
 
@@ -922,13 +953,14 @@ class RevisionSpec_mainline(RevisionIDSpec):
         if revspec.get_branch() is None:
             spec_branch = context_branch
         else:
-            spec_branch = _mod_branch.Branch.open(revspec.get_branch())
+            from .branch import Branch
+            spec_branch = Branch.open(revspec.get_branch())
         revision_id = revspec.as_revision_id(spec_branch)
         graph = context_branch.repository.get_graph()
         result = graph.find_lefthand_merger(revision_id,
                                             context_branch.last_revision())
         if result is None:
-            raise errors.InvalidRevisionSpec(self.user_spec, context_branch)
+            raise InvalidRevisionSpec(self.user_spec, context_branch)
         return result
 
 
@@ -940,7 +972,7 @@ RevisionSpec_dwim.append_possible_revspec(RevisionSpec_revid)
 RevisionSpec_dwim.append_possible_revspec(RevisionSpec_date)
 RevisionSpec_dwim.append_possible_revspec(RevisionSpec_branch)
 
-revspec_registry = registry.Registry()
+revspec_registry = registry.Registry[str, RevisionSpec]()
 
 
 def _register_revspec(revspec):
