@@ -38,7 +38,6 @@ from breezy.tests.features import (
     HardlinkFeature,
     )
 from breezy.tests.blackbox import test_switch
-from breezy.tests.matchers import ContainsNoVfsCalls
 from breezy.tests.test_sftp_transport import TestCaseWithSFTPServer
 from breezy.tests.script import run_script
 from breezy.urlutils import local_path_to_url, strip_trailing_slash
@@ -107,6 +106,17 @@ class TestBranch(tests.TestCaseWithTransport):
         self.assertEqual('Branched 0 revisions.\n', err)
         self.assertPathExists('a')
 
+    def test_from_name(self):
+        """Branch from a colocated branch into a regular branch."""
+        os.mkdir('b')
+        tree = self.example_branch('b/a', format='development-colo')
+        tree.controldir.create_branch(name='somecolo')
+        out, err = self.run_bzr('branch -b somecolo %s' %
+                                local_path_to_url('b/a'))
+        self.assertEqual('', out)
+        self.assertEqual('Branched 0 revisions.\n', err)
+        self.assertPathExists('a')
+
     def test_branch_broken_pack(self):
         """branching with a corrupted pack file."""
         self.example_branch('a')
@@ -217,31 +227,31 @@ class TestBranch(tests.TestCaseWithTransport):
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
         tree_a.add('file')
-        tree_a.commit('commit a-1', rev_id=b'a-1')
+        a1 = tree_a.commit('commit a-1')
         with open('repo/a/file', 'ab') as f:
             f.write(b'more stuff\n')
-        tree_a.commit('commit a-2', rev_id=b'a-2')
+        a2 = tree_a.commit('commit a-2')
 
         tree_b = make_shared_tree('b')
         self.build_tree(['repo/b/file'])
         tree_b.add('file')
-        tree_b.commit('commit b-1', rev_id=b'b-1')
+        b1 = tree_b.commit('commit b-1')
 
-        self.assertTrue(shared_repo.has_revision(b'a-1'))
-        self.assertTrue(shared_repo.has_revision(b'a-2'))
-        self.assertTrue(shared_repo.has_revision(b'b-1'))
+        self.assertTrue(shared_repo.has_revision(a1))
+        self.assertTrue(shared_repo.has_revision(a2))
+        self.assertTrue(shared_repo.has_revision(b1))
 
         # Now that we have a repository with shared files, make sure
         # that things aren't copied out by a 'branch'
         self.run_bzr('branch repo/b branch-b')
         pushed_tree = WorkingTree.open('branch-b')
         pushed_repo = pushed_tree.branch.repository
-        self.assertFalse(pushed_repo.has_revision(b'a-1'))
-        self.assertFalse(pushed_repo.has_revision(b'a-2'))
-        self.assertTrue(pushed_repo.has_revision(b'b-1'))
+        self.assertFalse(pushed_repo.has_revision(a1))
+        self.assertFalse(pushed_repo.has_revision(a2))
+        self.assertTrue(pushed_repo.has_revision(b1))
 
     def test_branch_hardlink(self):
-        self.requireFeature(HardlinkFeature)
+        self.requireFeature(HardlinkFeature(self.test_dir))
         source = self.make_branch_and_tree('source')
         self.build_tree(['source/file1'])
         source.add('file1')
@@ -260,7 +270,7 @@ class TestBranch(tests.TestCaseWithTransport):
         self.assertPathExists('target/file1')
 
     def test_branch_files_from_hardlink(self):
-        self.requireFeature(HardlinkFeature)
+        self.requireFeature(HardlinkFeature(self.test_dir))
         source = self.make_branch_and_tree('source')
         self.build_tree(['source/file1'])
         source.add('file1')
@@ -360,6 +370,54 @@ class TestBranch(tests.TestCaseWithTransport):
         self.assertEqual(rev2, new_branch.tags.lookup_tag('tag-a'))
         new_branch.repository.get_revision(rev2)
 
+    def test_branch_with_nested_trees(self):
+        orig = self.make_branch_and_tree('source', format='development-subtree')
+        subtree = self.make_branch_and_tree('source/subtree')
+        self.build_tree(['source/subtree/a'])
+        subtree.add(['a'])
+        subtree.commit('add subtree contents')
+        orig.add_reference(subtree)
+        orig.set_reference_info('subtree', subtree.branch.user_url)
+        orig.commit('add subtree')
+
+        self.run_bzr('branch source target')
+
+        target = WorkingTree.open('target')
+        target_subtree = WorkingTree.open('target/subtree')
+        self.assertTreesEqual(orig, target)
+        self.assertTreesEqual(subtree, target_subtree)
+
+    def test_branch_with_nested_trees_reference_unset(self):
+        orig = self.make_branch_and_tree('source', format='development-subtree')
+        subtree = self.make_branch_and_tree('source/subtree')
+        self.build_tree(['source/subtree/a'])
+        subtree.add(['a'])
+        subtree.commit('add subtree contents')
+        orig.add_reference(subtree)
+        orig.commit('add subtree')
+
+        self.run_bzr('branch source target')
+
+        target = WorkingTree.open('target')
+        self.assertRaises(errors.NotBranchError, WorkingTree.open, 'target/subtree')
+
+    def test_branch_with_nested_trees_no_recurse(self):
+        orig = self.make_branch_and_tree('source', format='development-subtree')
+        subtree = self.make_branch_and_tree('source/subtree')
+        self.build_tree(['source/subtree/a'])
+        subtree.add(['a'])
+        subtree.commit('add subtree contents')
+        orig.add_reference(subtree)
+        orig.commit('add subtree')
+
+        self.run_bzr('branch --no-recurse-nested source target')
+
+        target = WorkingTree.open('target')
+        self.addCleanup(subtree.lock_read().unlock)
+        basis = subtree.basis_tree()
+        self.addCleanup(basis.lock_read().unlock)
+        self.assertRaises(errors.NotBranchError, WorkingTree.open, 'target/subtree')
+
 
 class TestBranchStacked(tests.TestCaseWithTransport):
     """Tests for branch --stacked"""
@@ -493,132 +551,11 @@ class TestBranchStacked(tests.TestCaseWithTransport):
             err)
 
 
-class TestSmartServerBranching(tests.TestCaseWithTransport):
-
-    def test_branch_from_trivial_branch_to_same_server_branch_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        for count in range(9):
-            t.commit(message='commit %d' % count)
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', self.get_url('from'),
-                                 self.get_url('target')])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(2, self.hpss_connections)
-        self.assertLength(33, self.hpss_calls)
-        self.expectFailure("branching to the same branch requires VFS access",
-                           self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
-
-    def test_branch_from_trivial_branch_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        for count in range(9):
-            t.commit(message='commit %d' % count)
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', self.get_url('from'),
-                                 'local-target'])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-        self.assertLength(10, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-
-    def test_branch_from_trivial_stacked_branch_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('trunk')
-        for count in range(8):
-            t.commit(message='commit %d' % count)
-        tree2 = t.branch.controldir.sprout('feature', stacked=True
-                                           ).open_workingtree()
-        local_tree = t.branch.controldir.sprout(
-            'local-working').open_workingtree()
-        local_tree.commit('feature change')
-        local_tree.branch.push(tree2.branch)
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', self.get_url('feature'),
-                                 'local-target'])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(15, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-
-    def test_branch_from_branch_with_tags(self):
-        self.setup_smart_server_with_call_log()
-        builder = self.make_branch_builder('source')
-        source, rev1, rev2 = fixtures.build_branch_with_non_ancestral_rev(
-            builder)
-        source.get_config_stack().set('branch.fetch_tags', True)
-        source.tags.set_tag('tag-a', rev2)
-        source.tags.set_tag('tag-missing', b'missing-rev')
-        # Now source has a tag not in its ancestry.  Make a branch from it.
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', self.get_url('source'), 'target'])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(10, self.hpss_calls)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-        self.assertLength(1, self.hpss_connections)
-
-    def test_branch_to_stacked_from_trivial_branch_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        for count in range(9):
-            t.commit(message='commit %d' % count)
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', '--stacked', self.get_url('from'),
-                                 'local-target'])
-        # XXX: the number of hpss calls for this case isn't deterministic yet,
-        # so we can't easily assert about the number of calls.
-        #self.assertLength(XXX, self.hpss_calls)
-        # We can assert that none of the calls were readv requests for rix
-        # files, though (demonstrating that at least get_parent_map calls are
-        # not using VFS RPCs).
-        readvs_of_rix_files = [
-            c for c in self.hpss_calls
-            if c.call.method == 'readv' and c.call.args[-1].endswith('.rix')]
-        self.assertLength(1, self.hpss_connections)
-        self.assertLength(0, readvs_of_rix_files)
-        self.expectFailure("branching to stacked requires VFS access",
-                           self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
-
-    def test_branch_from_branch_with_ghosts(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        for count in range(9):
-            t.commit(message='commit %d' % count)
-        t.set_parent_ids([t.last_revision(), b'ghost'])
-        t.commit(message='add commit with parent')
-        self.reset_smart_call_log()
-        out, err = self.run_bzr(['branch', self.get_url('from'),
-                                 'local-target'])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-        self.assertLength(11, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-
-
 class TestRemoteBranch(TestCaseWithSFTPServer):
 
     def setUp(self):
-        super(TestRemoteBranch, self).setUp()
+        super().setUp()
+        self.skipTest('tests often hang - see pad.lv/1997033')
         tree = self.make_branch_and_tree('branch')
         self.build_tree_contents([('branch/file', b'file content\n')])
         tree.add('file')
