@@ -26,7 +26,6 @@ from testtools import matchers
 
 from breezy import (
     branch,
-    conflicts,
     controldir,
     merge_directive,
     osutils,
@@ -34,6 +33,9 @@ from breezy import (
     urlutils,
     workingtree,
     )
+from breezy.bzr import (
+    conflicts,
+    )
 from breezy.tests import (
     scenarios,
     script,
@@ -290,7 +292,7 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_b.commit('commit e')
         out, err = self.run_bzr('merge', working_dir='branch_b')
         self.assertStartsWith(
-            err, 'Merging from remembered submit location %s\n' % (base,))
+            err, 'Merging from remembered submit location {}\n'.format(base))
         # re-open tree as external run_brz modified it
         tree_b = branch_b.controldir.open_workingtree()
         tree_b.commit('merge branch_a')
@@ -328,7 +330,7 @@ class TestMerge(tests.TestCaseWithTransport):
                                               tree_b.get_parent_ids()[0])
         self.assertEqualDiff(testament_a.as_text(),
                              testament_b.as_text())
-        tree_a.set_conflicts(conflicts.ConflictList())
+        tree_a.set_conflicts([])
         tree_a.commit('message')
         # it is legal to attempt to merge an already-merged bundle
         err = self.run_bzr('merge ../bundle', working_dir='branch_a')[1]
@@ -413,7 +415,7 @@ class TestMerge(tests.TestCaseWithTransport):
     def test_merge_kind_change(self):
         tree_a = self.make_branch_and_tree('tree_a')
         self.build_tree_contents([('tree_a/file', b'content_1')])
-        tree_a.add('file', b'file-id')
+        tree_a.add('file', ids=b'file-id')
         tree_a.commit('added file')
         tree_b = tree_a.controldir.sprout('tree_b').open_workingtree()
         os.unlink('tree_a/file')
@@ -427,8 +429,7 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_b.commit('content change')
         self.run_bzr('merge ../tree_a', retcode=1, working_dir='tree_b')
         self.assertEqual(tree_b.conflicts(),
-                         [conflicts.ContentsConflict('file',
-                                                     file_id='file-id')])
+                         [conflicts.ContentsConflict('file', file_id='file-id')])
 
     def test_directive_cherrypick(self):
         source = self.make_branch_and_tree('source')
@@ -632,11 +633,11 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_a.merge_from_branch(tree_b.branch)
         self.build_tree_contents([('a/file',
                                    b'base-contents\nthis-contents\n')])
-        tree_a.set_conflicts(conflicts.ConflictList())
+        tree_a.set_conflicts([])
         tree_b.merge_from_branch(tree_a.branch)
         self.build_tree_contents([('b/file',
                                    b'base-contents\nother-contents\n')])
-        tree_b.set_conflicts(conflicts.ConflictList())
+        tree_b.set_conflicts([])
         tree_a.commit('', rev_id=b'rev3a')
         tree_b.commit('', rev_id=b'rev3b')
         out, err = self.run_bzr(['merge', '-d', 'a', 'b', '--lca'], retcode=1)
@@ -709,7 +710,7 @@ class TestMergeRevisionRange(tests.TestCaseWithTransport):
                  ('file-only', dict(context='a')))
 
     def setUp(self):
-        super(TestMergeRevisionRange, self).setUp()
+        super().setUp()
         self.tree = self.make_branch_and_tree(".")
         self.tree.commit('initial commit')
         for f in ("a", "b"):
@@ -739,7 +740,7 @@ $ brz merge -d target source
 class TestMergeForce(tests.TestCaseWithTransport):
 
     def setUp(self):
-        super(TestMergeForce, self).setUp()
+        super().setUp()
         self.tree_a = self.make_branch_and_tree('a')
         self.build_tree(['a/foo'])
         self.tree_a.add(['foo'])
