@@ -41,7 +41,6 @@ from breezy.tests import (
     script,
     test_foreign,
     )
-from breezy.tests.matchers import ContainsNoVfsCalls
 from breezy.transport import memory
 
 
@@ -236,7 +235,7 @@ class TestPush(tests.TestCaseWithTransport):
     def test_push_funky_id(self):
         t = self.make_branch_and_tree('tree')
         self.build_tree(['tree/filename'])
-        t.add('filename', b'funky-chars<>%&;"\'')
+        t.add('filename', ids=b'funky-chars<>%&;"\'')
         t.commit('commit filename')
         self.run_bzr('push -d tree new-tree')
 
@@ -268,75 +267,6 @@ class TestPush(tests.TestCaseWithTransport):
         self.assertEqual([(b'A',), (b'C',)], sorted(
             target_repo.revisions.keys()))
 
-    def test_push_smart_non_stacked_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        t.commit(allow_pointless=True, message='first commit')
-        self.reset_smart_call_log()
-        self.run_bzr(['push', self.get_url('to-one')], working_dir='from')
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(9, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-
-    def test_push_smart_stacked_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        parent = self.make_branch_and_tree('parent', format='1.9')
-        parent.commit(message='first commit')
-        local = parent.controldir.sprout('local').open_workingtree()
-        local.commit(message='local commit')
-        self.reset_smart_call_log()
-        self.run_bzr(['push', '--stacked', '--stacked-on', '../parent',
-                      self.get_url('public')], working_dir='local')
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(15, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-        remote = branch.Branch.open('public')
-        self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
-
-    def test_push_smart_tags_streaming_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        rev_id = t.commit(allow_pointless=True, message='first commit')
-        t.branch.tags.set_tag('new-tag', rev_id)
-        self.reset_smart_call_log()
-        self.run_bzr(['push', self.get_url('to-one')], working_dir='from')
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(11, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-
-    def test_push_smart_incremental_acceptance(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        rev_id1 = t.commit(allow_pointless=True, message='first commit')
-        rev_id2 = t.commit(allow_pointless=True, message='second commit')
-        self.run_bzr(
-            ['push', self.get_url('to-one'), '-r1'], working_dir='from')
-        self.reset_smart_call_log()
-        self.run_bzr(['push', self.get_url('to-one')], working_dir='from')
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(11, self.hpss_calls)
-        self.assertLength(1, self.hpss_connections)
-        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
-
     def test_push_smart_with_default_stacking_url_path_segment(self):
         # If the default stacked-on location is a path element then branches
         # we push there over the smart server are stacked and their
@@ -365,7 +295,7 @@ class TestPush(tests.TestCaseWithTransport):
     def create_simple_tree(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a'])
-        tree.add(['a'], [b'a-id'])
+        tree.add(['a'], ids=[b'a-id'])
         tree.commit('one', rev_id=b'r1')
         return tree
 
@@ -499,10 +429,10 @@ class TestPush(tests.TestCaseWithTransport):
         self.assertPublished(branch_tree.last_revision(),
                              trunk_tree.branch.base)
 
-    def test_push_new_branch_stacked_on(self):
+    def test_push_new_branch_stacked_on_can_use_dir_urls(self):
         """Pushing a new branch with --stacked-on can use directory URLs."""
         trunk_tree, branch_tree = self.create_trunk_and_feature_branch()
-        class FooDirectory(object):
+        class FooDirectory:
             def look_up(self, name, url, purpose=None):
                 if url == 'foo:':
                     return trunk_tree.branch.base
@@ -670,7 +600,7 @@ class RedirectingMemoryTransport(memory.MemoryTransport):
                                            self.abspath('../infinite-loop'),
                                            is_permanent=True)
         else:
-            return super(RedirectingMemoryTransport, self).mkdir(
+            return super().mkdir(
                 relpath, mode)
 
     def get(self, relpath):
@@ -679,7 +609,7 @@ class RedirectingMemoryTransport(memory.MemoryTransport):
                                            self.abspath('../infinite-loop'),
                                            is_permanent=True)
         else:
-            return super(RedirectingMemoryTransport, self).get(relpath)
+            return super().get(relpath)
 
     def _redirected_to(self, source, target):
         # We do accept redirections
@@ -709,7 +639,7 @@ class RedirectingMemoryServer(memory.MemoryServer):
 class TestPushRedirect(tests.TestCaseWithTransport):
 
     def setUp(self):
-        super(TestPushRedirect, self).setUp()
+        super().setUp()
         self.memory_server = RedirectingMemoryServer()
         self.start_server(self.memory_server)
         # Make the branch and tree that we'll be pushing.
@@ -744,7 +674,7 @@ class TestPushRedirect(tests.TestCaseWithTransport):
         self.assertEqual('', out)
 
 
-class TestPushStrictMixin(object):
+class TestPushStrictMixin:
 
     def make_local_branch_and_tree(self):
         self.tree = self.make_branch_and_tree('local')
@@ -796,7 +726,7 @@ class TestPushStrictWithoutChanges(tests.TestCaseWithTransport,
                                    TestPushStrictMixin):
 
     def setUp(self):
-        super(TestPushStrictWithoutChanges, self).setUp()
+        super().setUp()
         self.make_local_branch_and_tree()
 
     def test_push_default(self):
@@ -834,7 +764,7 @@ class TestPushStrictWithChanges(tests.TestCaseWithTransport,
     _changes_type = None  # Set by load_tests
 
     def setUp(self):
-        super(TestPushStrictWithChanges, self).setUp()
+        super().setUp()
         # Apply the changes defined in load_tests: one of _uncommitted_changes,
         # _pending_merges or _out_of_sync_trees
         getattr(self, self._changes_type)()
@@ -901,7 +831,7 @@ class TestPushStrictWithChanges(tests.TestCaseWithTransport,
 class TestPushForeign(tests.TestCaseWithTransport):
 
     def setUp(self):
-        super(TestPushForeign, self).setUp()
+        super().setUp()
         test_foreign.register_dummy_foreign_for_test(self)
 
     def make_dummy_builder(self, relpath):
