@@ -17,13 +17,9 @@
 
 """Tests of the brz serve command."""
 
-import os
 import signal
 import sys
-try:
-    from _thread import interrupt_main
-except ImportError:  # Python < 3
-    from thread import interrupt_main
+from _thread import interrupt_main  # type: ignore
 
 import threading
 
@@ -110,7 +106,7 @@ class TestBzrServeBase(TestCaseWithTransport):
 class TestBzrServe(TestBzrServeBase):
 
     def setUp(self):
-        super(TestBzrServe, self).setUp()
+        super().setUp()
         self.disable_missing_extensions_warning()
 
     def test_server_exception_with_hook(self):
@@ -144,14 +140,14 @@ class TestBzrServe(TestBzrServeBase):
         process.stdin.close()
         # Hide stdin from the subprocess module, so it won't fail to close it.
         process.stdin = None
-        result = self.finish_bzr_subprocess(process)
+        result = self.finish_brz_subprocess(process)
         self.assertEqual(b'', result[0])
         self.assertEqual(b'', result[1])
 
     def assertServerFinishesCleanly(self, process):
         """Shutdown the brz serve instance process looking for errors."""
         # Shutdown the server
-        result = self.finish_bzr_subprocess(process, retcode=3,
+        result = self.finish_brz_subprocess(process, retcode=3,
                                             send_signal=signal.SIGINT)
         self.assertEqual(b'', result[0])
         self.assertEqual(b'brz: interrupted\n', result[1])
@@ -161,19 +157,19 @@ class TestBzrServe(TestBzrServeBase):
         with branch.lock_read():
             branch.repository.all_revision_ids()
             self.assertEqual(_mod_revision.NULL_REVISION,
-                             _mod_revision.ensure_null(branch.last_revision()))
+                             branch.last_revision())
 
     def start_server_inet(self, extra_options=()):
         """Start a brz server subprocess using the --inet option.
 
         :param extra_options: extra options to give the server.
         :return: a tuple with the brz process handle for passing to
-            finish_bzr_subprocess, a client for the server, and a transport.
+            finish_brz_subprocess, a client for the server, and a transport.
         """
         # Serve from the current directory
         args = ['serve', '--inet']
         args.extend(extra_options)
-        process = self.start_bzr_subprocess(args)
+        process = self.start_brz_subprocess(args)
 
         # Connect to the server
         # We use this url because while this is no valid URL to connect to this
@@ -190,12 +186,12 @@ class TestBzrServe(TestBzrServeBase):
 
         :param extra_options: extra options to give the server.
         :return: a tuple with the brz process handle for passing to
-            finish_bzr_subprocess, and the base url for the server.
+            finish_brz_subprocess, and the base url for the server.
         """
         # Serve from the current directory
         args = ['serve', '--listen', 'localhost', '--port', '0']
         args.extend(extra_options)
-        process = self.start_bzr_subprocess(args, skip_if_plan_to_signal=True)
+        process = self.start_brz_subprocess(args, skip_if_plan_to_signal=True)
         port_line = process.stderr.readline()
         prefix = b'listening on port: '
         self.assertStartsWith(port_line, prefix)
