@@ -23,26 +23,25 @@ interface later, they will be non blackbox tests.
 """
 
 import codecs
+from io import (
+    BytesIO,
+    StringIO,
+    )
 from os import mkdir, chdir, rmdir, unlink
 import sys
 
 from ... import (
-    conflicts,
     errors,
     osutils,
     status,
     )
 from breezy.bzr import (
     bzrdir,
+    conflicts,
     )
 import breezy.branch
 from ...osutils import pathjoin
 from ...revisionspec import RevisionSpec
-from ...sixish import (
-    BytesIO,
-    StringIO,
-    PY3,
-    )
 from ...status import show_tree_status
 from .. import TestCaseWithTransport, TestSkipped
 from ...workingtree import WorkingTree
@@ -51,7 +50,7 @@ from ...workingtree import WorkingTree
 class BranchStatus(TestCaseWithTransport):
 
     def setUp(self):
-        super(BranchStatus, self).setUp()
+        super().setUp()
         # As TestCase.setUp clears all hooks, we install this default
         # post_status hook handler for the test.
         status.hooks.install_named_hook('post_status',
@@ -178,7 +177,7 @@ class BranchStatus(TestCaseWithTransport):
         b_2_dir = b.controldir.sprout('./copy')
         b_2 = b_2_dir.open_branch()
         wt2 = b_2_dir.open_workingtree()
-        wt.commit(u"\N{TIBETAN DIGIT TWO} Empty commit 2")
+        wt.commit("\N{TIBETAN DIGIT TWO} Empty commit 2")
         wt2.merge_from_branch(wt.branch)
         message = self.status_string(wt2, verbose=True)
         self.assertStartsWith(message, "pending merges:\n")
@@ -298,20 +297,17 @@ class BranchStatus(TestCaseWithTransport):
         self.build_tree(['dir2/'])
         tree.add('dir2')
         tree.commit('added dir2')
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('foo')]))
+        tree.set_conflicts([conflicts.ContentsConflict('foo')])
         tof = BytesIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff(b'', tof.getvalue())
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('dir2')]))
+        tree.set_conflicts([conflicts.ContentsConflict('dir2')])
         tof = StringIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff('conflicts:\n  Contents conflict in dir2\n',
                              tof.getvalue())
 
-        tree.set_conflicts(conflicts.ConflictList(
-            [conflicts.ContentsConflict('dir2/file1')]))
+        tree.set_conflicts([conflicts.ContentsConflict('dir2/file1')])
         tof = StringIO()
         show_tree_status(tree, specific_files=['dir2'], to_file=tof)
         self.assertEqualDiff('conflicts:\n  Contents conflict in dir2/file1\n',
@@ -594,7 +590,7 @@ class BranchStatus(TestCaseWithTransport):
 class CheckoutStatus(BranchStatus):
 
     def setUp(self):
-        super(CheckoutStatus, self).setUp()
+        super().setUp()
         mkdir('codir')
         chdir('codir')
 
@@ -776,8 +772,8 @@ class TestStatusEncodings(TestCaseWithTransport):
 
     def make_uncommitted_tree(self):
         """Build a branch with uncommitted unicode named changes in the cwd."""
-        working_tree = self.make_branch_and_tree(u'.')
-        filename = u'hell\u00d8'
+        working_tree = self.make_branch_and_tree('.')
+        filename = 'hell\u00d8'
         try:
             self.build_tree_contents([(filename, b'contents of hello')])
         except UnicodeEncodeError:
@@ -801,10 +797,8 @@ added:
         working_tree = self.make_uncommitted_tree()
         stdout, stderr = self.run_bzr('status')
 
-        expected = u"""\
+        expected = """\
 added:
   hell\u00d8
 """
-        if not PY3:
-            expected = expected.encode('latin-1')
         self.assertEqual(stdout, expected)
