@@ -28,7 +28,6 @@ from breezy import (
     merge,
     osutils,
     urlutils,
-    transform,
     transport,
     repository,
     revision,
@@ -37,12 +36,8 @@ from breezy import (
     tree as _mod_tree,
     )
 from breezy.bzr import (
-    branch as _mod_bzrbranch,
     remote,
     )
-from breezy.sixish import (
-    text_type,
-    )
 from breezy.tests import (
     per_branch,
     )
@@ -260,7 +255,7 @@ class TestBranch(per_branch.TestCaseWithBranch):
         """
         t = self.get_transport()
         branch = self.make_branch('bzr.dev')
-        if not isinstance(branch, _mod_bzrbranch.BzrBranch):
+        if not branch.repository._format.supports_storing_branch_nick:
             raise tests.TestNotApplicable("not a bzr branch format")
         # The nick will be 'bzr.dev', because there is no explicit nick set.
         self.assertEqual(branch.nick, 'bzr.dev')
@@ -280,8 +275,8 @@ class TestBranch(per_branch.TestCaseWithBranch):
         t.move('bzr.ab', 'integration')
         branch = _mod_branch.Branch.open(self.get_url('integration'))
         self.assertEqual(branch.nick, "Aaron's branch")
-        branch.nick = u"\u1234"
-        self.assertEqual(branch.nick, u"\u1234")
+        branch.nick = "\u1234"
+        self.assertEqual(branch.nick, "\u1234")
 
     def test_nicks(self):
         """Test explicit and implicit branch nicknames.
@@ -292,14 +287,14 @@ class TestBranch(per_branch.TestCaseWithBranch):
         branch = self.make_branch('bzr.dev')
         # An implicit nick name is set; what it is exactly depends on the
         # format.
-        self.assertIsInstance(branch.nick, text_type)
+        self.assertIsInstance(branch.nick, str)
         # Set the branch nick explicitly.
-        branch.nick = u"Aaron's branch"
+        branch.nick = "Aaron's branch"
         # Because the nick has been set explicitly, the nick is now always
         # "Aaron's branch".
-        self.assertEqual(branch.nick, u"Aaron's branch")
-        branch.nick = u"\u1234"
-        self.assertEqual(branch.nick, u"\u1234")
+        self.assertEqual(branch.nick, "Aaron's branch")
+        branch.nick = "\u1234"
+        self.assertEqual(branch.nick, "\u1234")
 
     def test_commit_nicks(self):
         """Nicknames are committed to the revision"""
@@ -546,7 +541,7 @@ class ChrootedTests(per_branch.TestCaseWithBranch):
     """
 
     def setUp(self):
-        super(ChrootedTests, self).setUp()
+        super().setUp()
         if not self.vfs_transport_factory == memory.MemoryServer:
             self.transport_readonly_server = HttpServer
 
@@ -569,7 +564,7 @@ class ChrootedTests(per_branch.TestCaseWithBranch):
         self.assertEqual('g/p/q', relpath)
 
 
-class InstrumentedTransaction(object):
+class InstrumentedTransaction:
 
     def finish(self):
         self.calls.append('finish')
@@ -578,7 +573,7 @@ class InstrumentedTransaction(object):
         self.calls = []
 
 
-class TestDecorator(object):
+class TestDecorator:
 
     def __init__(self):
         self._calls = []
@@ -611,7 +606,7 @@ class TestBranchPushLocations(per_branch.TestCaseWithBranch):
     def test_get_push_location_exact(self):
         b = self.get_branch()
         config.LocationConfig.from_string(
-            '[%s]\npush_location=foo\n' % (b.base,), b.base, save=True)
+            '[{}]\npush_location=foo\n'.format(b.base), b.base, save=True)
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
@@ -722,7 +717,7 @@ class TestBound(per_branch.TestCaseWithBranch):
         branch2 = self.make_branch('2')
         try:
             branch.bind(branch2)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Format does not support binding')
         self.assertTrue(branch.unbind())
         self.assertFalse(branch.unbind())
@@ -750,7 +745,7 @@ class TestBound(per_branch.TestCaseWithBranch):
         tree_b.commit('rev2b')
         try:
             tree_b.branch.bind(tree_a.branch)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Format does not support binding')
 
     def test_unbind_clears_cached_master_branch(self):
@@ -759,7 +754,7 @@ class TestBound(per_branch.TestCaseWithBranch):
         branch = self.make_branch('branch')
         try:
             branch.bind(master)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Format does not support binding')
         self.addCleanup(branch.lock_write().unlock)
         self.assertNotEqual(None, branch.get_master_branch())
@@ -773,7 +768,7 @@ class TestBound(per_branch.TestCaseWithBranch):
         branch = self.make_branch('branch')
         try:
             branch.bind(master1)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Format does not support binding')
         self.addCleanup(branch.lock_write().unlock)
         self.assertNotEqual(None, branch.get_master_branch())
@@ -789,7 +784,7 @@ class TestBound(per_branch.TestCaseWithBranch):
         branch = self.make_branch('branch')
         try:
             branch.bind(master1)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Format does not support binding')
         self.addCleanup(branch.lock_write().unlock)
         self.assertNotEqual(None, branch.get_master_branch())
@@ -857,14 +852,14 @@ class TestBranchControlComponent(per_branch.TestCaseWithBranch):
         self.assertEqual(br.control_url, br.control_transport.base)
 
 
-class FakeShelfCreator(object):
+class FakeShelfCreator:
 
     def __init__(self, branch):
         self.branch = branch
 
     def write_shelf(self, shelf_file, message=None):
         tree = self.branch.repository.revision_tree(revision.NULL_REVISION)
-        with transform.TransformPreview(tree) as tt:
+        with tree.preview_transform() as tt:
             shelf.ShelfCreator._write_shelf(
                 shelf_file, tt, revision.NULL_REVISION)
 
@@ -880,7 +875,7 @@ def skip_if_storing_uncommitted_unsupported():
 class TestUncommittedChanges(per_branch.TestCaseWithBranch):
 
     def setUp(self):
-        super(TestUncommittedChanges, self).setUp()
+        super().setUp()
         if not self.branch_format.supports_store_uncommitted():
             raise tests.TestNotApplicable(
                 'Branch format does not support store_uncommitted')
@@ -888,7 +883,7 @@ class TestUncommittedChanges(per_branch.TestCaseWithBranch):
     def bind(self, branch, master):
         try:
             branch.bind(master)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise tests.TestNotApplicable('Branch cannot be bound.')
 
     def test_store_uncommitted(self):
