@@ -24,6 +24,7 @@ from breezy import (
     errors,
     gpg,
     osutils,
+    repository as _mod_repository,
     revision as _mod_revision,
     transport,
     ui,
@@ -198,7 +199,7 @@ class TestControlDir(TestCaseWithControlDir):
                                     "control directories without working tree")
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
 
-    def test_clone_bzrdir_repository_under_shared(self):
+    def test_clone_controldir_repository_under_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(
             ['foo'], transport=tree.controldir.transport.clone('..'))
@@ -220,7 +221,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
 
-    def test_clone_bzrdir_repository_branch_both_under_shared(self):
+    def test_clone_controldir_repository_branch_both_under_shared(self):
         # Create a shared repository
         try:
             shared_repo = self.make_repository('shared', shared=True)
@@ -248,7 +249,7 @@ class TestControlDir(TestCaseWithControlDir):
         dir.create_branch()
         # Clone 'source' to 'target', also inside the shared repository.
         target = dir.clone(self.get_url('shared/target'))
-        # 'source', 'target', and the shared repo all have distinct bzrdirs.
+        # 'source', 'target', and the shared repo all have distinct controldirs.
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(
             dir.transport.base, shared_repo.controldir.transport.base)
@@ -257,7 +258,7 @@ class TestControlDir(TestCaseWithControlDir):
         # 'commit_tree' branch.
         self.assertTrue(shared_repo.has_revision(rev1))
 
-    def test_clone_bzrdir_repository_branch_only_source_under_shared(self):
+    def test_clone_controldir_repository_branch_only_source_under_shared(self):
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
@@ -290,7 +291,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.assertFalse(branch.repository.make_working_trees())
         self.assertTrue(branch.repository.is_shared())
 
-    def test_clone_bzrdir_repository_revision(self):
+    def test_clone_controldir_repository_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and clone it with a revision limit.
@@ -309,7 +310,7 @@ class TestControlDir(TestCaseWithControlDir):
         dir.clone(self.get_url('target'), revision_id=rev2)
         raise TestSkipped('revision limiting not strict yet')
 
-    def test_clone_bzrdir_branch_and_repo_fixed_user_id(self):
+    def test_clone_controldir_branch_and_repo_fixed_user_id(self):
         # Bug #430868 is about an email containing '.sig'
         self.overrideEnv('BRZ_EMAIL', 'murphy@host.sighup.org')
         tree = self.make_branch_and_tree('commit_tree')
@@ -319,11 +320,8 @@ class TestControlDir(TestCaseWithControlDir):
         tree_repo = tree.branch.repository
         if not tree_repo._format.supports_revision_signatures:
             self.skipTest('repository format does not support signing')
-        tree_repo.lock_write()
-        tree_repo.start_write_group()
-        tree_repo.sign_revision(rev1, gpg.LoopbackGPGStrategy(None))
-        tree_repo.commit_write_group()
-        tree_repo.unlock()
+        with tree_repo.lock_write(), _mod_repository.WriteGroup(tree_repo):
+            tree_repo.sign_revision(rev1, gpg.LoopbackGPGStrategy(None))
         target = self.make_branch('target')
         tree.branch.repository.copy_content_into(target.repository)
         tree.branch.copy_content_into(target)
@@ -332,7 +330,7 @@ class TestControlDir(TestCaseWithControlDir):
             tree_repo.get_signature_text(rev1),
             target.repository.get_signature_text(rev1))
 
-    def test_clone_bzrdir_branch_and_repo_into_shared_repo(self):
+    def test_clone_controldir_branch_and_repo_into_shared_repo(self):
         # by default cloning into a shared repo uses the shared repo.
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
@@ -356,7 +354,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.assertEqual(source.last_revision(),
                          target.open_branch().last_revision())
 
-    def test_clone_bzrdir_branch_revision(self):
+    def test_clone_controldir_branch_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a branch with some revisions,
         # and clone it with a revision limit.
@@ -373,6 +371,23 @@ class TestControlDir(TestCaseWithControlDir):
         target = dir.clone(self.get_url('target'), revision_id=rev1)
         self.assertEqual(rev1, target.open_branch().last_revision())
 
+    def test_clone_controldir_with_colocated(self):
+        if not self.bzrdir_format.colocated_branches:
+            raise TestNotApplicable(
+                'format does not supported colocated branches')
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        rev1 = tree.commit('revision 1')
+        rev2 = tree.commit('revision 2', allow_pointless=True)
+        rev3 = tree.commit('revision 2', allow_pointless=True)
+        dir = tree.branch.controldir
+        colo = dir.create_branch(name='colo')
+        colo.pull(tree.branch, stop_revision=rev1)
+        target = dir.clone(self.get_url('target'), revision_id=rev2)
+        self.assertEqual(rev2, target.open_branch().last_revision())
+        self.assertEqual(rev1, target.open_branch(name='colo').last_revision())
+
     def test_clone_on_transport_preserves_repo_format(self):
         if self.bzrdir_format == controldir.format_registry.make_controldir('default'):
             format = 'knit'
@@ -383,8 +398,8 @@ class TestControlDir(TestCaseWithControlDir):
         a_dir = breezy.branch.Branch.open_from_transport(
             self.get_transport('source')).controldir
         target_transport = self.get_transport('target')
-        target_bzrdir = a_dir.clone_on_transport(target_transport)
-        target_repo = target_bzrdir.open_repository()
+        target_controldir = a_dir.clone_on_transport(target_transport)
+        target_repo = target_controldir.open_repository()
         source_branch = breezy.branch.Branch.open(
             self.get_vfs_only_url('source'))
         if isinstance(target_repo, RemoteRepository):
@@ -392,7 +407,7 @@ class TestControlDir(TestCaseWithControlDir):
             target_repo = target_repo._real_repository
         self.assertEqual(target_repo._format, source_branch.repository._format)
 
-    def test_clone_bzrdir_tree_revision(self):
+    def test_clone_controldir_tree_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
         # and clone it with a revision limit.
@@ -408,7 +423,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.skipIfNoWorkingTree(target)
         self.assertEqual([rev1], target.open_workingtree().get_parent_ids())
 
-    def test_clone_bzrdir_into_notrees_repo(self):
+    def test_clone_controldir_into_notrees_repo(self):
         """Cloning into a no-trees repo should not create a working tree"""
         tree = self.make_branch_and_tree('source')
         self.build_tree(['source/foo'])
@@ -516,12 +531,12 @@ class TestControlDir(TestCaseWithControlDir):
         """get_branch_reference should not mask NotBranchErrors."""
         dir = self.make_controldir('source')
         if dir.has_branch():
-            # this format does not support branchless bzrdirs.
+            # this format does not support branchless controldirs.
             raise TestNotApplicable("format does not support "
                                     "branchless control directories")
         self.assertRaises(errors.NotBranchError, dir.get_branch_reference)
 
-    def test_sprout_bzrdir_empty(self):
+    def test_sprout_controldir_empty(self):
         dir = self.make_controldir('source')
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.control_transport.base,
@@ -531,7 +546,7 @@ class TestControlDir(TestCaseWithControlDir):
         target.open_branch()
         self.openWorkingTreeIfLocal(target)
 
-    def test_sprout_bzrdir_empty_under_shared_repo(self):
+    def test_sprout_controldir_empty_under_shared_repo(self):
         # sprouting an empty dir into a repo uses the repo
         dir = self.make_controldir('source')
         try:
@@ -545,13 +560,13 @@ class TestControlDir(TestCaseWithControlDir):
         try:
             target.open_workingtree()
         except errors.NoWorkingTree:
-            # Some bzrdirs can never have working trees.
+            # Some controldirs can never have working trees.
             repo = target.find_repository()
             self.assertFalse(repo.controldir._format.supports_workingtrees)
 
-    def test_sprout_bzrdir_empty_under_shared_repo_force_new(self):
+    def test_sprout_controldir_empty_under_shared_repo_force_new(self):
         # the force_new_repo parameter should force use of a new repo in an empty
-        # bzrdir's sprout logic
+        # controldir's sprout logic
         dir = self.make_controldir('source')
         try:
             self.make_repository('target', shared=True)
@@ -563,7 +578,7 @@ class TestControlDir(TestCaseWithControlDir):
         target.open_branch()
         self.openWorkingTreeIfLocal(target)
 
-    def test_sprout_bzrdir_with_repository_to_shared(self):
+    def test_sprout_controldir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -585,7 +600,7 @@ class TestControlDir(TestCaseWithControlDir):
                             target.user_transport.base)
         self.assertTrue(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_repository_branch_both_under_shared(self):
+    def test_sprout_controldir_repository_branch_both_under_shared(self):
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
@@ -611,7 +626,7 @@ class TestControlDir(TestCaseWithControlDir):
                             shared_repo.controldir.transport.base)
         self.assertTrue(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_repository_branch_only_source_under_shared(self):
+    def test_sprout_controldir_repository_branch_only_source_under_shared(self):
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
@@ -641,7 +656,7 @@ class TestControlDir(TestCaseWithControlDir):
             dir.transport.base,
             shared_repo.controldir.transport.base)
         branch = target.open_branch()
-        # The sprouted bzrdir has a branch, so only revisions referenced by
+        # The sprouted controldir has a branch, so only revisions referenced by
         # that branch are copied, rather than the whole repository.  It's an
         # empty branch, so none are copied.
         self.assertEqual([], branch.repository.all_revision_ids())
@@ -649,7 +664,7 @@ class TestControlDir(TestCaseWithControlDir):
             self.assertTrue(branch.repository.make_working_trees())
         self.assertFalse(branch.repository.is_shared())
 
-    def test_sprout_bzrdir_repository_under_shared_force_new_repo(self):
+    def test_sprout_controldir_repository_under_shared_force_new_repo(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -672,7 +687,7 @@ class TestControlDir(TestCaseWithControlDir):
             target.control_transport.base)
         self.assertFalse(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_repository_revision(self):
+    def test_sprout_controldir_repository_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and sprout it with a revision limit.
@@ -691,7 +706,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.sproutOrSkip(dir, self.get_url('target'), revision_id=rev2)
         raise TestSkipped('revision limiting not strict yet')
 
-    def test_sprout_bzrdir_branch_and_repo_shared(self):
+    def test_sprout_controldir_branch_and_repo_shared(self):
         # sprouting a branch with a repo into a shared repo uses the shared
         # repo
         tree = self.make_branch_and_tree('commit_tree')
@@ -710,7 +725,7 @@ class TestControlDir(TestCaseWithControlDir):
         dir.sprout(self.get_url('target/child'))
         self.assertTrue(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_branch_and_repo_shared_force_new_repo(self):
+    def test_sprout_controldir_branch_and_repo_shared_force_new_repo(self):
         # sprouting a branch with a repo into a shared repo uses the shared
         # repo
         tree = self.make_branch_and_tree('commit_tree')
@@ -731,7 +746,7 @@ class TestControlDir(TestCaseWithControlDir):
             dir.control_transport.base, target.control_transport.base)
         self.assertFalse(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_branch_reference(self):
+    def test_sprout_controldir_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
         referenced_branch = self.make_branch('referenced')
         dir = self.make_controldir('source')
@@ -749,7 +764,7 @@ class TestControlDir(TestCaseWithControlDir):
         # place
         target.open_repository()
 
-    def test_sprout_bzrdir_branch_reference_shared(self):
+    def test_sprout_controldir_branch_reference_shared(self):
         # sprouting should create a repository if needed and a sprouted branch.
         referenced_tree = self.make_branch_and_tree('referenced')
         rev1 = referenced_tree.commit('1', allow_pointless=True)
@@ -775,7 +790,7 @@ class TestControlDir(TestCaseWithControlDir):
         # and we want revision '1' in the shared repo
         self.assertTrue(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_branch_reference_shared_force_new_repo(self):
+    def test_sprout_controldir_branch_reference_shared_force_new_repo(self):
         # sprouting should create a repository if needed and a sprouted branch.
         referenced_tree = self.make_branch_and_tree('referenced')
         rev1 = referenced_tree.commit('1', allow_pointless=True)
@@ -801,7 +816,7 @@ class TestControlDir(TestCaseWithControlDir):
         # but not the shared one
         self.assertFalse(shared_repo.has_revision(rev1))
 
-    def test_sprout_bzrdir_branch_revision(self):
+    def test_sprout_controldir_branch_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and sprout it with a revision limit.
@@ -818,7 +833,7 @@ class TestControlDir(TestCaseWithControlDir):
         target = dir.sprout(self.get_url('target'), revision_id=rev1)
         self.assertEqual(rev1, target.open_branch().last_revision())
 
-    def test_sprout_bzrdir_branch_with_tags(self):
+    def test_sprout_controldir_branch_with_tags(self):
         # when sprouting a branch all revisions named in the tags are copied
         # too.
         builder = self.make_branch_builder('source')
@@ -837,7 +852,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.assertEqual(rev2, new_branch.tags.lookup_tag('tag-a'))
         new_branch.repository.get_revision(rev2)
 
-    def test_sprout_bzrdir_branch_with_absent_tag(self):
+    def test_sprout_controldir_branch_with_absent_tag(self):
         # tags referencing absent revisions are copied (and those absent
         # revisions do not prevent the sprout.)
         builder = self.make_branch_builder('source')
@@ -857,7 +872,7 @@ class TestControlDir(TestCaseWithControlDir):
         new_branch = target.open_branch()
         self.assertEqual(b'missing-rev', new_branch.tags.lookup_tag('tag-a'))
 
-    def test_sprout_bzrdir_passing_source_branch_with_absent_tag(self):
+    def test_sprout_controldir_passing_source_branch_with_absent_tag(self):
         # tags referencing absent revisions are copied (and those absent
         # revisions do not prevent the sprout.)
         builder = self.make_branch_builder('source')
@@ -877,9 +892,9 @@ class TestControlDir(TestCaseWithControlDir):
         new_branch = target.open_branch()
         self.assertEqual(b'missing-rev', new_branch.tags.lookup_tag('tag-a'))
 
-    def test_sprout_bzrdir_passing_rev_not_source_branch_copies_tags(self):
+    def test_sprout_controldir_passing_rev_not_source_branch_copies_tags(self):
         # dir.sprout(..., revision_id=b'rev1') copies rev1, and all the tags of
-        # the branch at that bzrdir, the ancestry of all of those, but no other
+        # the branch at that controldir, the ancestry of all of those, but no other
         # revs (not even the tip of the source branch).
         builder = self.make_branch_builder('source')
         base_rev = builder.build_commit(message="Base")
@@ -929,7 +944,7 @@ class TestControlDir(TestCaseWithControlDir):
             sorted([base_rev, rev_b1, rev_b2, rev_c1, rev_c2]),
             sorted(new_branch.repository.all_revision_ids()))
 
-    def test_sprout_bzrdir_tree_branch_reference(self):
+    def test_sprout_controldir_tree_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
         # the tree state should not be copied.
         referenced_branch = self.make_branch('referencced')
@@ -954,7 +969,7 @@ class TestControlDir(TestCaseWithControlDir):
         result_tree = target.open_workingtree()
         self.assertFalse(result_tree.has_filename('subdir'))
 
-    def test_sprout_bzrdir_tree_branch_reference_revision(self):
+    def test_sprout_controldir_tree_branch_reference_revision(self):
         # sprouting should create a repository if needed and a sprouted branch.
         # the tree state should not be copied but the revision changed,
         # and the likewise the new branch should be truncated too
@@ -984,7 +999,7 @@ class TestControlDir(TestCaseWithControlDir):
         self.assertEqual([rev1], target.open_workingtree().get_parent_ids())
         self.assertEqual(rev1, target.open_branch().last_revision())
 
-    def test_sprout_bzrdir_tree_revision(self):
+    def test_sprout_controldir_tree_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
         # and sprout it with a revision limit.
@@ -1036,13 +1051,13 @@ class TestControlDir(TestCaseWithControlDir):
         rev3 = builder.build_commit(message='Rev 3.')
         builder.finish_series()
         stack_on = builder.get_branch()
-        # Make a bzrdir with a default stacking policy to stack on that branch.
+        # Make a controldir with a default stacking policy to stack on that branch.
         config = self.make_controldir('policy-dir').get_config()
         try:
             config.set_default_stack_on(self.get_url('stack-on'))
         except errors.BzrError:
             raise TestNotApplicable('Only relevant for stackable formats.')
-        # Sprout the stacked-on branch into the bzrdir.
+        # Sprout the stacked-on branch into the controldir.
         sprouted = stack_on.controldir.sprout(
             self.get_url('policy-dir/sprouted'), revision_id=rev3)
         # Not all revisions are copied into the sprouted repository.
@@ -1096,7 +1111,7 @@ class TestControlDir(TestCaseWithControlDir):
         t = self.get_transport('dir')
         t.ensure_base()
         self.assertRaises(
-            errors.FileExists,
+            transport.FileExists,
             self.bzrdir_format.initialize_on_transport_ex, t,
             use_existing_dir=False)
 
@@ -1109,7 +1124,7 @@ class TestControlDir(TestCaseWithControlDir):
             raise TestNotApplicable("format is not initializable")
         t = self.get_transport('missing/dir')
         self.assertRaises(
-            errors.NoSuchFile, self.assertInitializeEx, t,
+            transport.NoSuchFile, self.assertInitializeEx, t,
             create_prefix=False)
 
     def test_format_initialize_on_transport_ex_force_new_repo_True(self):
@@ -1290,6 +1305,11 @@ class TestControlDir(TestCaseWithControlDir):
         repo.controldir.create_branch()
         self.assertEqual([""], list(repo.controldir.get_branches()))
 
+    def test_branch_names(self):
+        repo = self.make_repository('branch-1')
+        repo.controldir.create_branch()
+        self.assertEqual([""], repo.controldir.branch_names())
+
     def test_create_repository(self):
         # a bzrdir can construct a repository for itself.
         if not self.bzrdir_format.is_initializable():
@@ -1403,7 +1423,7 @@ class TestControlDir(TestCaseWithControlDir):
             made_tree = made_control.create_workingtree()
         except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped(
-                "Can't initialize %r on transport %r" % (
+                "Can't initialize {!r} on transport {!r}".format(
                     self.bzrdir_format, t))
         opened_tree = made_control.open_workingtree()
         self.assertEqual(made_control, opened_tree.controldir)
@@ -1437,7 +1457,7 @@ class TestControlDir(TestCaseWithControlDir):
             raise TestSkipped("Can't initialize %r on transport %r"
                               % (self.bzrdir_format, t))
         dir = controldir.ControlDir.open(t.base)
-        self.assertEqual(u"", dir._get_selected_branch())
+        self.assertEqual("", dir._get_selected_branch())
 
     def test_root_transport(self):
         dir = self.make_controldir('.')
