@@ -224,6 +224,8 @@ class TestInterRepository(TestCaseWithInterRepository):
         builder.finish_series()
         branch = builder.get_branch()
         revtree = branch.repository.revision_tree(merge)
+        if not revtree.supports_file_ids:
+            raise TestNotApplicable('from format does not support file ids')
         root_id = revtree.path2id('')
         file_id = revtree.path2id('file')
 
@@ -493,7 +495,7 @@ class TestInterRepository(TestCaseWithInterRepository):
         # start by adding a file so the data knit for the file exists in
         # repositories that have specific files for each fileid.
         self.build_tree(['source/id'])
-        source_tree.add(['id'], [b'id'])
+        source_tree.add(['id'], ids=[b'id'])
         source_tree.commit('a', rev_id=b'a')
         # now we manually insert a revision with an inventory referencing
         # file 'id' at revision 'b', but we do not insert revision b.
@@ -520,7 +522,7 @@ class TestInterRepository(TestCaseWithInterRepository):
             self.assertRaises(errors.RevisionNotPresent, target.fetch, source)
         except errors.NoRoundtrippingSupport:
             raise TestNotApplicable('roundtripping not supported')
-        self.assertFalse(target.has_revision('b'))
+        self.assertFalse(target.has_revision(b'b'))
 
     def test_fetch_funky_file_id(self):
         from_tree = self.make_branch_and_tree('tree')
@@ -531,7 +533,7 @@ class TestInterRepository(TestCaseWithInterRepository):
         if not from_tree.supports_setting_file_ids():
             raise TestNotApplicable(
                 'from tree format can not create custom file ids')
-        from_tree.add('filename', b'funky-chars<>%&;"\'')
+        from_tree.add('filename', ids=b'funky-chars<>%&;"\'')
         from_tree.commit('commit filename')
         to_repo = self.make_to_repository('to')
         try:
@@ -582,7 +584,8 @@ class TestFetchDependentData(TestCaseWithInterRepository):
         to_repo.fetch(from_tree.branch.repository, tree_rev)
         # to_repo should have a file_graph for from_tree.path2id('subtree') and
         # revid tree_rev.
-        file_id = from_tree.path2id('subtree')
-        with to_repo.lock_read():
-            self.assertEqual({(file_id, tree_rev): ()},
-                             to_repo.texts.get_parent_map([(file_id, tree_rev)]))
+        if from_tree.supports_file_ids:
+            file_id = from_tree.path2id('subtree')
+            with to_repo.lock_read():
+                self.assertEqual({(file_id, tree_rev): ()},
+                                 to_repo.texts.get_parent_map([(file_id, tree_rev)]))
