@@ -16,6 +16,7 @@
 
 """Tests for repository implementations - tests a repository format."""
 
+from io import BytesIO
 import re
 
 from ... import (
@@ -42,11 +43,6 @@ from ...bzr import (
 from ...bzr import (
     knitpack_repo,
     )
-from ...sixish import (
-    BytesIO,
-    text_type,
-    unichr,
-    )
 from .. import (
     per_repository,
     test_server,
@@ -87,6 +83,11 @@ class TestRepository(per_repository.TestCaseWithRepository):
         self.assertFormatAttribute('supports_nesting_repositories',
                                    (True, False))
 
+    def test_attribute_supports_multiple_authors(self):
+        """Test the format.supports_multiple_authors."""
+        self.assertFormatAttribute('supports_multiple_authors',
+                                   (True, False))
+
     def test_attribute_supports_unreferenced_revisions(self):
         """Test the format.supports_unreferenced_revisions."""
         self.assertFormatAttribute('supports_unreferenced_revisions',
@@ -173,7 +174,6 @@ class TestRepository(per_repository.TestCaseWithRepository):
         tree_a = self.make_branch_and_tree('a')
         self.build_tree(['a/foo'])
         tree_a.add('foo')
-        file_id = tree_a.path2id('foo')
         rev1 = tree_a.commit('rev1')
         bzrdirb = self.make_controldir('b')
         repo_b = tree_a.branch.repository.clone(bzrdirb)
@@ -190,7 +190,7 @@ class TestRepository(per_repository.TestCaseWithRepository):
         rev_tree = tree.branch.repository.revision_tree(second_revision)
         rev_tree.lock_read()
         self.addCleanup(rev_tree.unlock)
-        root_revision = rev_tree.get_file_revision(u'')
+        root_revision = rev_tree.get_file_revision('')
         rich_root = (root_revision != second_revision)
         self.assertEqual(rich_root,
                          tree.branch.repository.supports_rich_root())
@@ -296,14 +296,11 @@ class TestRepository(per_repository.TestCaseWithRepository):
     def test_revision_tree(self):
         wt = self.make_branch_and_tree('.')
         rev1 = wt.commit('lala!', allow_pointless=True)
-        root_id = wt.path2id('')
         tree = wt.branch.repository.revision_tree(rev1)
         with tree.lock_read():
-            self.assertEqual(rev1, tree.get_file_revision(u''))
-            expected = inventory.InventoryDirectory(root_id, '', None)
-            expected.revision = rev1
-            self.assertEqual([('', 'V', 'directory', expected)],
-                             list(tree.list_files(include_root=True)))
+            self.assertEqual(rev1, tree.get_file_revision(''))
+            [root] = list(tree.list_files(include_root=True))
+            self.assertEqual(('', 'V', 'directory'), root[:3])
         self.assertRaises(ValueError, wt.branch.repository.revision_tree, None)
         tree = wt.branch.repository.revision_tree(_mod_revision.NULL_REVISION)
         with tree.lock_read():
@@ -412,7 +409,7 @@ class TestRepository(per_repository.TestCaseWithRepository):
             # we have to manually escape this as we dont try to
             # roundtrip xml invalid characters in the xml-based serializers.
             escaped_message, escape_count = re.subn(
-                u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
+                '[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
                 lambda match: match.group(0).encode(
                     'unicode_escape').decode('ascii'),
                 message)
@@ -421,20 +418,20 @@ class TestRepository(per_repository.TestCaseWithRepository):
             self.assertEqual(rev.message, message)
         # insist the class is unicode no matter what came in for
         # consistency.
-        self.assertIsInstance(rev.message, text_type)
+        self.assertIsInstance(rev.message, str)
 
     def test_commit_unicode_message(self):
         # a siple unicode message should be preserved
-        self.assertMessageRoundtrips(u'foo bar gamm\xae plop')
+        self.assertMessageRoundtrips('foo bar gamm\xae plop')
 
     def test_commit_unicode_control_characters(self):
         # a unicode message with control characters should roundtrip too.
-        unichars = [unichr(x) for x in range(256)]
+        unichars = [chr(x) for x in range(256)]
         # '\r' is not directly allowed anymore, as it used to be translated
         # into '\n' anyway
-        unichars[ord('\r')] = u'\n'
+        unichars[ord('\r')] = '\n'
         self.assertMessageRoundtrips(
-            u"All 8-bit chars: " + ''.join(unichars))
+            "All 8-bit chars: " + ''.join(unichars))
 
     def test_check_repository(self):
         """Check a fairly simple repository's history"""
@@ -470,8 +467,8 @@ class TestRepository(per_repository.TestCaseWithRepository):
         revision_ids = [a_rev, c_rev, b_rev, d_rev]
         revid_with_rev = repo.iter_revisions(revision_ids)
         self.assertEqual(
-            set((revid, rev.revision_id if rev is not None else None)
-                for (revid, rev) in revid_with_rev),
+            {(revid, rev.revision_id if rev is not None else None)
+                for (revid, rev) in revid_with_rev},
             {(a_rev, a_rev),
              (b_rev, b_rev),
              (c_rev, c_rev),
@@ -483,7 +480,7 @@ class TestRepository(per_repository.TestCaseWithRepository):
         rev_tree = tree.branch.repository.revision_tree(tree.last_revision())
         rev_tree.lock_read()
         self.addCleanup(rev_tree.unlock)
-        self.assertEqual(revid, rev_tree.get_file_revision(u''))
+        self.assertEqual(revid, rev_tree.get_file_revision(''))
 
     def test_pointless_commit(self):
         tree = self.make_branch_and_tree('.')
@@ -504,6 +501,8 @@ class TestRepository(per_repository.TestCaseWithRepository):
         self.build_tree_contents([('tree/file1', b'foo'),
                                   ('tree/file2', b'bar')])
         tree.add(['file1', 'file2'])
+        if not tree.supports_file_ids:
+            raise tests.TestNotApplicable('tree does not support file ids')
         file1_id = tree.path2id('file1')
         file2_id = tree.path2id('file2')
         rev1 = tree.commit('rev1')
@@ -512,12 +511,12 @@ class TestRepository(per_repository.TestCaseWithRepository):
         repository = tree.branch.repository
         repository.lock_read()
         self.addCleanup(repository.unlock)
-        extracted = dict((i, b''.join(b)) for i, b in
+        extracted = {i: b''.join(b) for i, b in
                          repository.iter_files_bytes(
                          [(file1_id, rev1, 'file1-old'),
                           (file1_id, rev2, 'file1-new'),
                           (file2_id, rev1, 'file2'),
-                          ]))
+                          ])}
         self.assertEqual(b'foo', extracted['file1-old'])
         self.assertEqual(b'bar', extracted['file2'])
         self.assertEqual(b'baz', extracted['file1-new'])
@@ -964,7 +963,7 @@ class TestEscaping(tests.TestCaseWithTransport):
             self.skip("format does not support setting file ids")
         self.build_tree(["repo/foo"], line_endings='binary')
         # add file with id containing wierd characters
-        wt.add(['foo'], [FOO_ID])
+        wt.add(['foo'], ids=[FOO_ID])
         rev1 = wt.commit('this is my new commit')
         # now access over vfat; should be safe
         branch = controldir.ControlDir.open(self.get_url('repo')).open_branch()
@@ -997,27 +996,27 @@ class TestRepositoryControlComponent(per_repository.TestCaseWithRepository):
         self.assertEqual(repo.control_url, repo.control_transport.base)
 
 
-class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
+class TestDeltaRevisionFilesFiltered(per_repository.TestCaseWithRepository):
 
     def setUp(self):
-        super(TestDeltaRevisionFiltered, self).setUp()
+        super().setUp()
         self.tree_a = self.make_branch_and_tree('a')
-        self.build_tree(['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz'])
-        self.tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz'])
-        self.bar_id = self.tree_a.path2id('bar')
+        self.build_tree(
+            ['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz', 'a/oldname'])
+        self.tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz', 'oldname'])
         self.rev1 = self.tree_a.commit('rev1')
         self.build_tree(['a/bar/b3'])
         self.tree_a.add('bar/b3')
+        self.tree_a.rename_one('oldname', 'newname')
         self.rev2 = self.tree_a.commit('rev2')
         self.repository = self.tree_a.branch.repository
         self.addCleanup(self.repository.lock_read().unlock)
 
     def test_multiple_files(self):
         # Test multiple files
-        delta = list(self.repository.get_deltas_for_revisions(
-            [self.repository.get_revision(self.rev1)], specific_fileids=[
-                self.tree_a.path2id('foo'),
-                self.tree_a.path2id('baz')]))[0]
+        delta = list(self.repository.get_revision_deltas(
+            [self.repository.get_revision(self.rev1)], specific_files=[
+                'foo', 'baz']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('baz', 'file'),
@@ -1026,9 +1025,9 @@ class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
 
     def test_directory(self):
         # Test a directory
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev1)],
-            specific_fileids=[self.bar_id]))[0]
+            specific_files=['bar']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('bar', 'directory'),
@@ -1038,17 +1037,33 @@ class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
 
     def test_unrelated(self):
         # Try another revision
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev2)],
-            specific_fileids=[self.tree_a.path2id('foo')]))[0]
+            specific_files=['foo']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([], delta.added)
 
+    def test_renamed(self):
+        # Try another revision
+        self.assertTrue(
+            self.repository.revision_tree(self.rev2).has_filename('newname'))
+        self.assertTrue(
+            self.repository.revision_tree(self.rev1).has_filename('oldname'))
+        revs = [
+            self.repository.get_revision(self.rev2),
+            self.repository.get_revision(self.rev1)]
+        delta2, delta1 = list(self.repository.get_revision_deltas(
+            revs, specific_files=['newname']))
+        self.assertIsInstance(delta1, _mod_delta.TreeDelta)
+        self.assertEqual([('oldname', 'newname')], [c.path for c in delta2.renamed])
+        self.assertIsInstance(delta2, _mod_delta.TreeDelta)
+        self.assertEqual(['oldname'], [c.path[1] for c in delta1.added])
+
     def test_file_in_directory(self):
         # Test a file in a directory, both of which were added
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev1)],
-            specific_fileids=[self.tree_a.path2id('bar/b2')]))[0]
+            specific_files=['bar/b2']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('bar', 'directory'),
@@ -1056,9 +1071,9 @@ class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
             ], [(c.path[1], c.kind[1]) for c in delta.added])
 
     def test_file_in_unchanged_directory(self):
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev2)],
-            specific_fileids=[self.tree_a.path2id('bar/b3')]))[0]
+            specific_files=['bar/b3']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         if [(c.path[1], c.kind[1]) for c in delta.added] == [
                 ('bar', 'directory'), ('bar/b3', 'file')]:
