@@ -30,9 +30,10 @@ from ... import (
     revision as _mod_revision,
     tests,
     trace,
+    transport as _mod_transport,
     urlutils,
     )
-from...bzr import (
+from ...bzr import (
     bzrdir,
     )
 from ...errors import (
@@ -41,7 +42,7 @@ from ...errors import (
     )
 from ...bzr.inventory import Inventory
 from ...mutabletree import MutableTree
-from ...osutils import pathjoin, getcwd, has_symlinks
+from ...osutils import pathjoin, getcwd, supports_symlinks
 from .. import (
     features,
     TestSkipped,
@@ -83,7 +84,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
     def test_list_files(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/', 'file'])
-        if has_symlinks():
+        if supports_symlinks(self.test_dir):
             os.symlink('target', 'symlink')
         tree.lock_read()
         files = list(tree.list_files())
@@ -91,7 +92,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.assertEqual(
             files.pop(0), ('dir', '?', 'directory', TreeDirectory()))
         self.assertEqual(files.pop(0), ('file', '?', 'file', TreeFile()))
-        if has_symlinks():
+        if supports_symlinks(self.test_dir):
             self.assertEqual(
                 files.pop(0), ('symlink', '?', 'symlink', TreeLink()))
 
@@ -183,7 +184,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.assertEqual(wt.basedir + '/', local_base)
 
         # '.' opens this dir
-        wt, relpath = WorkingTree.open_containing(u'.')
+        wt, relpath = WorkingTree.open_containing('.')
         self.assertEqual('', relpath)
         self.assertEqual(wt.basedir + '/', local_base)
 
@@ -352,7 +353,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
     def test_add_missing(self):
         # adding a msising file -> NoSuchFile
         wt = self.make_branch_and_tree('.')
-        self.assertRaises(errors.NoSuchFile, wt.add, 'fpp')
+        self.assertRaises(_mod_transport.NoSuchFile, wt.add, 'fpp')
 
     def test_remove_verbose(self):
         # FIXME the remove api should not print or otherwise depend on the
@@ -570,12 +571,13 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # and update checkout
         self.assertEqual(0, checkout.update())
         self.assertPathExists('checkout/file')
-        self.assertEqual(wt.path2id(''), checkout.path2id(''))
-        self.assertNotEqual(None, wt.path2id(''))
+        if wt.supports_setting_file_ids():
+            self.assertEqual(wt.path2id(''), checkout.path2id(''))
+            self.assertNotEqual(None, wt.path2id(''))
 
     def test_update_sets_updated_root_id(self):
         wt = self.make_branch_and_tree('tree')
-        if not wt._format.supports_setting_file_ids:
+        if not wt.supports_setting_file_ids():
             self.assertRaises(SettingFileIdUnsupported, wt.set_root_id,
                               'first_root_id')
             return
@@ -662,7 +664,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree = self.make_branch_and_tree('tree')
         try:
             tree.branch.bind(master_tree.branch)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             # legacy branches cannot bind
             return
         foo = master_tree.commit('foo', allow_pointless=True)
@@ -685,7 +687,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree = self.make_branch_and_tree('tree')
         try:
             tree.branch.bind(master_tree.branch)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             # legacy branches cannot bind
             return
         # sync with master
@@ -861,31 +863,30 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # ensure that foo.pyc is ignored
         self.build_tree_contents([('.bzrignore', b'foo.pyc')])
         tree.add('foo.pyc')
-        anid = tree.path2id('foo.pyc')
         tree.lock_read()
         files = sorted(list(tree.list_files()))
         tree.unlock()
         self.assertEqual(
-            (u'.bzrignore', '?', 'file', None),
+            ('.bzrignore', '?', 'file', None),
             (files[0][0], files[0][1], files[0][2],
                 getattr(files[0][3], 'file_id', None)))
         self.assertEqual(
-            (u'foo.pyc', 'V', 'file', anid),
-            (files[1][0], files[1][1], files[1][2], files[1][3].file_id))
+            ('foo.pyc', 'V', 'file'),
+            (files[1][0], files[1][1], files[1][2]))
         self.assertEqual(2, len(files))
 
     def test_non_normalized_add_accessible(self):
         try:
-            self.build_tree([u'a\u030a'])
+            self.build_tree(['a\u030a'])
         except UnicodeError:
             raise TestSkipped('Filesystem does not support unicode filenames')
         tree = self.make_branch_and_tree('.')
         orig = osutils.normalized_filename
         osutils.normalized_filename = osutils._accessible_normalized_filename
         try:
-            tree.add([u'a\u030a'])
+            tree.add(['a\u030a'])
             with tree.lock_read():
-                self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
+                self.assertEqual([('', 'directory'), ('\xe5', 'file')],
                                  [(path, ie.kind) for path, ie in
                                   tree.iter_entries_by_dir()])
         finally:
@@ -893,7 +894,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
 
     def test_non_normalized_add_inaccessible(self):
         try:
-            self.build_tree([u'a\u030a'])
+            self.build_tree(['a\u030a'])
         except UnicodeError:
             raise TestSkipped('Filesystem does not support unicode filenames')
         tree = self.make_branch_and_tree('.')
@@ -901,7 +902,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         osutils.normalized_filename = osutils._inaccessible_normalized_filename
         try:
             self.assertRaises(errors.InvalidNormalization,
-                              tree.add, [u'a\u030a'])
+                              tree.add, ['a\u030a'])
         finally:
             osutils.normalized_filename = orig
 
@@ -941,7 +942,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['foo'])
         if tree.supports_setting_file_ids():
-            tree.add(['foo'], [b'foo-id'])
+            tree.add(['foo'], ids=[b'foo-id'])
             self.assertEqual(b'foo-id', tree.path2id('foo'))
             # the next assertion is for backwards compatibility with
             # WorkingTree3, though its probably a bad idea, it makes things
@@ -977,7 +978,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.addCleanup(tree.unlock)
         self.build_tree(['file', 'directory/'])
         names = ['file', 'directory']
-        if has_symlinks():
+        if supports_symlinks(self.test_dir):
             os.symlink('target', 'symlink')
             names.append('symlink')
         tree.add(names)
@@ -1012,10 +1013,10 @@ class TestWorkingTree(TestCaseWithWorkingTree):
     def test_stored_kind_nonexistent(self):
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
-        self.assertRaises(errors.NoSuchFile, tree.stored_kind, 'a')
+        self.assertRaises(_mod_transport.NoSuchFile, tree.stored_kind, 'a')
         self.addCleanup(tree.unlock)
         self.build_tree(['tree/a'])
-        self.assertRaises(errors.NoSuchFile, tree.stored_kind, 'a')
+        self.assertRaises(_mod_transport.NoSuchFile, tree.stored_kind, 'a')
         tree.add(['a'])
         self.assertIs('file', tree.stored_kind('a'))
 
@@ -1035,13 +1036,13 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
-        self.assertRaises(errors.NoSuchFile, tree.get_file_sha1,
+        self.assertRaises(_mod_transport.NoSuchFile, tree.get_file_sha1,
                           'nonexistant')
         self.build_tree(['file'])
         tree.add('file')
         tree.commit('foo')
         tree.remove('file')
-        self.assertRaises(errors.NoSuchFile, tree.get_file_sha1,
+        self.assertRaises(_mod_transport.NoSuchFile, tree.get_file_sha1,
                           'file')
 
     def test_case_sensitive(self):
@@ -1051,10 +1052,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         then testing whether it exists with an uppercase name.
         """
         self.build_tree(['filename'])
-        if os.path.exists('FILENAME'):
-            case_sensitive = False
-        else:
-            case_sensitive = True
+        case_sensitive = not features.CaseInsensitiveFilesystemFeature.available()
         tree = self.make_branch_and_tree('test')
         self.assertEqual(case_sensitive, tree.case_sensitive)
         if not isinstance(tree, InventoryWorkingTree):
@@ -1090,18 +1088,17 @@ class TestWorkingTree(TestCaseWithWorkingTree):
             self.assertFalse(tree.is_executable('filename'))
 
     def test_all_file_ids_with_missing(self):
+        if not self.workingtree_format.supports_setting_file_ids:
+            raise TestNotApplicable('does not support setting file ids')
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
         self.addCleanup(tree.unlock)
         self.build_tree(['tree/a', 'tree/b'])
         tree.add(['a', 'b'])
         os.unlink('tree/a')
-        try:
-            self.assertEqual(
-                {'a', 'b', ''},
-                set(tree.all_versioned_paths()))
-        except errors.UnsupportedOperation:
-            raise TestNotApplicable('tree does not support all_file_ids')
+        self.assertEqual(
+            {'a', 'b', ''},
+            set(tree.all_versioned_paths()))
 
     def test_sprout_hardlink(self):
         real_os_link = getattr(os, 'link', None)
@@ -1185,7 +1182,7 @@ class TestWorkingTreeUpdate(TestCaseWithWorkingTree):
             final_branch, stop_revision=branch_revid, overwrite=True)
         try:
             wt.branch.bind(master)
-        except errors.UpgradeRequired:
+        except _mod_branch.BindingUnsupported:
             raise TestNotApplicable(
                 "Can't bind %s" % wt.branch._format.__class__)
         return wt, master
@@ -1259,12 +1256,8 @@ class TestIllegalPaths(TestCaseWithWorkingTree):
         basis.lock_read()
         self.addCleanup(basis.unlock)
 
-        e = self.assertListRaises(errors.BadFilenameEncoding,
-                                  tree.iter_changes, tree.basis_tree(),
-                                  want_unversioned=True)
-        # We should display the relative path
-        self.assertEqual(b'subdir/m\xb5', e.filename)
-        self.assertEqual(osutils._fs_enc, e.fs_encoding)
+        changes = list(tree.iter_changes(tree.basis_tree(), want_unversioned=True))
+        self.assertIn('subdir/m\udcb5', [c.path[1] for c in changes])
 
 
 class TestControlComponent(TestCaseWithWorkingTree):
