@@ -67,7 +67,7 @@ class InstrumentedImportReplacer(lazy_import.ImportReplacer):
         return lazy_import.ScopeReplacer.__call__(self, *args, **kwargs)
 
 
-class TestClass(object):
+class TestClass:
     """Just a simple test class instrumented for the test cases"""
 
     class_member = 'class_member'
@@ -94,7 +94,7 @@ class TestScopeReplacer(TestCase):
     """
 
     def setUp(self):
-        super(TestScopeReplacer, self).setUp()
+        super().setUp()
         # These tests assume we will not be proxying, so make sure proxying is
         # disabled.
         orig_proxy = lazy_import.ScopeReplacer._should_proxy
@@ -354,7 +354,7 @@ class TestScopeReplacer(TestCase):
 
         # However, the next access on test_obj3 should raise an error
         # because only now are we able to detect the problem.
-        self.assertRaises(errors.IllegalUseOfScopeReplacer,
+        self.assertRaises(lazy_import.IllegalUseOfScopeReplacer,
                           getattr, test_obj3, 'foo')
 
         self.assertEqual([('__getattribute__', 'foo'),
@@ -448,7 +448,7 @@ class TestScopeReplacer(TestCase):
 
         self.assertEqual(InstrumentedReplacer,
                          object.__getattribute__(test_obj7, '__class__'))
-        e = self.assertRaises(errors.IllegalUseOfScopeReplacer, test_obj7)
+        e = self.assertRaises(lazy_import.IllegalUseOfScopeReplacer, test_obj7)
         self.assertIn("replace itself", e.msg)
         self.assertEqual([('__call__', (), {}),
                           'factory'], actions)
@@ -458,7 +458,7 @@ class ImportReplacerHelper(TestCaseInTempDir):
     """Test the ability to have a lazily imported module or object"""
 
     def setUp(self):
-        super(ImportReplacerHelper, self).setUp()
+        super().setUp()
         self.create_modules()
         base_path = self.test_dir + '/base'
 
@@ -917,7 +917,7 @@ class TestCanonicalize(TestCase):
 
     def test_missing_trailing(self):
         proc = lazy_import.ImportProcessor()
-        self.assertRaises(errors.InvalidImportLine,
+        self.assertRaises(lazy_import.InvalidImportLine,
                           proc._canonicalize_import_text,
                           "from foo import (\n  bar\n")
 
@@ -972,7 +972,7 @@ class TestImportProcessor(TestCase):
                         '    two,\n'
                         '    )\n')
 
-    def test_from_one_import_two(self):
+    def test_from_one_import_two_two(self):
         exp = {'two': (['one'], 'two', {})}
         self.check(exp, 'from one import two\n')
         self.check(exp, 'from one import (two)\n')
@@ -1016,13 +1016,13 @@ class TestImportProcessor(TestCase):
 
     def test_incorrect_line(self):
         proc = lazy_import.ImportProcessor()
-        self.assertRaises(errors.InvalidImportLine,
+        self.assertRaises(lazy_import.InvalidImportLine,
                           proc._build_map, 'foo bar baz')
-        self.assertRaises(errors.InvalidImportLine,
+        self.assertRaises(lazy_import.InvalidImportLine,
                           proc._build_map, 'improt foo')
-        self.assertRaises(errors.InvalidImportLine,
+        self.assertRaises(lazy_import.InvalidImportLine,
                           proc._build_map, 'importfoo')
-        self.assertRaises(errors.InvalidImportLine,
+        self.assertRaises(lazy_import.InvalidImportLine,
                           proc._build_map, 'fromimport')
 
     def test_name_collision(self):
@@ -1031,11 +1031,11 @@ class TestImportProcessor(TestCase):
 
         # All of these would try to create an object with the
         # same name as an existing object.
-        self.assertRaises(errors.ImportNameCollision,
+        self.assertRaises(lazy_import.ImportNameCollision,
                           proc._build_map, 'import bar as foo')
-        self.assertRaises(errors.ImportNameCollision,
+        self.assertRaises(lazy_import.ImportNameCollision,
                           proc._build_map, 'from foo import bar as foo')
-        self.assertRaises(errors.ImportNameCollision,
+        self.assertRaises(lazy_import.ImportNameCollision,
                           proc._build_map, 'from bar import foo')
 
     def test_relative_imports(self):
@@ -1058,7 +1058,7 @@ class TestLazyImportProcessor(ImportReplacerHelper):
         else:
             self.fail('root6 was not supposed to exist yet')
 
-        text = 'import %s as root6' % (self.root_name,)
+        text = 'import {} as root6'.format(self.root_name)
         proc = lazy_import.ImportProcessor(InstrumentedImportReplacer)
         proc.lazy_import(scope=globals(), text=text)
 
@@ -1118,7 +1118,7 @@ import %(root_name)s.%(sub_name)s.%(submoda_name)s as submoda7
         else:
             self.fail('root8 was not supposed to exist yet')
         lazy_import.lazy_import(globals(),
-                                'import %s as root8' % (self.root_name,),
+                                'import {} as root8'.format(self.root_name),
                                 lazy_import_class=InstrumentedImportReplacer)
 
         self.assertEqual(InstrumentedImportReplacer,
