@@ -17,7 +17,7 @@
 
 """Tests for the MemoryTree class."""
 
-from .. import errors
+from .. import errors, transport
 from ..memorytree import MemoryTree
 from . import TestCaseWithTransport
 from ..treebuilder import TreeBuilder
@@ -90,8 +90,7 @@ class TestMemoryTree(TestCaseWithTransport):
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         with tree.lock_write():
-            tree.add(['', 'afile', 'adir'], None,
-                     ['directory', 'file', 'directory'])
+            tree.add(['', 'afile', 'adir'], ['directory', 'file', 'directory'])
             self.assertEqual('afile', tree.id2path(tree.path2id('afile')))
             self.assertEqual('adir', tree.id2path(tree.path2id('adir')))
             self.assertFalse(tree.has_filename('afile'))
@@ -121,13 +120,13 @@ class TestMemoryTree(TestCaseWithTransport):
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         with tree.lock_write():
-            tree.add([''], [b'root-id'], ['directory'])
+            tree.add([''], ['directory'], [b'root-id'])
             # Unfortunately, the only way to 'mkdir' is to call 'tree.mkdir', but
             # that *always* adds the directory as well. So if you want to create a
             # file in a subdirectory, you have to split out the 'mkdir()' calls
             # from the add and put_file_bytes_non_atomic calls. :(
             tree.mkdir('adir', b'dir-id')
-            tree.add(['adir/afile'], [b'file-id'], ['file'])
+            tree.add(['adir/afile'], ids=[b'file-id'], kinds=['file'])
             self.assertEqual('adir/afile', tree.id2path(b'file-id'))
             self.assertEqual('adir', tree.id2path(b'dir-id'))
             tree.put_file_bytes_non_atomic('adir/afile', b'barshoom')
@@ -184,17 +183,17 @@ class TestMemoryTree(TestCaseWithTransport):
         tree = self.make_branch_and_memory_tree('branch')
         tree.lock_write()
         self.addCleanup(tree.unlock)
-        tree.add(['', 'foo'], [b'root-id', b'foo-id'], ['directory', 'file'])
+        tree.add(['', 'foo'], ['directory', 'file'], [b'root-id', b'foo-id'])
         tree.put_file_bytes_non_atomic('foo', b'content\n')
         tree.commit('one', rev_id=b'rev-one')
         tree.rename_one('foo', 'bar')
         self.assertEqual('bar', tree.id2path(b'foo-id'))
         self.assertEqual(b'content\n', tree._file_transport.get_bytes('bar'))
-        self.assertRaises(errors.NoSuchFile,
+        self.assertRaises(transport.NoSuchFile,
                           tree._file_transport.get_bytes, 'foo')
         tree.commit('two', rev_id=b'rev-two')
         self.assertEqual(b'content\n', tree._file_transport.get_bytes('bar'))
-        self.assertRaises(errors.NoSuchFile,
+        self.assertRaises(transport.NoSuchFile,
                           tree._file_transport.get_bytes, 'foo')
 
         rev_tree2 = tree.branch.repository.revision_tree(b'rev-two')
@@ -207,7 +206,7 @@ class TestMemoryTree(TestCaseWithTransport):
         self.addCleanup(tree.unlock)
         tree.add('')
         tree.mkdir('subdir', b'subdir-id')
-        tree.add('foo', b'foo-id', 'file')
+        tree.add('foo', 'file', b'foo-id')
         tree.put_file_bytes_non_atomic('foo', b'content\n')
         tree.commit('one', rev_id=b'rev-one')
 
