@@ -16,6 +16,7 @@
 
 
 import errno
+from io import BytesIO
 import os
 import subprocess
 import sys
@@ -28,9 +29,6 @@ from .. import (
     transport,
     urlutils,
     )
-from ..sixish import (
-    BytesIO,
-    )
 from ..transport import (
     chroot,
     fakenfs,
@@ -39,6 +37,9 @@ from ..transport import (
     memory,
     pathfilter,
     readonly,
+    FileExists,
+    NoSuchFile,
+    UnsupportedProtocol,
     )
 import breezy.transport.trace
 from . import (
@@ -66,7 +67,7 @@ class TestTransport(tests.TestCase):
         # don't pollute the current handlers
         transport._clear_protocol_handlers()
 
-        class SampleHandler(object):
+        class SampleHandler:
             """I exist, isnt that enough?"""
         transport._clear_protocol_handlers()
         transport.register_transport_proto('foo')
@@ -93,7 +94,7 @@ class TestTransport(tests.TestCase):
             'foo', 'breezy.tests.test_transport', 'BadTransportHandler')
         try:
             transport.get_transport_from_url('foo://fooserver/foo')
-        except errors.UnsupportedProtocol as e:
+        except UnsupportedProtocol as e:
             self.assertEqual('Unsupported protocol'
                              ' for url "foo://fooserver/foo":'
                              ' Unable to import library "some_lib":'
@@ -118,12 +119,14 @@ class TestTransport(tests.TestCase):
         """Transport ssh:// should raise an error pointing out bzr+ssh://"""
         try:
             transport.get_transport_from_url('ssh://fooserver/foo')
-        except errors.UnsupportedProtocol as e:
-            self.assertEqual('Unsupported protocol'
-                             ' for url "ssh://fooserver/foo":'
-                             ' bzr supports bzr+ssh to operate over ssh,'
-                             ' use "bzr+ssh://fooserver/foo".',
-                             str(e))
+        except UnsupportedProtocol as e:
+            self.assertEqual(
+                'Unsupported protocol'
+                ' for url "ssh://fooserver/foo":'
+                ' Use bzr+ssh for Bazaar operations over SSH, '
+                'e.g. "bzr+ssh://fooserver/foo". Use git+ssh '
+                'for Git operations over SSH, e.g. "git+ssh://fooserver/foo".',
+                str(e))
         else:
             self.fail('Did not raise UnsupportedProtocol')
 
@@ -241,7 +244,7 @@ class TestMemoryServer(tests.TestCase):
         del t
         server.stop_server()
         self.assertFalse(url in transport.transport_list_registry)
-        self.assertRaises(errors.UnsupportedProtocol,
+        self.assertRaises(UnsupportedProtocol,
                           transport.get_transport, url)
 
 
@@ -285,17 +288,17 @@ class TestMemoryTransport(tests.TestCase):
 
     def test_append_without_dir_fails(self):
         t = memory.MemoryTransport()
-        self.assertRaises(errors.NoSuchFile,
+        self.assertRaises(NoSuchFile,
                           t.append_bytes, 'dir/path', b'content')
 
     def test_put_without_dir_fails(self):
         t = memory.MemoryTransport()
-        self.assertRaises(errors.NoSuchFile,
+        self.assertRaises(NoSuchFile,
                           t.put_file, 'dir/path', BytesIO(b'content'))
 
     def test_get_missing(self):
         transport = memory.MemoryTransport()
-        self.assertRaises(errors.NoSuchFile, transport.get, 'foo')
+        self.assertRaises(NoSuchFile, transport.get, 'foo')
 
     def test_has_missing(self):
         t = memory.MemoryTransport()
@@ -325,12 +328,12 @@ class TestMemoryTransport(tests.TestCase):
 
     def test_mkdir_missing_parent(self):
         t = memory.MemoryTransport()
-        self.assertRaises(errors.NoSuchFile, t.mkdir, 'dir/dir')
+        self.assertRaises(NoSuchFile, t.mkdir, 'dir/dir')
 
     def test_mkdir_twice(self):
         t = memory.MemoryTransport()
         t.mkdir('dir')
-        self.assertRaises(errors.FileExists, t.mkdir, 'dir')
+        self.assertRaises(FileExists, t.mkdir, 'dir')
 
     def test_parameters(self):
         t = memory.MemoryTransport()
@@ -455,7 +458,7 @@ class TestHooks(tests.TestCase):
         """Check all expected transport hook points are set up"""
         hookpoint = transport.TransportHooks()
         self.assertTrue("post_connect" in hookpoint,
-                        "post_connect not in %s" % (hookpoint,))
+                        "post_connect not in {}".format(hookpoint))
 
     def test_post_connect(self):
         """Ensure the post_connect hook is called when _set_transport is"""
@@ -679,7 +682,7 @@ class TestTransportImplementation(tests.TestCaseInTempDir):
     """
 
     def setUp(self):
-        super(TestTransportImplementation, self).setUp()
+        super().setUp()
         self._server = self.transport_server()
         self.start_server(self._server)
 
@@ -814,7 +817,7 @@ class TestLocalTransportWriteStream(tests.TestCaseWithTransport):
 
     def test_missing_directory(self):
         t = self.get_transport('.')
-        self.assertRaises(errors.NoSuchFile, t.open_write_stream, 'dir/foo')
+        self.assertRaises(NoSuchFile, t.open_write_stream, 'dir/foo')
 
 
 class TestWin32LocalTransport(tests.TestCase):
@@ -1062,11 +1065,9 @@ class TestSSHConnections(tests.TestCaseWithTransport):
         self.start_server(ssh_server)
         port = ssh_server.port
 
-        if sys.platform == 'win32':
-            bzr_remote_path = sys.executable + ' ' + self.get_brz_path()
-        else:
-            bzr_remote_path = self.get_brz_path()
+        bzr_remote_path = self.get_brz_path()
         self.overrideEnv('BZR_REMOTE_PATH', bzr_remote_path)
+        self.overrideEnv('PYTHONPATH', ':'.join(sys.path))
 
         # Access the branch via a bzr+ssh URL.  The BZR_REMOTE_PATH environment
         # variable is used to tell bzr what command to run on the remote end.
@@ -1094,14 +1095,3 @@ class TestSSHConnections(tests.TestCaseWithTransport):
         # And the rest are threads
         for t in started[1:]:
             t.join()
-
-
-class TestUnhtml(tests.TestCase):
-
-    """Tests for unhtml_roughly"""
-
-    def test_truncation(self):
-        fake_html = "<p>something!\n" * 1000
-        result = http.unhtml_roughly(fake_html)
-        self.assertEqual(len(result), 1000)
-        self.assertStartsWith(result, " something!")
