@@ -21,6 +21,8 @@ from io import (
     StringIO,
     )
 import os
+import re
+import yaml
 
 from .. import (
     registry,
@@ -28,7 +30,7 @@ from .. import (
     version_info_formats,
     )
 from . import TestCaseWithTransport
-from ..rio import read_stanzas, read_stanzas_unicode
+from ..bzr.rio import read_stanzas
 
 from ..version_info_formats.format_custom import (
     CustomVersionInfoBuilder,
@@ -36,6 +38,7 @@ from ..version_info_formats.format_custom import (
     NoTemplate,
     )
 from ..version_info_formats.format_rio import RioVersionInfoBuilder
+from ..version_info_formats.format_yaml import YamlVersionInfoBuilder
 from ..version_info_formats.format_python import PythonVersionInfoBuilder
 
 
@@ -53,7 +56,7 @@ class VersionInfoTestCase(TestCaseWithTransport):
         wt.commit('b', rev_id=b'r2')
 
         self.build_tree_contents([('branch/a', b'new contents\n')])
-        wt.commit(u'\xe52', rev_id=b'r3')
+        wt.commit('\xe52', rev_id=b'r3')
 
         return wt
 
@@ -82,66 +85,66 @@ class TestVersionInfoRio(VersionInfoTestCase):
     def test_rio_null(self):
         wt = self.make_branch_and_tree('branch')
 
-        sio = StringIO()
+        bio = BytesIO()
         builder = RioVersionInfoBuilder(wt.branch, working_tree=wt)
-        builder.generate(sio)
-        val = sio.getvalue()
-        self.assertContainsRe(val, 'build-date:')
-        self.assertContainsRe(val, 'revno: 0')
+        builder.generate(bio)
+        val = bio.getvalue()
+        self.assertContainsRe(val, b'build-date:')
+        self.assertContainsRe(val, b'revno: 0')
 
     def test_rio_dotted_revno(self):
         wt = self.create_tree_with_dotted_revno()
 
-        sio = StringIO()
+        bio = BytesIO()
         builder = RioVersionInfoBuilder(wt.branch, working_tree=wt)
-        builder.generate(sio)
-        val = sio.getvalue()
-        self.assertContainsRe(val, 'revno: 1.1.1')
+        builder.generate(bio)
+        val = bio.getvalue()
+        self.assertContainsRe(val, b'revno: 1.1.1')
 
     def regen_text(self, wt, **kwargs):
-        sio = StringIO()
+        bio = BytesIO()
         builder = RioVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
-        builder.generate(sio)
-        val = sio.getvalue()
+        builder.generate(bio)
+        val = bio.getvalue()
         return val
 
     def test_simple(self):
         wt = self.create_branch()
 
         val = self.regen_text(wt)
-        self.assertContainsRe(val, 'build-date:')
-        self.assertContainsRe(val, 'date:')
-        self.assertContainsRe(val, 'revno: 3')
-        self.assertContainsRe(val, 'revision-id: r3')
+        self.assertContainsRe(val, b'build-date:')
+        self.assertContainsRe(val, b'date:')
+        self.assertContainsRe(val, b'revno: 3')
+        self.assertContainsRe(val, b'revision-id: r3')
 
     def test_clean(self):
         wt = self.create_branch()
         val = self.regen_text(wt, check_for_clean=True)
-        self.assertContainsRe(val, 'clean: True')
+        self.assertContainsRe(val, b'clean: True')
 
     def test_no_clean(self):
         wt = self.create_branch()
         self.build_tree(['branch/c'])
         val = self.regen_text(wt, check_for_clean=True)
-        self.assertContainsRe(val, 'clean: False')
+        self.assertContainsRe(val, b'clean: False')
 
     def test_history(self):
         wt = self.create_branch()
 
         val = self.regen_text(wt, include_revision_history=True)
-        self.assertContainsRe(val, 'id: r1')
-        self.assertContainsRe(val, 'message: a')
-        self.assertContainsRe(val, 'id: r2')
-        self.assertContainsRe(val, 'message: b')
-        self.assertContainsRe(val, 'id: r3')
-        self.assertContainsRe(val, 'message: \xe5')
+        self.assertContainsRe(val, b'id: r1')
+        self.assertContainsRe(val, b'message: a')
+        self.assertContainsRe(val, b'id: r2')
+        self.assertContainsRe(val, b'message: b')
+        self.assertContainsRe(val, b'id: r3')
+        self.assertContainsRe(val, b'message: \xc3\xa52')
 
     def regen(self, wt, **kwargs):
-        sio = StringIO()
+        bio = BytesIO()
         builder = RioVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
-        builder.generate(sio)
-        sio.seek(0)
-        stanzas = list(read_stanzas_unicode(sio))
+        builder.generate(bio)
+        bio.seek(0)
+        stanzas = list(read_stanzas(bio))
         self.assertEqual(1, len(stanzas))
         return stanzas[0]
 
@@ -192,7 +195,7 @@ class TestVersionInfoRio(VersionInfoTestCase):
         stanza = self.regen(wt, include_revision_history=True)
         revision_stanza = self.get_one_stanza(stanza, 'revisions')
         self.assertEqual(['r1', 'r2', 'r3'], revision_stanza.get_all('id'))
-        self.assertEqual(['a', 'b', u'\xe52'],
+        self.assertEqual(['a', 'b', '\xe52'],
                          revision_stanza.get_all('message'))
         self.assertEqual(3, len(revision_stanza.get_all('date')))
 
@@ -243,6 +246,167 @@ class TestVersionInfoRio(VersionInfoTestCase):
                          file_rev_stanza.get_all('revision'))
 
 
+class TestVersionInfoYaml(VersionInfoTestCase):
+
+    def test_yaml_null(self):
+        wt = self.make_branch_and_tree('branch')
+
+        bio = StringIO()
+        builder = YamlVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(bio)
+        val = bio.getvalue()
+        self.assertContainsRe(val, 'build-date:')
+        self.assertContainsRe(val, 'revno: \'0\'')
+
+    def test_yaml_dotted_revno(self):
+        wt = self.create_tree_with_dotted_revno()
+
+        bio = StringIO()
+        builder = YamlVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(bio)
+        val = bio.getvalue()
+        self.assertContainsRe(val, 'revno: 1.1.1')
+
+    def regen_text(self, wt, **kwargs):
+        bio = StringIO()
+        builder = YamlVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
+        builder.generate(bio)
+        val = bio.getvalue()
+        return val
+
+    def test_simple(self):
+        wt = self.create_branch()
+
+        val = self.regen_text(wt)
+        self.assertContainsRe(val, 'build-date:')
+        self.assertContainsRe(val, 'date:')
+        self.assertContainsRe(val, 'revno: \'3\'')
+        self.assertContainsRe(val, 'revision-id: r3')
+
+    def test_clean(self):
+        wt = self.create_branch()
+        val = self.regen_text(wt, check_for_clean=True)
+        self.assertContainsRe(val, 'clean: true')
+
+    def test_no_clean(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/c'])
+        val = self.regen_text(wt, check_for_clean=True)
+        self.assertContainsRe(val, 'clean: false')
+
+    def test_history(self):
+        wt = self.create_branch()
+
+        val = self.regen_text(wt, include_revision_history=True)
+        self.assertContainsRe(val, 'id: r1')
+        self.assertContainsRe(val, 'message: a')
+        self.assertContainsRe(val, 'id: r2')
+        self.assertContainsRe(val, 'message: ')
+        self.assertContainsRe(val, 'id: r3')
+        self.assertContainsRe(val, re.escape('message: \"\\xE52\"'))
+
+    def regen(self, wt, **kwargs):
+        bio = StringIO()
+        builder = YamlVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
+        builder.generate(bio)
+        bio.seek(0)
+        return yaml.safe_load(bio)
+
+    def test_yaml_version_hook(self):
+        def update_stanza(rev, stanza):
+            stanza['bla'] = 'bloe'
+        YamlVersionInfoBuilder.hooks.install_named_hook(
+            'revision', update_stanza, None)
+        wt = self.create_branch()
+
+        stanza = self.regen(wt)
+        self.assertEqual('bloe', stanza['bla'])
+
+    def test_build_date(self):
+        wt = self.create_branch()
+        stanza = self.regen(wt)
+        self.assertTrue('date' in stanza)
+        self.assertTrue('build-date' in stanza)
+        self.assertEqual('3', stanza['revno'])
+        self.assertEqual('r3', stanza['revision-id'])
+
+    def test_not_clean(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/c'])
+        stanza = self.regen(wt, check_for_clean=True,
+                            include_file_revisions=True)
+        self.assertEqual(False, stanza['clean'])
+
+    def test_file_revisions(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/c'])
+        stanza = self.regen(wt, check_for_clean=True,
+                            include_file_revisions=True)
+        # This assumes it's being run against a tree that does not update the
+        # root revision on every commit.
+        file_rev_stanza = stanza['file-revisions']
+        self.assertEqual(['', 'a', 'b', 'c'], [r['path'] for r in file_rev_stanza])
+        self.assertEqual(['r1', 'r3', 'r2', 'unversioned'],
+                         [r['revision'] for r in file_rev_stanza])
+
+    def test_revision_history(self):
+        wt = self.create_branch()
+        stanza = self.regen(wt, include_revision_history=True)
+        revision_stanza = stanza['revisions']
+        self.assertEqual(
+            ['r1', 'r2', 'r3'], [r['id'] for r in revision_stanza])
+        self.assertEqual(
+            ['a', 'b', '\xe52'],
+            [r['message'] for r in revision_stanza])
+        self.assertEqual(3, len([r['date'] for r in revision_stanza]))
+
+    def test_file_revisions_with_rename(self):
+        # a was modified, so it should show up modified again
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+        stanza = self.regen(wt, check_for_clean=True,
+                            include_file_revisions=True)
+        file_rev_stanza = stanza['file-revisions']
+        self.assertEqual(['', 'a', 'b', 'c', 'd'],
+                         [r['path'] for r in file_rev_stanza])
+        self.assertEqual(['r1', 'modified', 'renamed to d', 'new',
+                          'renamed from b'],
+                         [r['revision'] for r in file_rev_stanza])
+
+    def test_file_revisions_with_removal(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+
+        wt.commit('modified', rev_id=b'r4')
+
+        wt.remove(['c', 'd'])
+        os.remove('branch/d')
+        stanza = self.regen(wt, check_for_clean=True,
+                            include_file_revisions=True)
+        file_rev_stanza = stanza['file-revisions']
+        self.assertEqual(['', 'a', 'c', 'd'], [r['path'] for r in file_rev_stanza])
+        self.assertEqual(['r1', 'r4', 'unversioned', 'removed'],
+                         [r['revision'] for r in file_rev_stanza])
+
+    def test_revision(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+
+        stanza = self.regen(
+            wt, check_for_clean=True, include_file_revisions=True,
+            revision_id=wt.last_revision())
+        file_rev_stanza = stanza['file-revisions']
+        self.assertEqual(['', 'a', 'b'], [r['path'] for r in file_rev_stanza])
+        self.assertEqual(['r1', 'r3', 'r2'],
+                         [r['revision'] for r in file_rev_stanza])
+
+
 class PythonVersionInfoTests(VersionInfoTestCase):
 
     def test_python_null(self):
@@ -302,7 +466,7 @@ class PythonVersionInfoTests(VersionInfoTestCase):
         rev_info = [(rev, message) for rev, message, timestamp, timezone
                     in tvi['revisions']]
         self.assertEqual([(b'r1', 'a'), (b'r2', 'b'),
-                          (b'r3', u'\xe52')], rev_info)
+                          (b'r3', '\xe52')], rev_info)
 
         # a was modified, so it should show up modified again
         self.build_tree(['branch/a', 'branch/c'])
@@ -403,7 +567,7 @@ class TestBuilder(version_info_formats.VersionInfoBuilder):
 class TestVersionInfoFormatRegistry(tests.TestCase):
 
     def setUp(self):
-        super(TestVersionInfoFormatRegistry, self).setUp()
+        super().setUp()
         self.overrideAttr(version_info_formats,
                           'format_registry', registry.Registry())
 
