@@ -28,6 +28,7 @@ from .. import (
     )
 from ..bzr import (
     bzrdir,
+    conflicts as _mod_bzr_conflicts,
     workingtree as bzrworkingtree,
     workingtree_3,
     workingtree_4,
@@ -257,7 +258,7 @@ class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
         tree = self.make_branch_and_tree('tree', format='development-subtree')
         self.build_tree(['tree/a/', 'tree/a/b/', 'tree/a/b/c'])
         tree.set_root_id(b'root-id')
-        tree.add(['a', 'a/b', 'a/b/c'], [b'a-id', b'b-id', b'c-id'])
+        tree.add(['a', 'a/b', 'a/b/c'], ids=[b'a-id', b'b-id', b'c-id'])
         tree.commit('initial')
         return tree
 
@@ -296,7 +297,7 @@ class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
 class TestWorkingTreeFormatRegistry(TestCase):
 
     def setUp(self):
-        super(TestWorkingTreeFormatRegistry, self).setUp()
+        super().setUp()
         self.registry = workingtree.WorkingTreeFormatRegistry()
 
     def test_register_unregister_format(self):
@@ -411,16 +412,10 @@ class TestRevert(TestCaseWithTransport):
 
 class TestAutoResolve(TestCaseWithTransport):
 
-    def _auto_resolve(self, tree):
-        """Call auto_resolve on tree expecting deprecation"""
-        return self.applyDeprecated(
-            symbol_versioning.deprecated_in((3, 0, 1)),
-            tree.auto_resolve,)
-
     def test_auto_resolve(self):
         base = self.make_branch_and_tree('base')
         self.build_tree_contents([('base/hello', b'Hello')])
-        base.add('hello', b'hello_id')
+        base.add('hello', ids=b'hello_id')
         base.commit('Hello')
         other = base.controldir.sprout('other').open_workingtree()
         self.build_tree_contents([('other/hello', b'hELLO')])
@@ -430,41 +425,41 @@ class TestAutoResolve(TestCaseWithTransport):
         self.build_tree_contents([('this/hello', b'Hello World')])
         this.commit('Add World')
         this.merge_from_branch(other.branch)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
-        self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        this.auto_resolve()
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '<<<<<<<')])
-        self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        this.auto_resolve()
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '=======')])
-        self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        this.auto_resolve()
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '\n>>>>>>>')])
-        remaining, resolved = self._auto_resolve(this)
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        remaining, resolved = this.auto_resolve()
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          this.conflicts())
         self.assertEqual([], resolved)
         self.build_tree_contents([('this/hello', b'hELLO wORLD')])
-        remaining, resolved = self._auto_resolve(this)
+        remaining, resolved = this.auto_resolve()
         self.assertEqual([], this.conflicts())
-        self.assertEqual([conflicts.TextConflict('hello', b'hello_id')],
+        self.assertEqual([_mod_bzr_conflicts.TextConflict('hello', b'hello_id')],
                          resolved)
         self.assertPathDoesNotExist('this/hello.BASE')
 
     def test_unsupported_symlink_auto_resolve(self):
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(SymlinkFeature(self.test_dir))
         base = self.make_branch_and_tree('base')
         self.build_tree_contents([('base/hello', 'Hello')])
-        base.add('hello', b'hello_id')
+        base.add('hello', ids=b'hello_id')
         base.commit('commit 0')
         other = base.controldir.sprout('other').open_workingtree()
         self.build_tree_contents([('other/hello', 'Hello')])
         os.symlink('other/hello', 'other/foo')
-        other.add('foo', b'foo_id')
+        other.add('foo', ids=b'foo_id')
         other.commit('commit symlink')
         this = base.controldir.sprout('this').open_workingtree()
         self.assertPathExists('this/hello')
@@ -486,24 +481,24 @@ class TestAutoResolve(TestCaseWithTransport):
     def test_auto_resolve_dir(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello/'])
-        tree.add('hello', b'hello-id')
-        file_conflict = conflicts.TextConflict('hello', b'hello-id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
-        remaining, resolved = self._auto_resolve(tree)
+        tree.add('hello', ids=b'hello-id')
+        file_conflict = _mod_bzr_conflicts.TextConflict('hello', b'hello-id')
+        tree.set_conflicts([file_conflict])
+        remaining, resolved = tree.auto_resolve()
         self.assertEqual(
             remaining,
-            conflicts.ConflictList([conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict('hello', 'hello-id')]))
         self.assertEqual(resolved, [])
 
     def test_auto_resolve_missing(self):
         tree = self.make_branch_and_tree('tree')
-        file_conflict = conflicts.TextConflict('hello', b'hello-id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
-        remaining, resolved = self._auto_resolve(tree)
+        file_conflict = _mod_bzr_conflicts.TextConflict('hello', b'hello-id')
+        tree.set_conflicts([file_conflict])
+        remaining, resolved = tree.auto_resolve()
         self.assertEqual(remaining, [])
         self.assertEqual(
             resolved,
-            conflicts.ConflictList([conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict('hello', 'hello-id')]))
 
 
 class TestStoredUncommitted(TestCaseWithTransport):
@@ -512,7 +507,7 @@ class TestStoredUncommitted(TestCaseWithTransport):
         tree = self.make_branch_and_tree('tree')
         tree.commit('get root in there')
         self.build_tree_contents([('tree/file', b'content')])
-        tree.add('file', b'file-id')
+        tree.add('file', ids=b'file-id')
         tree.store_uncommitted()
         return tree
 
