@@ -29,6 +29,7 @@ it.
 import errno
 from io import BytesIO
 import sys
+from typing import Dict, Any, Callable, TypeVar
 
 from stat import S_ISDIR
 
@@ -47,7 +48,7 @@ from .. import (
 
 # a dictionary of open file streams. Keys are absolute paths, values are
 # transport defined.
-_file_streams = {}
+_file_streams: Dict[str, Any] = {}
 
 
 def _get_protocol_handlers():
@@ -89,10 +90,28 @@ class UnusableRedirect(errors.BzrError):
             "%(reason)s.")
 
     def __init__(self, source, target, reason):
-        super(UnusableRedirect, self).__init__(
+        super().__init__(
             source=source, target=target, reason=reason)
 
 
+class UnsupportedProtocol(errors.PathError):
+
+    _fmt = 'Unsupported protocol for url "%(path)s"%(extra)s'
+
+    def __init__(self, url, extra=""):
+        errors.PathError.__init__(self, url, extra=extra)
+
+
+class NoSuchFile(errors.PathError):
+
+    _fmt = "No such file: %(path)r%(extra)s"
+
+
+class FileExists(errors.PathError):
+
+    _fmt = "File exists: %(path)r%(extra)s"
+
+
 class TransportListRegistry(registry.Registry):
     """A registry which simplifies tracking available Transports.
 
@@ -172,7 +191,7 @@ def unregister_transport(scheme, factory):
         transport_list_registry.remove(scheme)
 
 
-class _CoalescedOffset(object):
+class _CoalescedOffset:
     """A data container for keeping track of coalesced offsets."""
 
     __slots__ = ['start', 'length', 'ranges']
@@ -182,20 +201,20 @@ class _CoalescedOffset(object):
         self.length = length
         self.ranges = ranges
 
-    def __cmp__(self, other):
-        return cmp((self.start, self.length, self.ranges),
-                   (other.start, other.length, other.ranges))
+    def __lt__(self, other):
+        return ((self.start, self.length, self.ranges) <
+                (other.start, other.length, other.ranges))
 
     def __eq__(self, other):
         return ((self.start, self.length, self.ranges) ==
                 (other.start, other.length, other.ranges))
 
     def __repr__(self):
-        return '%s(%r, %r, %r)' % (self.__class__.__name__,
+        return '{}({!r}, {!r}, {!r})'.format(self.__class__.__name__,
                                    self.start, self.length, self.ranges)
 
 
-class LateReadError(object):
+class LateReadError:
     """A helper for transports which pretends to be a readable file.
 
     When read() is called, errors.ReadError is raised.
@@ -233,7 +252,7 @@ class LateReadError(object):
         self._fail()
 
 
-class FileStream(object):
+class FileStream:
     """Base class for FileStreams."""
 
     def __init__(self, transport, relpath):
@@ -267,7 +286,7 @@ class FileStream(object):
             flush to disk.
         """
         raise errors.TransportNotPossible(
-            "%s cannot fdatasync" % (self.transport,))
+            "{} cannot fdatasync".format(self.transport))
 
 
 class FileFileStream(FileStream):
@@ -310,14 +329,14 @@ class TransportHooks(hooks.Hooks):
     """Mapping of hook names to registered callbacks for transport hooks"""
 
     def __init__(self):
-        super(TransportHooks, self).__init__()
+        super().__init__()
         self.add_hook("post_connect",
                       "Called after a new connection is established or a reconnect "
                       "occurs. The sole argument passed is either the connected "
                       "transport or smart medium instance.", (2, 5))
 
 
-class Transport(object):
+class Transport:
     """This class encapsulates methods for retrieving or putting a file
     from/to a storage location.
 
@@ -337,8 +356,10 @@ class Transport(object):
 
     hooks = TransportHooks()
 
+    base: str
+
     def __init__(self, base):
-        super(Transport, self).__init__()
+        super().__init__()
         self.base = base
         (self._raw_base, self._segment_parameters) = (
             urlutils.split_segment_parameters(base))
@@ -350,18 +371,18 @@ class Transport(object):
         """
         if getattr(e, 'errno', None) is not None:
             if e.errno in (errno.ENOENT, errno.ENOTDIR):
-                raise errors.NoSuchFile(path, extra=e)
+                raise NoSuchFile(path, extra=e)
             elif e.errno == errno.EINVAL:
-                mutter("EINVAL returned on path %s: %r" % (path, e))
-                raise errors.NoSuchFile(path, extra=e)
+                mutter("EINVAL returned on path {}: {!r}".format(path, e))
+                raise NoSuchFile(path, extra=e)
             # I would rather use errno.EFOO, but there doesn't seem to be
             # any matching for 267
             # This is the error when doing a listdir on a file:
             # WindowsError: [Errno 267] The directory name is invalid
             if sys.platform == 'win32' and e.errno in (errno.ESRCH, 267):
-                raise errors.NoSuchFile(path, extra=e)
+                raise NoSuchFile(path, extra=e)
             if e.errno == errno.EEXIST:
-                raise errors.FileExists(path, extra=e)
+                raise FileExists(path, extra=e)
             if e.errno == errno.EACCES:
                 raise errors.PermissionDenied(path, extra=e)
             if e.errno == errno.ENOTEMPTY:
@@ -391,10 +412,10 @@ class Transport(object):
                     % cur_transport.base)
             try:
                 new_transport.mkdir('.', mode=mode)
-            except errors.NoSuchFile:
+            except NoSuchFile:
                 needed.append(new_transport)
                 cur_transport = new_transport
-            except errors.FileExists:
+            except FileExists:
                 break
             else:
                 break
@@ -414,8 +435,12 @@ class Transport(object):
         # suppress FileExists and PermissionDenied (for Windows) exceptions.
         try:
             self.mkdir('.', mode=mode)
-        except (errors.FileExists, errors.PermissionDenied):
+        except (FileExists, errors.PermissionDenied):
             return False
+        except errors.TransportNotPossible:
+            if self.has('.'):
+                return False
+            raise
         else:
             return True
 
@@ -449,8 +474,9 @@ class Transport(object):
     def set_segment_parameter(self, name, value):
         """Set a segment parameter.
 
-        :param name: Segment parameter name (urlencoded string)
-        :param value: Segment parameter value (urlencoded string)
+        Args:
+          name: Segment parameter name (urlencoded string)
+          value: Segment parameter value (urlencoded string)
         """
         if value is None:
             try:
@@ -485,8 +511,9 @@ class Transport(object):
         Be careful that it's not called twice, if one method is implemented on
         top of another.
 
-        :param bytes: Number of bytes read or written.
-        :param direction: 'read' or 'write' or None.
+        Args:
+          bytes: Number of bytes read or written.
+          direction: 'read' or 'write' or None.
         """
         ui.ui_factory.report_transport_activity(self, bytes, direction)
 
@@ -645,20 +672,21 @@ class Transport(object):
               upper_limit=None):
         """Get parts of the file at the given relative path.
 
-        :param relpath: The path to read data from.
-        :param offsets: A list of (offset, size) tuples.
-        :param adjust_for_latency: Adjust the requested offsets to accomodate
+        Args:
+          relpath: The path to read data from.
+          offsets: A list of (offset, size) tuples.
+          adjust_for_latency: Adjust the requested offsets to accomodate
             transport latency. This may re-order the offsets, expand them to
             grab adjacent data when there is likely a high cost to requesting
             data relative to delivering it.
-        :param upper_limit: When adjust_for_latency is True setting upper_limit
+          upper_limit: When adjust_for_latency is True setting upper_limit
             allows the caller to tell the transport about the length of the
             file, so that requests are not issued for ranges beyond the end of
             the file. This matters because some servers and/or transports error
             in such a case rather than just satisfying the available ranges.
             upper_limit should always be provided when adjust_for_latency is
             True, and should be the size of the file in bytes.
-        :return: A list or generator of (offset, data) tuples
+        Returns: A list or generator of (offset, data) tuples
         """
         if adjust_for_latency:
             # Design note: We may wish to have different algorithms for the
@@ -860,7 +888,7 @@ class Transport(object):
             coalesced_offsets.append(cur)
         return coalesced_offsets
 
-    def put_bytes(self, relpath, raw_bytes, mode=None):
+    def put_bytes(self, relpath: str, raw_bytes: bytes, mode=None):
         """Atomically put the supplied bytes into the given location.
 
         :param relpath: The location to put the contents, relative to the
@@ -874,7 +902,7 @@ class Transport(object):
                 'raw_bytes must be a plain string, not %s' % type(raw_bytes))
         return self.put_file(relpath, BytesIO(raw_bytes), mode=mode)
 
-    def put_bytes_non_atomic(self, relpath, raw_bytes, mode=None,
+    def put_bytes_non_atomic(self, relpath, raw_bytes: bytes, mode=None,
                              create_parent_dir=False,
                              dir_mode=None):
         """Copy the string into the target location.
@@ -932,7 +960,7 @@ class Transport(object):
         # Default implementation just does an atomic put.
         try:
             return self.put_file(relpath, f, mode=mode)
-        except errors.NoSuchFile:
+        except NoSuchFile:
             if not create_parent_dir:
                 raise
             parent_dir = osutils.dirname(relpath)
@@ -998,7 +1026,8 @@ class Transport(object):
         Override this for efficiency if a specific transport can do it
         faster than this default implementation.
         """
-        self.put_file(rel_to, self.get(rel_from))
+        with self.get(rel_from) as f:
+            self.put_file(rel_to, f)
 
     def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
@@ -1115,7 +1144,7 @@ class Transport(object):
         self.rmdir(relpath)
 
     def __repr__(self):
-        return "<%s.%s url=%s>" % (self.__module__, self.__class__.__name__, self.base)
+        return "<{}.{} url={}>".format(self.__module__, self.__class__.__name__, self.base)
 
     def stat(self, relpath):
         """Return the stat information for a file.
@@ -1241,7 +1270,7 @@ class Transport(object):
             source, target, "transport does not support redirection")
 
 
-class _SharedConnection(object):
+class _SharedConnection:
     """A connection shared between several transports."""
 
     def __init__(self, connection=None, credentials=None, base=None):
@@ -1292,7 +1321,7 @@ class ConnectedTransport(Transport):
 
         base = str(self._parsed_url)
 
-        super(ConnectedTransport, self).__init__(base)
+        super().__init__(base)
         if _from_transport is None:
             self._shared_connection = _SharedConnection()
         else:
@@ -1344,12 +1373,13 @@ class ConnectedTransport(Transport):
         user, password, host and path will be quoted if they contain reserved
         chars.
 
-        :param scheme: protocol
-        :param user: login
-        :param password: associated password
-        :param host: the server address
-        :param port: the associated port
-        :param path: the absolute path on the server
+        Args:
+          scheme: protocol
+          user: login
+          password: associated password
+          host: the server address
+          port: the associated port
+          path: the absolute path on the server
 
         :return: The corresponding URL.
         """
@@ -1358,7 +1388,7 @@ class ConnectedTransport(Transport):
             # Note that we don't put the password back even if we
             # have one so that it doesn't get accidentally
             # exposed.
-            netloc = '%s@%s' % (urlutils.quote(user), netloc)
+            netloc = '{}@{}'.format(urlutils.quote(user), netloc)
         if port is not None:
             netloc = '%s:%d' % (netloc, port)
         path = urlutils.escape(path)
@@ -1388,7 +1418,8 @@ class ConnectedTransport(Transport):
     def abspath(self, relpath):
         """Return the full url to the given relative path.
 
-        :param relpath: the relative path urlencoded
+        Args:
+          relpath: the relative path urlencoded
 
         :returns: the Unicode version of the absolute path for relpath.
         """
@@ -1401,7 +1432,8 @@ class ConnectedTransport(Transport):
         requests, daughter classes should redefine this method if needed and
         use the result to build their requests.
 
-        :param relpath: the path relative to the transport base urlencoded.
+        Args:
+          relpath: the path relative to the transport base urlencoded.
 
         :return: the absolute Unicode path on the server,
         """
@@ -1425,10 +1457,10 @@ class ConnectedTransport(Transport):
         always call this method to set the connection and do so each time a new
         connection is created.
 
-        :param connection: An opaque object representing the connection used by
+        Args:
+          connection: An opaque object representing the connection used by
             the daughter class.
-
-        :param credentials: An opaque object representing the credentials
+          credentials: An opaque object representing the credentials
             needed to create the connection.
         """
         self._shared_connection.connection = connection
@@ -1520,11 +1552,12 @@ def get_transport_from_path(path, possible_transports=None):
 def get_transport_from_url(url, possible_transports=None):
     """Open a transport to access a URL.
 
-    :param base: a URL
-    :param transports: optional reusable transports list. If not None, created
+    Args:
+      base: a URL
+      transports: optional reusable transports list. If not None, created
         transports will be added to the list.
 
-    :return: A new transport optionally sharing its connection with one of
+    Returns: A new transport optionally sharing its connection with one of
         possible_transports.
     """
     transport = None
@@ -1549,18 +1582,17 @@ def get_transport_from_url(url, possible_transports=None):
                 return transport
     if not urlutils.is_url(url):
         raise urlutils.InvalidURL(path=url)
-    raise errors.UnsupportedProtocol(url, last_err)
+    raise UnsupportedProtocol(url, last_err)
 
 
 def get_transport(base, possible_transports=None, purpose=None):
     """Open a transport to access a URL or directory.
 
-    :param base: either a URL or a directory name.
-
-    :param transports: optional reusable transports list. If not None, created
+    Args:
+      base: either a URL or a directory name.
+      transports: optional reusable transports list. If not None, created
         transports will be added to the list.
-
-    :param purpose: Purpose for which the transport will be used
+      purpose: Purpose for which the transport will be used
         (e.g. 'read', 'write' or None)
 
     :return: A new transport optionally sharing its connection with one of
@@ -1587,7 +1619,14 @@ def _try_transport_factories(base, factory_list):
     return None, last_err
 
 
-def do_catching_redirections(action, transport, redirected):
+T = TypeVar('T')
+
+
+def do_catching_redirections(
+        action: Callable[[Transport], T],
+        transport: Transport,
+        redirected: Callable[
+            [Transport, errors.RedirectRequested, str], Transport]) -> T:
     """Execute an action with given transport catching redirections.
 
     This is a facility provided for callers needing to follow redirections
@@ -1595,10 +1634,11 @@ def do_catching_redirections(action, transport, redirected):
     inform the user about each redirection or only inform the user of a user
     via the exception parameter.
 
-    :param action: A callable, what the caller want to do while catching
+    Args:
+      action: A callable, what the caller want to do while catching
                   redirections.
-    :param transport: The initial transport used.
-    :param redirected: A callable receiving the redirected transport and the
+      transport: The initial transport used.
+      redirected: A callable receiving the redirected transport and the
                   RedirectRequested exception.
 
     :return: Whatever 'action' returns
@@ -1612,7 +1652,7 @@ def do_catching_redirections(action, transport, redirected):
         try:
             return action(transport)
         except errors.RedirectRequested as e:
-            redirection_notice = '%s is%s redirected to %s' % (
+            redirection_notice = '{} is{} redirected to {}'.format(
                 e.source, e.permanently, e.target)
             transport = redirected(transport, e, redirection_notice)
     else:
@@ -1629,7 +1669,7 @@ def do_catching_redirections(action, transport, redirected):
         raise errors.TooManyRedirections
 
 
-class Server(object):
+class Server:
     """A Transport Server.
 
     The Server interface provides a server for a given transport type.
@@ -1642,6 +1682,31 @@ class Server(object):
         """Remove the server and cleanup any resources it owns."""
 
 
+def open_file(url):
+    """Open a file from a URL.
+
+    :param url: URL to open
+    :return: A file-like object.
+    """
+    base, filename = urlutils.split(path)
+    transport = get_transport(base)
+    return open_file_via_transport(filename, transport)
+
+
+def open_file_via_transport(filename, transport):
+    """Open a file using the transport, follow redirects as necessary."""
+    def open_file(transport):
+        return transport.get(filename)
+
+    def follow_redirection(transport, e, redirection_notice):
+        mutter(redirection_notice)
+        base, filename = urlutils.split(e.target)
+        redirected_transport = get_transport(base)
+        return redirected_transport
+
+    return do_catching_redirections(open_file, transport, follow_redirection)
+
+
 # None is the default transport, for things with no url scheme
 register_transport_proto('file://',
                          help="Access using the standard filesystem (default)")
@@ -1750,7 +1815,7 @@ register_lazy_transport('ssh:', 'breezy.transport.remote',
                         'HintingSSHTransport')
 
 
-transport_server_registry = registry.Registry()
+transport_server_registry = registry.Registry[str, Callable]()
 transport_server_registry.register_lazy('bzr', 'breezy.bzr.smart.server',
                                         'serve_bzr', help="The Bazaar smart server protocol over TCP. (default port: 4155)")
 transport_server_registry.default_key = 'bzr'
