@@ -19,22 +19,15 @@
 This is a fairly thin wrapper on regular file IO.
 """
 
+import errno
 import os
 from stat import ST_MODE, S_ISDIR, S_IMODE
 import sys
 
-from ..lazy_import import lazy_import
-lazy_import(globals(), """
-import errno
-import shutil
-
-from breezy import (
-    atomicfile,
+from .. import (
     osutils,
     urlutils,
     )
-from breezy.transport import LateReadError
-""")
 
 from .. import transport
 
@@ -58,10 +51,10 @@ class LocalTransport(transport.Transport):
         if sys.platform == 'win32' and base == 'file:///':
             base = ''
             self._local_base = ''
-            super(LocalTransport, self).__init__(base)
+            super().__init__(base)
             return
 
-        super(LocalTransport, self).__init__(base)
+        super().__init__(base)
         self._local_base = urlutils.local_path_from_url(base)
         if self._local_base[-1] != '/':
             self._local_base = self._local_base + '/'
@@ -135,7 +128,7 @@ class LocalTransport(transport.Transport):
         """Return the local path portion from a given absolute path.
         """
         if abspath is None:
-            abspath = u'.'
+            abspath = '.'
 
         return urlutils.file_relpath(self.base, abspath)
 
@@ -152,10 +145,10 @@ class LocalTransport(transport.Transport):
             transport._file_streams[canonical_url].flush()
         try:
             path = self._abspath(relpath)
-            return osutils.open_file(path, 'rb')
-        except (IOError, OSError) as e:
+            return open(path, 'rb')
+        except OSError as e:
             if e.errno == errno.EISDIR:
-                return LateReadError(relpath)
+                return transport.LateReadError(relpath)
             self._translate_error(e, path)
 
     def put_file(self, relpath, f, mode=None):
@@ -166,13 +159,14 @@ class LocalTransport(transport.Transport):
         :param mode: The mode for the newly created file,
                      None means just use the default
         """
+        from ..atomicfile import AtomicFile
 
         path = relpath
         try:
             path = self._abspath(relpath)
             osutils.check_legal_path(path)
-            fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
-        except (IOError, OSError) as e:
+            fp = AtomicFile(path, 'wb', new_mode=mode)
+        except OSError as e:
             self._translate_error(e, path)
         try:
             length = self._pump(f, fp)
@@ -181,12 +175,13 @@ class LocalTransport(transport.Transport):
             fp.close()
         return length
 
-    def put_bytes(self, relpath, raw_bytes, mode=None):
+    def put_bytes(self, relpath: str, raw_bytes: bytes, mode=None):
         """Copy the string into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param raw_bytes:   String
         """
+        from ..atomicfile import AtomicFile
         if not isinstance(raw_bytes, bytes):
             raise TypeError(
                 'raw_bytes must be bytes, not %s' % type(raw_bytes))
@@ -194,11 +189,11 @@ class LocalTransport(transport.Transport):
         try:
             path = self._abspath(relpath)
             osutils.check_legal_path(path)
-            fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
-        except (IOError, OSError) as e:
+            fp = AtomicFile(path, 'wb', new_mode=mode)
+        except OSError as e:
             self._translate_error(e, path)
         try:
-            if bytes:
+            if raw_bytes:
                 fp.write(raw_bytes)
             fp.commit()
         finally:
@@ -227,7 +222,7 @@ class LocalTransport(transport.Transport):
             local_mode = mode
         try:
             fd = os.open(abspath, _put_non_atomic_flags, local_mode)
-        except (IOError, OSError) as e:
+        except OSError as e:
             # We couldn't create the file, maybe we need to create
             # the parent directory, and try again
             if (not create_parent_dir
@@ -241,7 +236,7 @@ class LocalTransport(transport.Transport):
             # file again
             try:
                 fd = os.open(abspath, _put_non_atomic_flags, local_mode)
-            except (IOError, OSError) as e:
+            except OSError as e:
                 self._translate_error(e, relpath)
         try:
             st = os.fstat(fd)
@@ -277,18 +272,18 @@ class LocalTransport(transport.Transport):
                                     create_parent_dir=create_parent_dir,
                                     dir_mode=dir_mode)
 
-    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+    def put_bytes_non_atomic(self, relpath: str, raw_bytes: bytes, mode=None,
                              create_parent_dir=False, dir_mode=None):
         def writer(fd):
-            if bytes:
-                os.write(fd, bytes)
+            if raw_bytes:
+                os.write(fd, raw_bytes)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
                                     create_parent_dir=create_parent_dir,
                                     dir_mode=dir_mode)
 
     def iter_files_recursive(self):
         """Iter the relative paths of files in the transports sub-tree."""
-        queue = list(self.list_dir(u'.'))
+        queue = list(self.list_dir('.'))
         while queue:
             relpath = queue.pop(0)
             st = self.stat(relpath)
@@ -307,12 +302,12 @@ class LocalTransport(transport.Transport):
             local_mode = mode
         try:
             os.mkdir(abspath, local_mode)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, abspath)
         if mode is not None:
             try:
                 osutils.chmod_if_possible(abspath, mode)
-            except (IOError, OSError) as e:
+            except OSError as e:
                 self._translate_error(e, abspath)
 
     def mkdir(self, relpath, mode=None):
@@ -323,8 +318,8 @@ class LocalTransport(transport.Transport):
         """See Transport.open_write_stream."""
         abspath = self._abspath(relpath)
         try:
-            handle = osutils.open_file(abspath, 'wb')
-        except (IOError, OSError) as e:
+            handle = open(abspath, 'wb')
+        except OSError as e:
             self._translate_error(e, abspath)
         handle.truncate()
         if mode is not None:
@@ -342,7 +337,7 @@ class LocalTransport(transport.Transport):
             local_mode = mode
         try:
             return file_abspath, os.open(file_abspath, _append_flags, local_mode)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, relpath)
 
     def _check_mode_and_size(self, file_abspath, fd, mode=None):
@@ -388,9 +383,10 @@ class LocalTransport(transport.Transport):
         """Copy the item at rel_from to the location at rel_to"""
         path_from = self._abspath(rel_from)
         path_to = self._abspath(rel_to)
+        import shutil
         try:
             shutil.copy(path_from, path_to)
-        except (IOError, OSError) as e:
+        except OSError as e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)
 
@@ -402,7 +398,7 @@ class LocalTransport(transport.Transport):
             # detect conflicting names on rename, and osutils.rename tries to
             # mask cross-platform differences there
             os.rename(path_from, path_to)
-        except (IOError, OSError) as e:
+        except OSError as e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)
 
@@ -414,7 +410,7 @@ class LocalTransport(transport.Transport):
         try:
             # this version will delete the destination if necessary
             osutils.rename(path_from, path_to)
-        except (IOError, OSError) as e:
+        except OSError as e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)
 
@@ -424,7 +420,7 @@ class LocalTransport(transport.Transport):
         try:
             path = self._abspath(relpath)
             os.remove(path)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, path)
 
     def external_url(self):
@@ -438,6 +434,7 @@ class LocalTransport(transport.Transport):
         :param relpaths: A list/generator of entries to be copied.
         """
         if isinstance(other, LocalTransport):
+            import shutil
             # Both from & to are on the local filesystem
             # Unfortunately, I can't think of anything faster than just
             # copying them across, one by one :(
@@ -451,12 +448,12 @@ class LocalTransport(transport.Transport):
                     shutil.copy(mypath, otherpath)
                     if mode is not None:
                         osutils.chmod_if_possible(otherpath, mode)
-                except (IOError, OSError) as e:
+                except OSError as e:
                     self._translate_error(e, path)
                 count += 1
             return count
         else:
-            return super(LocalTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
+            return super().copy_to(relpaths, other, mode=mode, pb=pb)
 
     def listable(self):
         """See Transport.listable."""
@@ -470,7 +467,7 @@ class LocalTransport(transport.Transport):
         path = self._abspath(relpath)
         try:
             entries = os.listdir(path)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, path)
         return [urlutils.escape(entry) for entry in entries]
 
@@ -481,7 +478,7 @@ class LocalTransport(transport.Transport):
         try:
             path = self._abspath(relpath)
             return os.lstat(path)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, path)
 
     def lock_read(self, relpath):
@@ -493,7 +490,7 @@ class LocalTransport(transport.Transport):
         try:
             path = self._abspath(relpath)
             return ReadLock(path)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, path)
 
     def lock_write(self, relpath):
@@ -511,37 +508,35 @@ class LocalTransport(transport.Transport):
         try:
             path = self._abspath(relpath)
             os.rmdir(path)
-        except (IOError, OSError) as e:
+        except OSError as e:
             self._translate_error(e, path)
 
-    if osutils.host_os_dereferences_symlinks():
-        def readlink(self, relpath):
-            """See Transport.readlink."""
-            try:
-                return osutils.readlink(self._abspath(relpath))
-            except (IOError, OSError) as e:
-                self._translate_error(e, relpath)
+    def symlink(self, source, link_name):
+        """See Transport.symlink."""
+        abs_link_dirpath = urlutils.dirname(self.abspath(link_name))
+        source_rel = urlutils.file_relpath(
+            abs_link_dirpath, self.abspath(source))
+
+        try:
+            os.symlink(source_rel, self._abspath(link_name))
+        except OSError as e:
+            self._translate_error(e, source_rel)
+
+    def readlink(self, relpath):
+        """See Transport.readlink."""
+        try:
+            return osutils.readlink(self._abspath(relpath))
+        except OSError as e:
+            self._translate_error(e, relpath)
 
     if osutils.hardlinks_good():
         def hardlink(self, source, link_name):
             """See Transport.link."""
             try:
                 os.link(self._abspath(source), self._abspath(link_name))
-            except (IOError, OSError) as e:
+            except OSError as e:
                 self._translate_error(e, source)
 
-    if getattr(os, 'symlink', None) is not None:
-        def symlink(self, source, link_name):
-            """See Transport.symlink."""
-            abs_link_dirpath = urlutils.dirname(self.abspath(link_name))
-            source_rel = urlutils.file_relpath(
-                abs_link_dirpath, self.abspath(source))
-
-            try:
-                os.symlink(source_rel, self._abspath(link_name))
-            except (IOError, OSError) as e:
-                self._translate_error(e, source_rel)
-
     def _can_roundtrip_unix_modebits(self):
         if sys.platform == 'win32':
             # anyone else?
