@@ -19,6 +19,7 @@
 import os
 import re
 import sys
+from typing import Tuple, Union
 
 from urllib import parse as urlparse
 
@@ -27,10 +28,7 @@ from . import (
     osutils,
     )
 
-from .lazy_import import lazy_import
-lazy_import(globals(), """
-from posixpath import split as _posix_split
-""")
+import posixpath
 
 
 
@@ -64,24 +62,27 @@ class InvalidRebaseURLs(errors.PathError):
 def basename(url, exclude_trailing_slash=True):
     """Return the last component of a URL.
 
-    :param url: The URL in question
-    :param exclude_trailing_slash: If the url looks like "path/to/foo/"
+    Args:
+      url: The URL in question
+      exclude_trailing_slash: If the url looks like "path/to/foo/"
         ignore the final slash and return 'foo' rather than ''
-    :return: Just the final component of the URL. This can return ''
-        if you don't exclude_trailing_slash, or if you are at the
-        root of the URL.
+    Returns:
+      Just the final component of the URL. This can return ''
+      if you don't exclude_trailing_slash, or if you are at the
+      root of the URL.
     """
     return split(url, exclude_trailing_slash=exclude_trailing_slash)[1]
 
 
-def dirname(url, exclude_trailing_slash=True):
+def dirname(url: str, exclude_trailing_slash: bool = True) -> str:
     """Return the parent directory of the given path.
 
-    :param url: Relative or absolute URL
-    :param exclude_trailing_slash: Remove a final slash
+    Args:
+      url: Relative or absolute URL
+      exclude_trailing_slash: Remove a final slash
         (treat http://host/foo/ as http://host/foo, but
         http://host/ stays http://host/)
-    :return: Everything in the URL except the last path chunk
+    Returns: Everything in the URL except the last path chunk
     """
     # TODO: jam 20060502 This was named dirname to be consistent
     #       with the os functions, but maybe "parent" would be better
@@ -94,12 +95,12 @@ unquote_to_bytes = urlparse.unquote_to_bytes
 unquote = urlparse.unquote
 
 
-def escape(relpath, safe='/~'):
+def escape(relpath: Union[bytes, str], safe: str = '/~') -> str:
     """Escape relpath to be a valid url."""
     return quote(relpath, safe=safe)
 
 
-def file_relpath(base, path):
+def file_relpath(base: str, path: str) -> str:
     """Compute just the relative sub-portion of a url
 
     This assumes that both paths are already fully specified file:// URLs.
@@ -139,7 +140,7 @@ def is_url(url):
     return _url_scheme_re.match(url) is not None
 
 
-def join(base, *args):
+def join(base: str, *args) -> str:
     """Create a URL by joining sections.
 
     This will normalize '..', assuming that paths are absolute
@@ -231,7 +232,7 @@ def _posix_local_path_to_url(path):
     """
     # importing directly from posixpath allows us to test this
     # on non-posix platforms
-    return 'file://' + escape(osutils._posix_abspath(path))
+    return 'file://' + escape(posixpath.abspath(path))
 
 
 def _win32_local_path_from_url(url):
@@ -262,7 +263,7 @@ def _win32_local_path_from_url(url):
             or win32_url[5] != '/'):
         raise InvalidURL(url, 'Win32 file urls start with'
                          ' file:///x:/, where x is a valid drive letter')
-    return win32_url[3].upper() + u':' + unescape(win32_url[5:])
+    return win32_url[3].upper() + ':' + unescape(win32_url[5:])
 
 
 def _win32_local_path_to_url(path):
@@ -332,8 +333,9 @@ def normalize_url(url):
     escaped (like +?&# etc), but may have unicode characters, etc
     which would not be valid in a real URL.
 
-    :param url: Either a hybrid URL or a local path
-    :return: A normalized URL which only includes 7-bit ASCII characters.
+    Args:
+      url: Either a hybrid URL or a local path
+    Returns: A normalized URL which only includes 7-bit ASCII characters.
     """
     scheme_end, path_start = _find_scheme_and_separator(url)
     if scheme_end is None:
@@ -424,14 +426,15 @@ def _win32_extract_drive_letter(url_base, path):
     return url_base, path
 
 
-def split(url, exclude_trailing_slash=True):
+def split(url: str, exclude_trailing_slash: bool = True) -> Tuple[str, str]:
     """Split a URL into its parent directory and a child directory.
 
-    :param url: A relative or absolute URL
-    :param exclude_trailing_slash: Strip off a final '/' if it is part
+    Args:
+      url: A relative or absolute URL
+      exclude_trailing_slash: Strip off a final '/' if it is part
         of the path (but not if it is part of the protocol specification)
 
-    :return: (parent_url, child_dir).  child_dir may be the empty string if
+    Returns: (parent_url, child_dir).  child_dir may be the empty string if
         we're at the root.
     """
     scheme_loc, first_path_slash = _find_scheme_and_separator(url)
@@ -442,7 +445,7 @@ def split(url, exclude_trailing_slash=True):
             # Relative path
             if exclude_trailing_slash and url.endswith('/'):
                 url = url[:-1]
-            return _posix_split(url)
+            return posixpath.split(url)
         else:
             # Scheme with no path
             return url, ''
@@ -460,15 +463,16 @@ def split(url, exclude_trailing_slash=True):
 
     if exclude_trailing_slash and len(path) > 1 and path.endswith('/'):
         path = path[:-1]
-    head, tail = _posix_split(path)
+    head, tail = posixpath.split(path)
     return url_base + head, tail
 
 
 def split_segment_parameters_raw(url):
     """Split the subsegment of the last segment of a URL.
 
-    :param url: A relative or absolute URL
-    :return: (url, subsegments)
+    Args:
+      url: A relative or absolute URL
+    Returns: (url, subsegments)
     """
     # GZ 2011-11-18: Dodgy removing the terminal slash like this, function
     #                operates on urls not url+segments, and Transport classes
@@ -485,8 +489,9 @@ def split_segment_parameters_raw(url):
 def split_segment_parameters(url):
     """Split the segment parameters of the last segment of a URL.
 
-    :param url: A relative or absolute URL
-    :return: (url, segment_parameters)
+    Args:
+      url: A relative or absolute URL
+    Returns: (url, segment_parameters)
     """
     (base_url, subsegments) = split_segment_parameters_raw(url)
     parameters = {}
@@ -506,8 +511,9 @@ def split_segment_parameters(url):
 def strip_segment_parameters(url):
     """Strip the segment parameters from a URL.
 
-    :param url: A relative or absolute URL
-    :return: url
+    Args:
+      url: A relative or absolute URL
+    Returns: url
     """
     base_url, subsegments = split_segment_parameters_raw(url)
     return base_url
@@ -538,8 +544,9 @@ def join_segment_parameters(url, parameters):
     The parameters of the last segment in the URL will be updated; if a
     parameter with the same key already exists it will be overwritten.
 
-    :param url: A URL, as string
-    :param parameters: Dictionary of parameters, keys and values as bytestrings
+    Args:
+      url: A URL, as string
+      parameters: Dictionary of parameters, keys and values as bytestrings
     """
     (base, existing_parameters) = split_segment_parameters(url)
     new_parameters = {}
@@ -625,7 +632,7 @@ def unescape(url):
             url.encode("ascii")
         except UnicodeError as e:
             raise InvalidURL(
-                url, 'URL was not a plain ASCII url: %s' % (e,))
+                url, 'URL was not a plain ASCII url: {}'.format(e))
     return urlparse.unquote(url)
 
 
@@ -634,8 +641,8 @@ _no_decode_chars = ';/?:@&=+$,#'
 _no_decode_ords = [ord(c) for c in _no_decode_chars]
 _no_decode_hex = (['%02x' % o for o in _no_decode_ords]
                   + ['%02X' % o for o in _no_decode_ords])
-_hex_display_map = dict(([('%02x' % o, bytes([o])) for o in range(256)]
-                         + [('%02X' % o, bytes([o])) for o in range(256)]))
+_hex_display_map = dict([('%02x' % o, bytes([o])) for o in range(256)]
+                         + [('%02X' % o, bytes([o])) for o in range(256)])
 # These entries get mapped to themselves
 _hex_display_map.update((hex, b'%' + hex.encode('ascii'))
                         for hex in _no_decode_hex)
@@ -665,10 +672,11 @@ def _unescape_segment_for_display(segment, encoding):
 
     Helper for unescape_for_display
 
-    :param url: A 7-bit ASCII URL
-    :param encoding: The final output encoding
+    Args:
+      url: A 7-bit ASCII URL
+      encoding: The final output encoding
 
-    :return: A unicode string which can be safely encoded into the
+    Returns: A unicode string which can be safely encoded into the
          specified encoding.
     """
     escaped_chunks = segment.split('%')
@@ -711,10 +719,11 @@ def unescape_for_display(url, encoding):
     Any sections of the URL which can't be represented in the encoding or
     need to stay as escapes are left alone.
 
-    :param url: A 7-bit ASCII URL
-    :param encoding: The final output encoding
+    Args:
+      url: A 7-bit ASCII URL
+      encoding: The final output encoding
 
-    :return: A unicode string which can be safely encoded into the
+    Returns: A unicode string which can be safely encoded into the
          specified encoding.
     """
     if encoding is None:
@@ -731,7 +740,7 @@ def unescape_for_display(url, encoding):
     res = url.split('/')
     for i in range(1, len(res)):
         res[i] = _unescape_segment_for_display(res[i], encoding)
-    return u'/'.join(res)
+    return '/'.join(res)
 
 
 def derive_to_location(from_location):
@@ -798,7 +807,7 @@ def determine_relative_path(from_path, to_path):
     return osutils.pathjoin(*segments)
 
 
-class URL(object):
+class URL:
     """Parsed URL."""
 
     def __init__(self, scheme, quoted_user, quoted_password, quoted_host,
@@ -830,7 +839,7 @@ class URL(object):
                 self.path == other.path)
 
     def __repr__(self):
-        return "<%s(%r, %r, %r, %r, %r, %r)>" % (
+        return "<{}({!r}, {!r}, {!r}, {!r}, {!r}, {!r})>".format(
             self.__class__.__name__,
             self.scheme, self.quoted_user, self.quoted_password,
             self.quoted_host, self.port, self.quoted_path)
@@ -839,7 +848,8 @@ class URL(object):
     def from_string(cls, url):
         """Create a URL object from a string.
 
-        :param url: URL as bytestring
+        Args:
+          url: URL as bytestring
         """
         # GZ 2017-06-09: Actually validate ascii-ness
         # pad.lv/1696545: For the moment, accept both native strings and
@@ -887,14 +897,14 @@ class URL(object):
             # Note that we don't put the password back even if we
             # have one so that it doesn't get accidentally
             # exposed.
-            netloc = '%s@%s' % (self.quoted_user, netloc)
+            netloc = '{}@{}'.format(self.quoted_user, netloc)
         if self.port is not None:
             netloc = '%s:%d' % (netloc, self.port)
         return urlparse.urlunparse(
             (self.scheme, netloc, self.quoted_path, None, None, None))
 
     @staticmethod
-    def _combine_paths(base_path, relpath):
+    def _combine_paths(base_path: str, relpath: str) -> str:
         """Transform a Transport-relative path to a remote absolute path.
 
         This does not handle substitution of ~ but does handle '..' and '.'
@@ -909,20 +919,12 @@ class URL(object):
             t._combine_paths('/home/sarah', '/etc')
                 => '/etc'
 
-        :param base_path: base path
-        :param relpath: relative url string for relative part of remote path.
-        :return: urlencoded string for final path.
+        Args:
+          base_path: base path
+          relpath: relative url string for relative part of remote path.
+        Returns: urlencoded string for final path.
         """
-        # pad.lv/1696545: For the moment, accept both native strings and
-        # unicode.
-        if isinstance(relpath, str):
-            pass
-        elif isinstance(relpath, str):
-            try:
-                relpath = relpath.encode()
-            except UnicodeEncodeError:
-                raise InvalidURL(relpath)
-        else:
+        if not isinstance(relpath, str):
             raise InvalidURL(relpath)
         relpath = _url_hex_escapes_re.sub(_unescape_safe_chars, relpath)
         if relpath.startswith('/'):
@@ -950,8 +952,9 @@ class URL(object):
     def clone(self, offset=None):
         """Return a new URL for a path relative to this URL.
 
-        :param offset: A relative path, already urlencoded
-        :return: `URL` instance
+        Args:
+          offset: A relative path, already urlencoded
+        Returns: `URL` instance
         """
         if offset is not None:
             relative = unescape(offset)
@@ -970,8 +973,9 @@ def parse_url(url):
     user, password, host and path should be quoted if they contain reserved
     chars.
 
-    :param url: an quoted url
-    :return: (scheme, user, password, host, port, path) tuple, all fields
+    Args:
+      url: an quoted url
+    Returns: (scheme, user, password, host, port, path) tuple, all fields
         are unquoted.
     """
     parsed_url = URL.from_string(url)
