@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+__docformat__ = "google"
+
 """WorkingTree object and friends.
 
 A WorkingTree represents the editable working copy of a branch.
@@ -31,30 +33,26 @@ import contextlib
 import errno
 import os
 import sys
+from typing import Optional, Tuple, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from .branch import Branch
+    from .revisiontree import RevisionTree
 
 import breezy
 
 from .lazy_import import lazy_import
 lazy_import(globals(), """
-import shutil
 import stat
 
 from breezy import (
-    conflicts as _mod_conflicts,
-    filters as _mod_filters,
-    merge,
-    revision as _mod_revision,
-    transform,
-    transport,
     views,
     )
-from breezy.bzr import (
-    generate_ids,
-    )
 """)
 
 from . import (
     errors,
+    revision as _mod_revision,
     )
 from .controldir import (
     ControlComponent,
@@ -70,6 +68,7 @@ from .i18n import gettext
 from . import mutabletree
 from .symbol_versioning import deprecated_method, deprecated_in
 from .trace import mutter, note
+from .transport import NoSuchFile
 
 
 class SettingFileIdUnsupported(errors.BzrError):
@@ -82,13 +81,21 @@ class ShelvingUnsupported(errors.BzrError):
     _fmt = "This format does not support shelving changes."
 
 
+class PointlessMerge(errors.BzrError):
+
+    _fmt = "Nothing to merge."
+
+
 class WorkingTree(mutabletree.MutableTree, ControlComponent):
     """Working copy tree.
 
-    :ivar basedir: The root of the tree on disk. This is a unicode path object
+    Attributes:
+      basedir: The root of the tree on disk. This is a unicode path object
         (as opposed to a URL).
     """
 
+    branch: "Branch"
+
     # override this to set the strategy for storing views
     def _make_views(self):
         return views.DisabledViews(self)
@@ -101,7 +108,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                  _controldir=None):
         """Construct a WorkingTree instance. This is not a public API.
 
-        :param branch: A branch to override probing for the branch.
+        Args:
+          branch: A branch to override probing for the branch.
         """
         self._format = _format
         self.controldir = _controldir
@@ -133,7 +141,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def is_control_filename(self, filename):
         """True if filename is the name of a control file in this tree.
 
-        :param filename: A filename within the tree. This is a relative path
+        Args:
+          filename: A filename within the tree. This is a relative path
             from the root of this tree.
 
         This is true IF and ONLY IF the filename is part of the meta data
@@ -142,7 +151,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         """
         return self.controldir.is_control_filename(filename)
 
-    branch = property(
+    branch = property(  # type: ignore
         fget=lambda self: self._branch,
         doc="""The branch this WorkingTree is connected to.
 
@@ -190,13 +199,14 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def get_config_stack(self):
         """Retrieve the config stack for this tree.
 
-        :return: A ``breezy.config.Stack``
+        Returns:
+          A ``breezy.config.Stack``
         """
         # For the moment, just provide the branch config stack.
         return self.branch.get_config_stack()
 
     @staticmethod
-    def open(path=None, _unsupported=False):
+    def open(path=None, _unsupported=False) -> "WorkingTree":
         """Open an existing working tree at path.
 
         """
@@ -206,7 +216,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         return control.open_workingtree(unsupported=_unsupported)
 
     @staticmethod
-    def open_containing(path=None):
+    def open_containing(path: Optional[str] = None) -> Tuple["WorkingTree", str]:
         """Open an existing working tree which has its root about path.
 
         This probes for a working tree at path and searches upwards from there.
@@ -216,7 +226,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         TODO: give this a new exception.
         If there is one, it is returned, along with the unused portion of path.
 
-        :return: The WorkingTree that contains 'path', and the rest of path
+        Returns:
+          The WorkingTree that contains 'path', and the rest of path
         """
         if path is None:
             path = osutils.getcwd()
@@ -234,7 +245,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         any number of files and that require they all be in the same tree.
         """
         if default_directory is None:
-            default_directory = u'.'
+            default_directory = '.'
         # recommended replacement for builtins.internal_tree_files
         if file_list is None or len(file_list) == 0:
             tree = WorkingTree.open_containing(default_directory)[0]
@@ -248,7 +259,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                     view_str = views.view_display_str(view_files)
                     note(gettext("Ignoring files outside view. View is %s") % view_str)
             return tree, file_list
-        if default_directory == u'.':
+        if default_directory == '.':
             seed = file_list[0]
         else:
             seed = default_directory
@@ -261,13 +272,18 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def safe_relpath_files(self, file_list, canonicalize=True, apply_view=True):
         """Convert file_list into a list of relpaths in tree.
 
-        :param self: A tree to operate on.
-        :param file_list: A list of user provided paths or None.
-        :param apply_view: if True and a view is set, apply it or check that
+        Args:
+          self: A tree to operate on.
+          file_list: A list of user provided paths or None.
+          apply_view: if True and a view is set, apply it or check that
             specified files are within it
-        :return: A list of relative paths.
-        :raises errors.PathNotChild: When a provided path is in a different self
-            than self.
+
+        Returns:
+          A list of relative paths.
+
+        Raises:
+          errors.PathNotChild: When a provided path is in a different self than
+             self
         """
         if file_list is None:
             return None
@@ -291,7 +307,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         return new_list
 
     @staticmethod
-    def open_downlevel(path=None):
+    def open_downlevel(path=None) -> "WorkingTree":
         """Open an unsupported working tree.
 
         Only intended for advanced situations like upgrading part of a controldir.
@@ -299,13 +315,13 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         return WorkingTree.open(path, _unsupported=True)
 
     def __repr__(self):
-        return "<%s of %s>" % (self.__class__.__name__,
+        return "<{} of {}>".format(self.__class__.__name__,
                                getattr(self, 'basedir', None))
 
     def abspath(self, filename):
         return osutils.pathjoin(self.basedir, filename)
 
-    def basis_tree(self):
+    def basis_tree(self) -> "RevisionTree":
         """Return RevisionTree for the current last revision.
 
         If the left most parent is a ghost then the returned tree will be an
@@ -359,16 +375,21 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         abspath = self.abspath(path)
         try:
             file_obj = open(abspath, 'rb')
-        except EnvironmentError as e:
+        except OSError as e:
             if e.errno == errno.ENOENT:
-                raise errors.NoSuchFile(path)
+                raise NoSuchFile(path)
             raise
         stat_value = _fstat(file_obj.fileno())
         if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(path)
             if filters:
-                file_obj, size = _mod_filters.filtered_input_file(
-                    file_obj, filters)
+                from . import filters as _mod_filters
+                orig_file_obj = file_obj
+                try:
+                    file_obj, size = _mod_filters.filtered_input_file(
+                        file_obj, filters)
+                finally:
+                    orig_file_obj.close()
                 stat_value = _mod_filters.FilteredStat(
                     stat_value, st_size=size)
         return (file_obj, stat_value)
@@ -388,14 +409,14 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         This implementation reads the pending merges list and last_revision
         value and uses that to decide what the parents list should be.
         """
-        last_rev = _mod_revision.ensure_null(self._last_revision())
+        last_rev = self._last_revision()
         if _mod_revision.NULL_REVISION == last_rev:
             parents = []
         else:
             parents = [last_rev]
         try:
             merges_bytes = self._transport.get_bytes('pending-merges')
-        except errors.NoSuchFile:
+        except NoSuchFile:
             pass
         else:
             for l in osutils.split_lines(merges_bytes):
@@ -424,10 +445,11 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
 
     def copy_content_into(self, tree, revision_id=None):
         """Copy the current content and user files of this tree into tree."""
+        from .merge import transform_tree
         with self.lock_read():
             tree.set_root_id(self.path2id(''))
             if revision_id is None:
-                merge.transform_tree(tree, self)
+                transform_tree(tree, self)
             else:
                 # TODO now merge from tree.last_revision to revision (to
                 # preserve user local changes)
@@ -437,7 +459,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                     other_tree = self.branch.repository.revision_tree(
                         revision_id)
 
-                merge.transform_tree(tree, other_tree)
+                transform_tree(tree, other_tree)
                 if revision_id == _mod_revision.NULL_REVISION:
                     new_parents = []
                 else:
@@ -466,7 +488,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                         kinds[pos] = osutils.file_kind(fullpath)
                     except OSError as e:
                         if e.errno == errno.ENOENT:
-                            raise errors.NoSuchFile(fullpath)
+                            raise NoSuchFile(fullpath)
 
     def add_parent_tree_id(self, revision_id, allow_leftmost_as_ghost=False):
         """Add revision_id as a parent.
@@ -474,10 +496,11 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         This is equivalent to retrieving the current list of parent ids
         and setting the list to its value plus revision_id.
 
-        :param revision_id: The revision id to add to the parent list. It may
+        Args:
+          revision_id: The revision id to add to the parent list. It may
             be a ghost revision as long as its not the first parent to be
             added, or the allow_leftmost_as_ghost parameter is set True.
-        :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
+          allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         with self.lock_write():
             parents = self.get_parent_ids() + [revision_id]
@@ -493,9 +516,10 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         simpler to use that api. If you have the parent already available, using
         this api is preferred.
 
-        :param parent_tuple: The (revision id, tree) to add to the parent list.
+        Args:
+          parent_tuple: The (revision id, tree) to add to the parent list.
             If the revision_id is a ghost, pass None for the tree.
-        :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
+          allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         with self.lock_tree_write():
             parent_ids = self.get_parent_ids() + [parent_tuple[0]]
@@ -520,39 +544,9 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
             if updated:
                 self.set_parent_ids(parents, allow_leftmost_as_ghost=True)
 
-    def path_content_summary(self, path, _lstat=os.lstat,
-                             _mapper=osutils.file_kind_from_stat_mode):
+    def path_content_summary(self, path):
         """See Tree.path_content_summary."""
-        abspath = self.abspath(path)
-        try:
-            stat_result = _lstat(abspath)
-        except OSError as e:
-            if getattr(e, 'errno', None) == errno.ENOENT:
-                # no file.
-                return ('missing', None, None, None)
-            # propagate other errors
-            raise
-        kind = _mapper(stat_result.st_mode)
-        if kind == 'file':
-            return self._file_content_summary(path, stat_result)
-        elif kind == 'directory':
-            # perhaps it looks like a plain directory, but it's really a
-            # reference.
-            if self._directory_is_tree_reference(path):
-                kind = 'tree-reference'
-            return kind, None, None, None
-        elif kind == 'symlink':
-            target = osutils.readlink(abspath)
-            return ('symlink', None, None, target)
-        else:
-            return (kind, None, None, None)
-
-    def _file_content_summary(self, path, stat_result):
-        size = stat_result.st_size
-        executable = self._is_executable_from_path_and_stat(path, stat_result)
-        # try for a stat cache lookup
-        return ('file', size, executable, self._sha_from_stat(
-            path, stat_result))
+        raise NotImplementedError(self.path_content_summary)
 
     def _check_parents_for_ghosts(self, revision_ids, allow_leftmost_as_ghost):
         """Common ghost checking functionality from set_parent_*.
@@ -599,7 +593,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         set_parent_trees rather than set_parent_ids. set_parent_ids is however
         an easier API to use.
 
-        :param revision_ids: The revision_ids to set as the parent ids of this
+        Args:
+          revision_ids: The revision_ids to set as the parent ids of this
             working tree. Any of these may be ghosts.
         """
         with self.lock_tree_write():
@@ -633,8 +628,9 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
 
         The default implementation assumes no stat cache is present.
 
-        :param path: The path.
-        :param stat_result: The stat result being looked up.
+        Args:
+          path: The path.
+          stat_result: The stat result being looked up.
         """
         return None
 
@@ -642,8 +638,9 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                           merge_type=None, force=False):
         """Merge from a branch into this working tree.
 
-        :param branch: The branch to merge from.
-        :param to_revision: If non-None, the merge will merge to to_revision,
+        Args:
+          branch: The branch to merge from.
+          to_revision: If non-None, the merge will merge to to_revision,
             but not beyond it. to_revision does not need to be in the history
             of the branch when it is supplied. If None, to_revision defaults to
             branch.last_revision().
@@ -655,7 +652,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
             if not force and self.has_changes():
                 raise errors.UncommittedChanges(self)
             if to_revision is None:
-                to_revision = _mod_revision.ensure_null(branch.last_revision())
+                to_revision = branch.last_revision()
             merger.other_rev_id = to_revision
             if _mod_revision.is_null(merger.other_rev_id):
                 raise errors.NoCommits(branch)
@@ -669,7 +666,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
             else:
                 merger.set_base_revision(from_revision, branch)
             if merger.base_rev_id == merger.other_rev_id:
-                raise errors.PointlessMerge
+                raise PointlessMerge()
             merger.backup_files = False
             if merge_type is None:
                 merger.merge_type = Merge3Merger
@@ -694,18 +691,11 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         """
         raise NotImplementedError(self.merge_modified)
 
-    def mkdir(self, path, file_id=None):
+    def mkdir(self, path):
         """See MutableTree.mkdir()."""
-        if file_id is None:
-            if self.supports_setting_file_ids():
-                file_id = generate_ids.gen_file_id(os.path.basename(path))
-        else:
-            if not self.supports_setting_file_ids():
-                raise SettingFileIdUnsupported()
         with self.lock_write():
             os.mkdir(self.abspath(path))
-            self.add(path, file_id, 'directory')
-            return file_id
+            self.add([path], ['directory'])
 
     def get_symlink_target(self, path):
         abspath = self.abspath(path)
@@ -713,7 +703,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
             return osutils.readlink(abspath)
         except OSError as e:
             if getattr(e, 'errno', None) == errno.ENOENT:
-                raise errors.NoSuchFile(path)
+                raise NoSuchFile(path)
             raise
 
     def subsume(self, other_tree):
@@ -744,9 +734,10 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         This does not include files that have been deleted in this
         tree. Skips the control directory.
 
-        :param include_root: if True, return an entry for the root
-        :param from_dir: start from this directory or None for the root
-        :param recursive: whether to recurse into subdirectories or not
+        Args:
+          include_root: if True, return an entry for the root
+          from_dir: start from this directory or None for the root
+          recursive: whether to recurse into subdirectories or not
         """
         raise NotImplementedError(self.list_files)
 
@@ -792,9 +783,11 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         This default implementation just copies the file, then
         adds the target.
 
-        :param from_rel: From location (relative to tree root)
-        :param to_rel: Target location (relative to tree root)
+        Args:
+          from_rel: From location (relative to tree root)
+          to_rel: Target location (relative to tree root)
         """
+        import shutil
         shutil.copyfile(self.abspath(from_rel), self.abspath(to_rel))
         self.add(to_rel)
 
@@ -816,18 +809,22 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         When a path is unversioned, all of its children are automatically
         unversioned.
 
-        :param paths: The paths to stop versioning.
-        :raises NoSuchFile: if any path is not currently versioned.
+        Args:
+          paths: The paths to stop versioning.
+
+        Raises:
+          NoSuchFile: if any path is not currently versioned.
         """
         raise NotImplementedError(self.unversion)
 
     def pull(self, source, overwrite=False, stop_revision=None,
              change_reporter=None, possible_transports=None, local=False,
              show_base=False, tag_selector=None):
+        from .merge import merge_inner
         with self.lock_write(), source.lock_read():
             old_revision_info = self.branch.last_revision_info()
             basis_tree = self.basis_tree()
-            count = self.branch.pull(source, overwrite, stop_revision,
+            count = self.branch.pull(source, overwrite=overwrite, stop_revision=stop_revision,
                                      possible_transports=possible_transports,
                                      local=local, tag_selector=tag_selector)
             new_revision_info = self.branch.last_revision_info()
@@ -840,7 +837,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                         basis_tree = repository.revision_tree(basis_id)
                 with basis_tree.lock_read():
                     new_basis_tree = self.branch.basis_tree()
-                    merge.merge_inner(
+                    merge_inner(
                         self.branch,
                         new_basis_tree,
                         basis_tree,
@@ -937,7 +934,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def _last_revision(self):
         """helper for get_parent_ids."""
         with self.lock_read():
-            return _mod_revision.ensure_null(self.branch.last_revision())
+            return self.branch.last_revision()
 
     def is_locked(self):
         """Check if this tree is locked."""
@@ -948,21 +945,21 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
 
         This also locks the branch, and can be unlocked via self.unlock().
 
-        :return: A breezy.lock.LogicalLockResult.
+        Returns: A breezy.lock.LogicalLockResult.
         """
         raise NotImplementedError(self.lock_read)
 
     def lock_tree_write(self):
         """See MutableTree.lock_tree_write, and WorkingTree.unlock.
 
-        :return: A breezy.lock.LogicalLockResult.
+        return: A breezy.lock.LogicalLockResult.
         """
         raise NotImplementedError(self.lock_tree_write)
 
     def lock_write(self):
         """See MutableTree.lock_write, and WorkingTree.unlock.
 
-        :return: A breezy.lock.LogicalLockResult.
+        Returns: A breezy.lock.LogicalLockResult.
         """
         raise NotImplementedError(self.lock_write)
 
@@ -1004,6 +1001,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def revert(self, filenames=None, old_tree=None, backups=True,
                pb=None, report_changes=False):
         from .conflicts import resolve
+        from .transform import revert
         with contextlib.ExitStack() as exit_stack:
             exit_stack.enter_context(self.lock_tree_write())
             if old_tree is None:
@@ -1012,8 +1010,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
                 old_tree = basis_tree
             else:
                 basis_tree = None
-            conflicts = transform.revert(self, old_tree, filenames, backups, pb,
-                                         report_changes)
+            conflicts = revert(self, old_tree, filenames, backups, pb,
+                               report_changes)
             if filenames is None and len(self.get_parent_ids()) > 1:
                 parent_trees = []
                 last_revision = self.last_revision()
@@ -1036,7 +1034,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         """Restore uncommitted changes from the branch into the tree."""
         raise NotImplementedError(self.restore_uncommitted)
 
-    def revision_tree(self, revision_id):
+    def revision_tree(self, revision_id) -> "RevisionTree":
         """See Tree.revision_tree.
 
         For trees that can be obtained from the working tree, this
@@ -1058,7 +1056,8 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def _set_root_id(self, file_id):
         """Set the root id for this tree, in a format specific manner.
 
-        :param file_id: The file id to assign to the root. It must not be
+        Args:
+          file_id: The file id to assign to the root. It must not be
             present in the current inventory or an error will occur. It must
             not be None, but rather a valid file id.
         """
@@ -1102,9 +1101,10 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
           basis.
         - Do a 'normal' merge of the old branch basis if it is relevant.
 
-        :param revision: The target revision to update to. Must be in the
+        Args:
+          revision: The target revision to update to. Must be in the
             revision history.
-        :param old_tip: If branch.update() has already been run, the value it
+          old_tip: If branch.update() has already been run, the value it
             returned (old tip of the branch or None). _marker is used
             otherwise.
         """
@@ -1135,7 +1135,6 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         """
         raise NotImplementedError(self.walkdirs)
 
-    @deprecated_method(deprecated_in((3, 0, 1)))
     def auto_resolve(self):
         """Automatically resolve text conflicts according to contents.
 
@@ -1144,7 +1143,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         into files that have text conflicts.  The corresponding .THIS .BASE and
         .OTHER files are deleted, as per 'resolve'.
 
-        :return: a tuple of lists: (un_resolved, resolved).
+        Returns: a tuple of lists: (un_resolved, resolved).
         """
         with self.lock_tree_write():
             un_resolved = []
@@ -1167,7 +1166,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         corruption after actions have occurred. The default implementation is a
         just a no-op.
 
-        :return: None. An exception should be raised if there is an error.
+        Returns: None. An exception should be raised if there is an error.
         """
         return
 
@@ -1186,8 +1185,7 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def _get_rules_searcher(self, default_searcher):
         """See Tree._get_rules_searcher."""
         if self._rules_searcher is None:
-            self._rules_searcher = super(WorkingTree,
-                                         self)._get_rules_searcher(default_searcher)
+            self._rules_searcher = super()._get_rules_searcher(default_searcher)
         return self._rules_searcher
 
     def get_shelf_manager(self):
@@ -1197,14 +1195,17 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
     def get_canonical_paths(self, paths):
         """Like get_canonical_path() but works on multiple items.
 
-        :param paths: A sequence of paths relative to the root of the tree.
-        :return: A list of paths, with each item the corresponding input path
-            adjusted to account for existing elements that match case
-            insensitively.
+        Args:
+          paths: A sequence of paths relative to the root of the tree.
+
+        Returns:
+          A list of paths, with each item the corresponding input path
+          adjusted to account for existing elements that match case
+          insensitively.
         """
         with self.lock_read():
             for path in paths:
-                yield path
+                yield path.strip('/')
 
     def get_canonical_path(self, path):
         """Returns the first item in the tree that matches a path.
@@ -1223,9 +1224,12 @@ class WorkingTree(mutabletree.MutableTree, ControlComponent):
         If you need to resolve many names from the same tree, you should
         use get_canonical_paths() to avoid O(N) behaviour.
 
-        :param path: A paths relative to the root of the tree.
-        :return: The input path adjusted to account for existing elements
-        that match case insensitively.
+        Args:
+          path: A paths relative to the root of the tree.
+
+        Returns:
+          The input path adjusted to account for existing elements
+          that match case insensitively.
         """
         with self.lock_read():
             return next(self.get_canonical_paths([path]))
@@ -1244,7 +1248,7 @@ class WorkingTreeFormatRegistry(ControlComponentFormatRegistry):
     """Registry for working tree formats."""
 
     def __init__(self, other_registry=None):
-        super(WorkingTreeFormatRegistry, self).__init__(other_registry)
+        super().__init__(other_registry)
         self._default_format = None
         self._default_format_key = None
 
@@ -1298,7 +1302,7 @@ class WorkingTreeFormat(ControlComponentFormat):
     missing_parent_conflicts = False
     """If this format supports missing parent conflicts."""
 
-    supports_versioned_directories = None
+    supports_versioned_directories: bool
 
     supports_merge_modified = True
     """If this format supports storing merge modified hashes."""
@@ -1313,22 +1317,23 @@ class WorkingTreeFormat(ControlComponentFormat):
 
     supports_righthand_parent_id_as_ghost = True
 
-    ignore_filename = None
+    ignore_filename: Optional[str] = None
     """Name of file with ignore patterns, if any. """
 
     def initialize(self, controldir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """Initialize a new working tree in controldir.
 
-        :param controldir: ControlDir to initialize the working tree in.
-        :param revision_id: allows creating a working tree at a different
+        Args:
+          controldir: ControlDir to initialize the working tree in.
+          revision_id: allows creating a working tree at a different
             revision than the branch is at.
-        :param from_branch: Branch to checkout
-        :param accelerator_tree: A tree which can be used for retrieving file
+          from_branch: Branch to checkout
+          accelerator_tree: A tree which can be used for retrieving file
             contents more quickly than the revision tree, i.e. a workingtree.
             The revision tree will be used for cases where accelerator_tree's
             content is different.
-        :param hardlink: If true, hard-link files from accelerator_tree,
+          hardlink: If true, hard-link files from accelerator_tree,
             where possible.
         """
         raise NotImplementedError(self.initialize)
