@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf8 -*-
 # $Id: rst2pdf.py 5560 2008-05-20 13:00:31Z milde $
 
 # rst2pdf.py
@@ -62,7 +61,7 @@ sys.path.append("/usr/share/rubber")
 try:
     import rubber.cmdline
     import rubber.cmd_pipe
-except ImportError:
+except ModuleNotFoundError:
     print("Cannot find the rubber modules, rubber not installed correctly.")
     sys.exit(1)
 
